@@ -6,6 +6,7 @@
  */
 
 #include <fcntl.h>
+#include <glob.h>
 
 #include "amd_memory.h"
 #include "amd_ip_blocks.h"
@@ -800,6 +801,7 @@ int setup_amdgpu_ip_blocks(uint32_t major, uint32_t minor, struct amdgpu_gpu_inf
 		identify_chip(GFX1150);
 		identify_chip(GFX1151);
 		identify_chip(GFX1152);
+		identify_chip(GFX1153);
 		break;
 	case FAMILY_GFX1200:
 		identify_chip(GFX1200);
@@ -809,10 +811,9 @@ int setup_amdgpu_ip_blocks(uint32_t major, uint32_t minor, struct amdgpu_gpu_inf
 		igt_info("amdgpu: unknown (family_id, chip_external_rev): (%u, %u)\n",
 			 amdinfo->family_id, amdinfo->chip_external_rev);
 		return -1;
-	} else {
-		igt_info("amdgpu: %s (family_id, chip_external_rev): (%u, %u)\n",
-				info->name, amdinfo->family_id, amdinfo->chip_external_rev);
 	}
+	igt_info("amdgpu: %s (family_id, chip_external_rev): (%u, %u)\n",
+				info->name, amdinfo->family_id, amdinfo->chip_external_rev);
 
 	if (info->family >= CHIP_GFX1100) {
 		info->chip_class = GFX11;
@@ -1004,45 +1005,157 @@ asic_rings_readness(amdgpu_device_handle device_handle, uint32_t mask,
  */
 
 bool
-is_reset_enable(enum amd_ip_block_type ip_type, uint32_t reset_type)
+is_reset_enable(enum amd_ip_block_type ip_type, uint32_t reset_type, const struct pci_addr *pci)
 {
-        char cmd[256];
-        FILE *fp, *fp2;
-        char buffer[100];
-        bool enable = false;
-        char reset_mask[100];
+	const struct reset_arr {
+		const char *name;
+		unsigned int reset;
+	} reset_arr[] = {
+		{"full",	AMDGPU_RESET_TYPE_FULL		},
+		{"soft",	AMDGPU_RESET_TYPE_SOFT_RESET},
+		{"queue",	AMDGPU_RESET_TYPE_PER_QUEUE },
+		{"pipe",	AMDGPU_RESET_TYPE_PER_PIPE },
+		{NULL, 0}
+	};
 
-        if(ip_type == AMD_IP_GFX)
-                snprintf(reset_mask, sizeof(reset_mask) - 1, "gfx_reset_mask");
-        else if (ip_type == AMD_IP_COMPUTE)
-                snprintf(reset_mask, sizeof(reset_mask) - 1, "compute_reset_mask");
-        else
-                snprintf(reset_mask, sizeof(reset_mask) - 1, "sdma_reset_mask");
+	bool enable = false;
+	char cmd[256];
+	FILE *fp;
+	char buffer[128];
 
-        snprintf(cmd, sizeof(cmd) - 1, "sudo cat /sys/kernel/debug/dri/0/name |grep -oP '(?<=dev=)[0-9:.]+'");
-        fp = popen(cmd, "r");
-        if (fp == NULL)
-                return false;
+	char *token, *buf;
+	char reset_mask[32];
+	uint32_t mask = 0;
+	int i;
 
-        if (fgets(buffer, 13, fp) != NULL) {
-                snprintf(cmd,sizeof(cmd) - 1,"sudo cat /sys/bus/pci/devices/%s/%s | grep -oP '%s'",
-                        buffer,reset_mask,
-			reset_type & AMDGPU_RESET_TYPE_FULL ? "full":
-			reset_type & AMDGPU_RESET_TYPE_SOFT_RESET ? "soft":
-			reset_type & AMDGPU_RESET_TYPE_PER_QUEUE ? "queue": "pipe");
+	if (ip_type == AMD_IP_GFX)
+		snprintf(reset_mask, sizeof(reset_mask) - 1, "gfx_reset_mask");
+	else if (ip_type == AMD_IP_COMPUTE)
+		snprintf(reset_mask, sizeof(reset_mask) - 1, "compute_reset_mask");
+	else
+		snprintf(reset_mask, sizeof(reset_mask) - 1, "sdma_reset_mask");
 
-                fp2 = popen(cmd, "r");
-                if (fp2 == NULL) {
-                        pclose(fp);
-                        return false;
-                }
+	snprintf(cmd, sizeof(cmd) - 1, "sudo cat /sys/bus/pci/devices/%04x:%02x:%02x.%01x/%s",
+			pci->domain, pci->bus, pci->device, pci->function, reset_mask);
 
-                if (fgets(buffer, 13, fp2) != NULL) {
-                        enable = true;
-                }
-                pclose(fp2);
-        }
-        pclose(fp);
+	fp = popen(cmd, "r");
+	if (fp == NULL) {
+		igt_kmsg("***FAILURE popen %s LINE %d FILE %s\n", cmd, __LINE__, __FILE__);
+		return false;
+	}
 
-        return enable;
+	buf = fgets(buffer, sizeof(buffer)-1, fp);
+
+	if (buf != NULL) {
+		token = strtok(buf, " \n");
+		while (token != NULL) {
+			for (i = 0; reset_arr[i].name != NULL; i++) {
+				if (reset_type == reset_arr[i].reset &&
+					strcmp(token, reset_arr[i].name) == 0) {
+					mask |= reset_arr[i].reset;
+					break;
+				}
+			}
+			token = strtok(NULL, " \n");
+		}
+	} else {
+		igt_kmsg("***FAILURE fgets %s LINE %d FILE %s\n",
+				buffer, __LINE__, __FILE__);
+	}
+	if (mask == reset_type)
+		enable = true;
+	else
+		igt_kmsg("***FAILURE mask found 0x%x requested 0x%x operation is not supported LINE %d FILE %s\n",
+				mask, reset_type, __LINE__, __FILE__);
+
+	pclose(fp);
+
+	return enable;
+}
+
+/**
+ * get_pci_addr_from_fd - Extracts the PCI device address from a file descriptor.
+ * @fd: The file descriptor to extract the address from.
+ * @pci: Pointer to a pci_addr struct to store the extracted address.
+ *
+ * Returns 0 on success, or a negative error code on failure.
+ */
+int get_pci_addr_from_fd(int fd, struct pci_addr *pci)
+{
+	char path[80];
+	struct stat st;
+	char link[20], pci_path[256];
+	char *buf;
+	int len, sysfs;
+	int ret;
+
+	// Check if the file descriptor is a character device and can be accessed
+	if (fstat(fd, &st) < 0 || !S_ISCHR(st.st_mode))
+		return -1;
+
+	snprintf(path, sizeof(path), "/sys/dev/char/%d:%d",
+			major(st.st_rdev), minor(st.st_rdev));
+
+	// Check if the sysfs path exists
+	if (access(path, F_OK) < 0)
+		return -1;
+
+	// Open the sysfs directory
+	sysfs = open(path, O_RDONLY);
+	if (sysfs < 0)
+		return -1;
+
+	// Read the "device" link from the sysfs directory
+	snprintf(link, sizeof(link), "device");
+	len = readlinkat(sysfs, link, pci_path, sizeof(pci_path) - 1);
+	if (len == -1) {
+		close(sysfs);
+		return -ENOENT;
+	}
+	close(sysfs);
+ // Null-terminate the extracted path
+	pci_path[len] = '\0';
+
+	// Find the last occurrence of '/' in the extracted path
+	buf = strrchr(pci_path, '/');
+	if (!buf)
+		return -ENOENT;
+
+	// Extract the PCI device address from the path using sscanf
+	ret = sscanf(buf, "/%4x:%2x:%2x.%2x", &pci->domain, &pci->bus,
+			&pci->device, &pci->function);
+
+	if (ret != 4) {
+		igt_info("error %s Unable to extract PCI device address from '%s\n",
+				__func__, buf);
+		return -ENOENT;
+	}
+
+	return 0;
+}
+
+/*
+ * Function to check if page queue files exist for a given IP block type and PCI address
+ */
+bool is_support_page_queue(enum amd_ip_block_type ip_type, const struct pci_addr *pci)
+{
+	glob_t glob_result;
+	int ret;
+	char search_pattern[1024];
+
+	/* If the IP type is not SDMA, return false */
+	if (ip_type != AMD_IP_DMA)
+		return false;
+
+	/* Construct the search pattern for the page queue files */
+	snprintf(search_pattern, sizeof(search_pattern) - 1, "/sys/kernel/debug/dri/%04x:%02x:%02x.%01x/amdgpu_ring_page*",
+		pci->domain, pci->bus, pci->device, pci->function);
+
+	/* Use glob to find files matching the pattern */
+	ret = glob(search_pattern, GLOB_NOSORT, NULL, &glob_result);
+	/* Free the memory allocated by glob */
+	globfree(&glob_result);
+
+	/* Return true if files matching the pattern were found, otherwise return false */
+	return (ret == 0 && glob_result.gl_pathc > 0);
 }
