@@ -38,6 +38,8 @@
 
 #include "igt_core.h"
 
+int __get_drm_device_name(int fd, char *name, int name_size);
+
 /*
  * NOTE: Theser are _only_ for testcases exercising driver specific rendering
  * ioctls and uapi (and a bunch of historical reasons). And KMS testcase should
@@ -72,6 +74,8 @@ enum intel_driver {
 
 void __set_forced_driver(const char *name);
 
+unsigned int drm_get_chipset(int fd);
+
 /**
  * ARRAY_SIZE:
  * @arr: static array
@@ -87,7 +91,8 @@ void __set_forced_driver(const char *name);
  *
  * Macro to align a value @v to a specified unit @a.
  */
-#define ALIGN(v, a) (((v) + (a)-1) & ~((a)-1))
+#define ALIGN(v, a) ALIGN_MASK(v, (typeof(v))(a) - 1)
+#define ALIGN_MASK(v, mask) (((v) + (mask)) & ~(mask))
 
 /**
  * ALIGN_DOWN:
@@ -98,17 +103,32 @@ void __set_forced_driver(const char *name);
  */
 #define ALIGN_DOWN(x, a)	ALIGN((x) - ((a) - 1), (a))
 
+/**
+ * IS_ALIGNED:
+ * @v: value to check
+ * @a: alignment unit in bytes
+ *
+ * Macro to check if value @v is aligned to @a
+ */
+#define IS_ALIGNED(v, a)	(((v) & ((typeof(v))(a) - 1)) == 0)
+
+int __drm_open_device(const char *name, unsigned int chipset);
 void drm_load_module(unsigned int chipset);
+int drm_open_driver_another(int idx, int chipset);
 int drm_open_driver(int chipset);
 int drm_open_driver_master(int chipset);
 int drm_open_driver_render(int chipset);
 int __drm_open_driver_another(int idx, int chipset);
 int __drm_open_driver(int chipset);
 int __drm_open_driver_render(int chipset);
+int __drm_close_driver(int fd);
 int drm_close_driver(int fd);
 
 int drm_reopen_driver(int fd);
 
+int drm_prepare_filtered_multigpu(int chipset);
+int drm_open_filtered_card(int idx);
+
 void igt_require_amdgpu(int fd);
 void igt_require_intel(int fd);
 void igt_require_i915(int fd);
