@@ -207,6 +207,8 @@ struct intel_perf_logical_counter {
 	const char *name;
 	const char *symbol_name;
 	const char *desc;
+	const char *group;
+	bool (*availability)(const struct intel_perf *perf);
 	intel_perf_logical_counter_storage_t storage;
 	intel_perf_logical_counter_type_t type;
 	intel_perf_logical_counter_unit_t unit;
@@ -256,13 +258,13 @@ struct intel_perf_metric_set {
 	int c_offset;
 	int perfcnt_offset;
 
-	struct intel_perf_register_prog *b_counter_regs;
+	const struct intel_perf_register_prog *b_counter_regs;
 	uint32_t n_b_counter_regs;
 
-	struct intel_perf_register_prog *mux_regs;
+	const struct intel_perf_register_prog *mux_regs;
 	uint32_t n_mux_regs;
 
-	struct intel_perf_register_prog *flex_regs;
+	const struct intel_perf_register_prog *flex_regs;
 	uint32_t n_flex_regs;
 
 	struct igt_list_head link;
@@ -316,7 +318,7 @@ intel_perf_devinfo_eu_available(const struct intel_perf_devinfo *devinfo,
 	return (devinfo->eu_masks[subslice_offset + eu / 8] & (1U << eu % 8)) != 0;
 }
 
-struct intel_perf *intel_perf_for_fd(int drm_fd);
+struct intel_perf *intel_perf_for_fd(int drm_fd, int gt);
 struct intel_perf *intel_perf_for_devinfo(uint32_t device_id,
 					  uint32_t revision,
 					  uint64_t timestamp_frequency,
