@@ -24,8 +24,8 @@
 
 #include <time.h>
 #include <signal.h>
+#include <poll.h>
 #include <pthread.h>
-#include <sys/poll.h>
 #include <sys/timerfd.h>
 
 #include <i915_drm.h>
@@ -64,8 +64,6 @@
 
 #define ENGINE_MASK  (I915_EXEC_RING_MASK | I915_EXEC_BSD_MASK)
 
-#define MI_ARB_CHK (0x5 << 23)
-
 static const int BATCH_SIZE = 4096;
 static const int LOOP_START_OFFSET = 64;
 
@@ -219,7 +217,7 @@ emit_recursive_batch(igt_spin_t *spin,
 	} else if (opts->flags & IGT_SPIN_POLL_RUN) {
 		igt_assert(!opts->dependency);
 
-		if (gen == 4 || gen == 5) {
+		if (gem_store_dword_needs_secure(fd)) {
 			execbuf->flags |= I915_EXEC_SECURE;
 			igt_require(__igt_device_set_master(fd) == 0);
 		}
@@ -286,7 +284,7 @@ emit_recursive_batch(igt_spin_t *spin,
 
 	/* Allow ourselves to be preempted */
 	if (!(opts->flags & IGT_SPIN_NO_PREEMPTION))
-		*cs++ = MI_ARB_CHK;
+		*cs++ = MI_ARB_CHECK;
 	if (opts->flags & IGT_SPIN_INVALID_CS) {
 		igt_assert(opts->ctx);
 		if (!gem_engine_has_cmdparser(fd, &opts->ctx->cfg, opts->engine))
