@@ -593,14 +593,6 @@ static int igt_always_unload_audio_driver(char **who)
 	return 0;
 }
 
-struct module_ref {
-	char *name;
-	unsigned long mem;
-	unsigned int ref_count;
-	unsigned int num_required;
-	unsigned int *required_by;
-};
-
 int igt_audio_driver_unload(char **who)
 {
 	/*
@@ -620,16 +612,9 @@ int __igt_intel_driver_unload(char **who, const char *driver)
 	const char *aux[] = {
 		/* gen5: ips uses symbol_get() so only a soft module dependency */
 		"intel_ips",
-		/* mei_gsc uses an i915 aux dev and the other mei mods depend on it */
-		"mei_pxp",
-		"mei_hdcp",
-		"mei_gsc",
 		NULL,
 	};
 
-	/* unbind vt */
-	bind_fbcon(false);
-
 	ret = igt_audio_driver_unload(who);
 	if (ret)
 		return ret;
@@ -648,6 +633,8 @@ int __igt_intel_driver_unload(char **who, const char *driver)
 	}
 
 	if (igt_kmod_is_loaded(driver)) {
+		igt_kmod_unbind(driver);
+
 		ret = igt_kmod_unload(driver);
 		if (ret) {
 			if (who)
@@ -694,11 +681,8 @@ int igt_kmod_unbind(const char *mod_name)
 		igt_assert(ret);
 
 		close(devfd);
-		errno = 0;
 	}
 
-	igt_assert_eq(errno, 0);
-
 	return 0;
 }
 
@@ -706,7 +690,6 @@ int igt_kmod_unbind(const char *mod_name)
  * igt_intel_driver_unload:
  *
  * Unloads an Intel driver and its dependencies.
- *
  */
 int
 igt_intel_driver_unload(const char *driver)
@@ -725,13 +708,7 @@ igt_intel_driver_unload(const char *driver)
 	}
 	free(who);
 
-	if (igt_kmod_is_loaded("intel-gtt"))
-		igt_kmod_unload("intel-gtt");
-
-	igt_kmod_unload("drm_kms_helper");
-	igt_kmod_unload("drm");
-
-	if (igt_kmod_is_loaded("driver")) {
+	if (igt_kmod_is_loaded(driver)) {
 		igt_warn("%s.ko still loaded!\n", driver);
 		return -EBUSY;
 	}
