@@ -16,6 +16,7 @@
 #include "intel_pat.h"
 #include "xe_eudebug.h"
 #include "xe_ioctl.h"
+#include "xe/xe_query.h"
 
 struct event_trigger {
 	xe_eudebug_trigger_fn fn;
@@ -77,8 +78,6 @@ static const char *type_to_str(unsigned int type)
 		return "vm";
 	case DRM_XE_EUDEBUG_EVENT_EXEC_QUEUE:
 		return "exec_queue";
-	case DRM_XE_EUDEBUG_EVENT_EXEC_QUEUE_PLACEMENTS:
-		return "exec_queue_placements";
 	case DRM_XE_EUDEBUG_EVENT_EU_ATTENTION:
 		return "attention";
 	case DRM_XE_EUDEBUG_EVENT_VM_BIND:
@@ -124,36 +123,24 @@ static const char *flags_to_str(unsigned int flags)
 	return "flags unknown";
 }
 
-static const char *eu_engine_class_to_str(uint16_t engine_class)
-{
-	switch (engine_class) {
-	case DRM_XE_ENGINE_CLASS_COMPUTE:
-		return "ccs";
-	case DRM_XE_ENGINE_CLASS_RENDER:
-		return "rcs";
-	default:
-		return "unsupported class";
-	}
-}
-
 static const char *event_members_to_str(struct drm_xe_eudebug_event *e, char *buf)
 {
 	switch (e->type) {
 	case DRM_XE_EUDEBUG_EVENT_OPEN: {
-		struct drm_xe_eudebug_event_client *ec = (void *)e;
+		struct drm_xe_eudebug_event_client *ec = igt_container_of(e, ec, base);
 
 		sprintf(buf, "handle=%llu", ec->client_handle);
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM: {
-		struct drm_xe_eudebug_event_vm *evm = (void *)e;
+		struct drm_xe_eudebug_event_vm *evm = igt_container_of(e, evm, base);
 
 		sprintf(buf, "client_handle=%llu, handle=%llu",
 			evm->client_handle, evm->vm_handle);
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_EXEC_QUEUE: {
-		struct drm_xe_eudebug_event_exec_queue *ee = (void *)e;
+		struct drm_xe_eudebug_event_exec_queue *ee = igt_container_of(e, ee, base);
 
 		sprintf(buf, "client_handle=%llu, vm_handle=%llu, "
 			"exec_queue_handle=%llu, engine_class=%d, exec_queue_width=%d",
@@ -161,28 +148,8 @@ static const char *event_members_to_str(struct drm_xe_eudebug_event *e, char *bu
 			ee->exec_queue_handle, ee->engine_class, ee->width);
 		break;
 	}
-	case DRM_XE_EUDEBUG_EVENT_EXEC_QUEUE_PLACEMENTS: {
-		struct drm_xe_eudebug_event_exec_queue_placements *ee = (void *)e;
-		struct drm_xe_engine_class_instance *instances = (void *)(ee->instances);
-		int i, l;
-
-		l = sprintf(buf, "client_handle=%llu, vm_handle=%llu, "
-			    "exec_queue_handle=%llu, lrc_handle=%llu, "
-			    "num_placements=%d, gt_id=%d, mask=[",
-			    ee->client_handle, ee->vm_handle,
-			    ee->exec_queue_handle, ee->lrc_handle,
-			    ee->num_placements, instances[0].gt_id);
-
-		for (i = 0; i < ee->num_placements; i++)
-			l += sprintf(buf + l, "%s%d pad%d, ",
-				     eu_engine_class_to_str(instances[i].engine_class),
-				     instances[i].engine_instance, instances[i].pad);
-		buf[l - 2] = ']';
-
-		break;
-	}
 	case DRM_XE_EUDEBUG_EVENT_EU_ATTENTION: {
-		struct drm_xe_eudebug_event_eu_attention *ea = (void *)e;
+		struct drm_xe_eudebug_event_eu_attention *ea = igt_container_of(e, ea, base);
 
 		sprintf(buf, "client_handle=%llu, exec_queue_handle=%llu, "
 			"lrc_handle=%llu, bitmask_size=%d",
@@ -191,34 +158,34 @@ static const char *event_members_to_str(struct drm_xe_eudebug_event *e, char *bu
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM_BIND: {
-		struct drm_xe_eudebug_event_vm_bind *evmb = (void *)e;
+		struct drm_xe_eudebug_event_vm_bind *evmb = igt_container_of(e, evmb, base);
 
 		sprintf(buf, "client_handle=%llu, vm_handle=%llu, flags=0x%x, num_binds=%u",
 			evmb->client_handle, evmb->vm_handle, evmb->flags, evmb->num_binds);
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM_BIND_OP: {
-		struct drm_xe_eudebug_event_vm_bind_op *op = (void *)e;
+		struct drm_xe_eudebug_event_vm_bind_op *op = igt_container_of(e, op, base);
 
 		sprintf(buf, "vm_bind_ref_seqno=%lld, addr=%016llx, range=%llu num_extensions=%llu",
 			op->vm_bind_ref_seqno, op->addr, op->range, op->num_extensions);
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM_BIND_UFENCE: {
-		struct drm_xe_eudebug_event_vm_bind_ufence *f = (void *)e;
+		struct drm_xe_eudebug_event_vm_bind_ufence *f = igt_container_of(e, f, base);
 
 		sprintf(buf, "vm_bind_ref_seqno=%lld", f->vm_bind_ref_seqno);
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_METADATA: {
-		struct drm_xe_eudebug_event_metadata *em = (void *)e;
+		struct drm_xe_eudebug_event_metadata *em = igt_container_of(e, em, base);
 
 		sprintf(buf, "client_handle=%llu, metadata_handle=%llu, type=%llu, len=%llu",
 			em->client_handle, em->metadata_handle, em->type, em->len);
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM_BIND_OP_METADATA: {
-		struct drm_xe_eudebug_event_vm_bind_op_metadata *op = (void *)e;
+		struct drm_xe_eudebug_event_vm_bind_op_metadata *op = igt_container_of(e, op, base);
 
 		sprintf(buf, "vm_bind_op_ref_seqno=%lld, metadata_handle=%llu, metadata_cookie=%llu",
 			op->vm_bind_op_ref_seqno, op->metadata_handle, op->metadata_cookie);
@@ -334,14 +301,14 @@ static uint64_t __wait_token(int pipe[2], const uint64_t token, int timeout_ms)
 
 	ret = safe_pipe_read(pipe, &in, sizeof(in), timeout_ms);
 	igt_assert_f(ret > 0,
-		     "Pipe read timeout waiting for token '%s:(%ld)'\n",
+		     "Pipe read timeout waiting for token '%s:(%" PRId64 ")'\n",
 		     token_to_str(token), token);
 
 	igt_assert_eq(in, token);
 
 	ret = safe_pipe_read(pipe, &in, sizeof(in), timeout_ms);
 	igt_assert_f(ret > 0,
-		     "Pipe read timeout waiting for token value '%s:(%ld)'\n",
+		     "Pipe read timeout waiting for token value '%s:(%" PRId64 ")'\n",
 		     token_to_str(token), token);
 
 	return in;
@@ -461,35 +428,24 @@ static int match_fields(struct drm_xe_eudebug_event *a, void *data)
 
 	switch (a->type) {
 	case DRM_XE_EUDEBUG_EVENT_EXEC_QUEUE: {
-		struct drm_xe_eudebug_event_exec_queue *ae = (void *)a;
-		struct drm_xe_eudebug_event_exec_queue *be = (void *)b;
+		struct drm_xe_eudebug_event_exec_queue *ae = igt_container_of(a, ae, base);
+		struct drm_xe_eudebug_event_exec_queue *be = igt_container_of(b, be, base);
 
 		if (ae->engine_class == be->engine_class && ae->width == be->width)
 			ret = 1;
 		break;
 	}
-	case DRM_XE_EUDEBUG_EVENT_EXEC_QUEUE_PLACEMENTS: {
-		struct drm_xe_eudebug_event_exec_queue_placements *ae = (void *)a;
-		struct drm_xe_eudebug_event_exec_queue_placements *be = (void *)b;
-
-		if (ae->num_placements == be->num_placements &&
-		    memcmp(ae->instances, be->instances,
-			   sizeof(uint64_t) * ae->num_placements) == 0)
-			ret = 1;
-
-		break;
-	}
 	case DRM_XE_EUDEBUG_EVENT_VM_BIND: {
-		struct drm_xe_eudebug_event_vm_bind *ea = (void *)a;
-		struct drm_xe_eudebug_event_vm_bind *eb = (void *)b;
+		struct drm_xe_eudebug_event_vm_bind *ea = igt_container_of(a, ea, base);
+		struct drm_xe_eudebug_event_vm_bind *eb = igt_container_of(b, eb, base);
 
 		if (ea->num_binds == eb->num_binds)
 			ret = 1;
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM_BIND_OP: {
-		struct drm_xe_eudebug_event_vm_bind_op *ea = (void *)a;
-		struct drm_xe_eudebug_event_vm_bind_op *eb = (void *)b;
+		struct drm_xe_eudebug_event_vm_bind_op *ea = igt_container_of(a, ea, base);
+		struct drm_xe_eudebug_event_vm_bind_op *eb = igt_container_of(b, eb, base);
 
 		if (ea->addr == eb->addr && ea->range == eb->range &&
 		    ea->num_extensions == eb->num_extensions)
@@ -497,8 +453,8 @@ static int match_fields(struct drm_xe_eudebug_event *a, void *data)
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM_BIND_OP_METADATA: {
-		struct drm_xe_eudebug_event_vm_bind_op_metadata *ea = (void *)a;
-		struct drm_xe_eudebug_event_vm_bind_op_metadata *eb = (void *)b;
+		struct drm_xe_eudebug_event_vm_bind_op_metadata *ea = igt_container_of(a, ea, base);
+		struct drm_xe_eudebug_event_vm_bind_op_metadata *eb = igt_container_of(b, eb, base);
 
 		if (ea->metadata_handle == eb->metadata_handle &&
 		    ea->metadata_cookie == eb->metadata_cookie)
@@ -526,35 +482,28 @@ static int match_client_handle(struct drm_xe_eudebug_event *e, void *data)
 
 	switch (e->type) {
 	case DRM_XE_EUDEBUG_EVENT_OPEN: {
-		struct drm_xe_eudebug_event_client *client = (void *)e;
+		struct drm_xe_eudebug_event_client *client = igt_container_of(e, client, base);
 
 		if (client->client_handle == h)
 			return 1;
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM: {
-		struct drm_xe_eudebug_event_vm *vm = (void *)e;
+		struct drm_xe_eudebug_event_vm *vm = igt_container_of(e, vm, base);
 
 		if (vm->client_handle == h)
 			return 1;
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_EXEC_QUEUE: {
-		struct drm_xe_eudebug_event_exec_queue *ee = (void *)e;
-
-		if (ee->client_handle == h)
-			return 1;
-		break;
-	}
-	case DRM_XE_EUDEBUG_EVENT_EXEC_QUEUE_PLACEMENTS: {
-		struct drm_xe_eudebug_event_exec_queue_placements *ee = (void *)e;
+	struct drm_xe_eudebug_event_exec_queue *ee = igt_container_of(e, ee, base);
 
 		if (ee->client_handle == h)
 			return 1;
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM_BIND: {
-		struct drm_xe_eudebug_event_vm_bind *evmb = (void *)e;
+		struct drm_xe_eudebug_event_vm_bind *evmb = igt_container_of(e, evmb, base);
 
 		if (evmb->client_handle == h) {
 			*bind_seqno = evmb->base.seqno;
@@ -563,7 +512,7 @@ static int match_client_handle(struct drm_xe_eudebug_event *e, void *data)
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM_BIND_OP: {
-		struct drm_xe_eudebug_event_vm_bind_op *eo = (void *)e;
+		struct drm_xe_eudebug_event_vm_bind_op *eo = igt_container_of(e, eo, base);
 
 		if (eo->vm_bind_ref_seqno == *bind_seqno) {
 			*bind_op_seqno = eo->base.seqno;
@@ -572,7 +521,7 @@ static int match_client_handle(struct drm_xe_eudebug_event *e, void *data)
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM_BIND_UFENCE: {
-		struct drm_xe_eudebug_event_vm_bind_ufence *ef = (void *)e;
+		struct drm_xe_eudebug_event_vm_bind_ufence *ef = igt_container_of(e, ef, base);
 
 		if (ef->vm_bind_ref_seqno == *bind_seqno)
 			return 1;
@@ -580,14 +529,14 @@ static int match_client_handle(struct drm_xe_eudebug_event *e, void *data)
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_METADATA: {
-		struct drm_xe_eudebug_event_metadata *em = (void *)e;
+		struct drm_xe_eudebug_event_metadata *em = igt_container_of(e, em, base);
 
 		if (em->client_handle == h)
 			return 1;
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM_BIND_OP_METADATA: {
-		struct drm_xe_eudebug_event_vm_bind_op_metadata *eo = (void *)e;
+		struct drm_xe_eudebug_event_vm_bind_op_metadata *eo = igt_container_of(e, eo, base);
 
 		if (eo->vm_bind_op_ref_seqno == *bind_op_seqno)
 			return 1;
@@ -615,7 +564,7 @@ static int match_opposite_resource(struct drm_xe_eudebug_event *e, void *data)
 
 	switch (e->type) {
 	case DRM_XE_EUDEBUG_EVENT_OPEN: {
-		struct drm_xe_eudebug_event_client *client = (void *)e;
+		struct drm_xe_eudebug_event_client *client = igt_container_of(e, client, base);
 		struct drm_xe_eudebug_event_client *filter = data;
 
 		if (client->client_handle == filter->client_handle)
@@ -623,7 +572,7 @@ static int match_opposite_resource(struct drm_xe_eudebug_event *e, void *data)
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM: {
-		struct drm_xe_eudebug_event_vm *vm = (void *)e;
+		struct drm_xe_eudebug_event_vm *vm = igt_container_of(e, vm, base);
 		struct drm_xe_eudebug_event_vm *filter = data;
 
 		if (vm->vm_handle == filter->vm_handle)
@@ -631,7 +580,7 @@ static int match_opposite_resource(struct drm_xe_eudebug_event *e, void *data)
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_EXEC_QUEUE: {
-		struct drm_xe_eudebug_event_exec_queue *ee = (void *)e;
+		struct drm_xe_eudebug_event_exec_queue *ee = igt_container_of(e, ee, base);
 		struct drm_xe_eudebug_event_exec_queue *filter = data;
 
 		if (ee->exec_queue_handle == filter->exec_queue_handle)
@@ -639,7 +588,7 @@ static int match_opposite_resource(struct drm_xe_eudebug_event *e, void *data)
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM_BIND: {
-		struct drm_xe_eudebug_event_vm_bind *evmb = (void *)e;
+		struct drm_xe_eudebug_event_vm_bind *evmb = igt_container_of(e, evmb, base);
 		struct drm_xe_eudebug_event_vm_bind *filter = data;
 
 		if (evmb->vm_handle == filter->vm_handle &&
@@ -648,7 +597,7 @@ static int match_opposite_resource(struct drm_xe_eudebug_event *e, void *data)
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM_BIND_OP: {
-		struct drm_xe_eudebug_event_vm_bind_op *avmb = (void *)e;
+		struct drm_xe_eudebug_event_vm_bind_op *avmb = igt_container_of(e, avmb, base);
 		struct drm_xe_eudebug_event_vm_bind_op *filter = data;
 
 		if (avmb->addr == filter->addr &&
@@ -657,7 +606,7 @@ static int match_opposite_resource(struct drm_xe_eudebug_event *e, void *data)
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_METADATA: {
-		struct drm_xe_eudebug_event_metadata *em = (void *)e;
+		struct drm_xe_eudebug_event_metadata *em = igt_container_of(e, em, base);
 		struct drm_xe_eudebug_event_metadata *filter = data;
 
 		if (em->metadata_handle == filter->metadata_handle)
@@ -665,7 +614,8 @@ static int match_opposite_resource(struct drm_xe_eudebug_event *e, void *data)
 		break;
 	}
 	case DRM_XE_EUDEBUG_EVENT_VM_BIND_OP_METADATA: {
-		struct drm_xe_eudebug_event_vm_bind_op_metadata *avmb = (void *)e;
+		struct drm_xe_eudebug_event_vm_bind_op_metadata *avmb = igt_container_of(e, avmb,
+											 base);
 		struct drm_xe_eudebug_event_vm_bind_op_metadata *filter = data;
 
 		if (avmb->metadata_handle == filter->metadata_handle &&
@@ -760,8 +710,8 @@ static void compare_client(struct xe_eudebug_event_log *log1, struct drm_xe_eude
 			   struct xe_eudebug_event_log *log2, struct drm_xe_eudebug_event *ev2,
 			   uint32_t filter)
 {
-	struct drm_xe_eudebug_event_client *ev1_client = (void *)ev1;
-	struct drm_xe_eudebug_event_client *ev2_client = (void *)ev2;
+	struct drm_xe_eudebug_event_client *ev1_client = igt_container_of(ev1, ev1_client, base);
+	struct drm_xe_eudebug_event_client *ev2_client = igt_container_of(ev2, ev2_client, base);
 	uint64_t cbs = 0, dbs = 0, cbso = 0, dbso = 0;
 
 	struct igt_list_head matched_seqno_list;
@@ -834,7 +784,8 @@ xe_eudebug_event_log_find_seqno(struct xe_eudebug_event_log *l, uint64_t seqno)
 	xe_eudebug_for_each_event(e, l) {
 		if (e->seqno == seqno) {
 			if (found) {
-				igt_warn("Found multiple events with the same seqno %lu\n", seqno);
+				igt_warn("Found multiple events with the same seqno %" PRIu64 "\n",
+					 seqno);
 				xe_eudebug_event_log_print(l, false);
 				igt_assert(!found);
 			}
@@ -1049,11 +1000,10 @@ xe_eudebug_event_log_match_opposite(struct xe_eudebug_event_log *l, uint32_t fil
 			if (XE_EUDEBUG_EVENT_IS_FILTERED(ev1->type, filter))
 				continue;
 
-			/* No opposite matching for some events */
+			/* No opposite matching for binds */
 			if ((ev1->type >= DRM_XE_EUDEBUG_EVENT_VM_BIND &&
 			     ev1->type <= DRM_XE_EUDEBUG_EVENT_VM_BIND_UFENCE) ||
-			    ev1->type == DRM_XE_EUDEBUG_EVENT_VM_BIND_OP_METADATA ||
-			    ev1->type == DRM_XE_EUDEBUG_EVENT_EXEC_QUEUE_PLACEMENTS)
+			    ev1->type == DRM_XE_EUDEBUG_EVENT_VM_BIND_OP_METADATA)
 				continue;
 
 			ev2 = opposite_event_match(l, ev1, ev1);
@@ -1100,6 +1050,11 @@ xe_eudebug_read_event(int fd, struct drm_xe_eudebug_event *event)
 	return ret;
 }
 
+static void terminate_debugger(int sig)
+{
+	pthread_exit(NULL);
+}
+
 static void *debugger_worker_loop(void *data)
 {
 	uint8_t buf[MAX_EVENT_SIZE];
@@ -1110,9 +1065,14 @@ static void *debugger_worker_loop(void *data)
 		.revents = 0,
 	};
 	int timeout_ms = 100, ret;
+	struct sigaction sa = { 0 };
 
 	igt_assert(d->master_fd >= 0);
 
+	igt_assert_eq(sigaction(SIGINT, NULL, &sa), 0);
+	sa.sa_handler = terminate_debugger;
+	igt_assert_eq(sigaction(SIGINT, &sa, NULL), 0);
+
 	do {
 		p.fd = d->fd;
 		ret = poll(&p, 1, timeout_ms);
@@ -1244,7 +1204,7 @@ int xe_eudebug_debugger_attach(struct xe_eudebug_debugger *d,
 	d->p_client[0] = c->p_in[0];
 	d->p_client[1] = c->p_in[1];
 
-	igt_debug("debugger connected to %lu\n", d->target_pid);
+	igt_debug("debugger connected to %" PRIu64 "\n", d->target_pid);
 
 	return 0;
 }
@@ -1397,11 +1357,11 @@ void xe_eudebug_debugger_wait_stage(struct xe_eudebug_session *s, uint64_t stage
 {
 	u64 stage_in;
 
-	igt_debug("debugger xe client fd: %d pausing for stage %lu\n", s->debugger->master_fd,
+	igt_debug("debugger xe client fd: %d pausing for stage %" PRIu64 "\n", s->debugger->master_fd,
 		  stage);
 
 	stage_in = wait_from_client(s->client, DEBUGGER_STAGE);
-	igt_debug("debugger xe client fd: %d stage %lu, expected %lu, stage\n",
+	igt_debug("debugger xe client fd: %d stage %" PRIu64 ", expected %" PRIu64 ", stage\n",
 		  s->debugger->master_fd, stage_in, stage);
 
 	igt_assert_eq(stage_in, stage);
@@ -1490,12 +1450,16 @@ void xe_eudebug_client_stop(struct xe_eudebug_client *c)
 {
 	if (c->pid) {
 		int waitstatus;
+		int ret;
 
 		xe_eudebug_client_wait_done(c);
 
 		token_signal(c->p_in, CLIENT_STOP, c->pid);
-		igt_assert_eq(waitpid(c->pid, &waitstatus, 0),
-			      c->pid);
+		ret = waitpid(c->pid, &waitstatus, 0);
+		/* process may be gone already */
+		if (!(ret == -1 && errno == ECHILD))
+			igt_assert_eq(ret, c->pid);
+
 		c->pid = 0;
 	}
 }
@@ -1590,14 +1554,15 @@ void xe_eudebug_client_wait_stage(struct xe_eudebug_client *c, uint64_t stage)
 	u64 stage_in;
 
 	if (c->done) {
-		igt_warn("client: %d already done before %lu\n", c->pid, stage);
+		igt_warn("client: %d already done before %" PRIu64 "\n", c->pid, stage);
 		return;
 	}
 
-	igt_debug("client: %d pausing for stage %lu\n", c->pid, stage);
+	igt_debug("client: %d pausing for stage %" PRIu64 "\n", c->pid, stage);
 
 	stage_in = client_wait_token(c, CLIENT_STAGE);
-	igt_debug("client: %d stage %lu, expected %lu, stage\n", c->pid, stage_in, stage);
+	igt_debug("client: %d stage %" PRIu64 ", expected %" PRIu64 ", stage\n", c->pid,
+		  stage_in, stage);
 
 	igt_assert_eq(stage_in, stage);
 }
@@ -1743,36 +1708,6 @@ static void exec_queue_event(struct xe_eudebug_client *c, uint32_t flags,
 	xe_eudebug_event_log_write(c->log, (void *)&ee);
 }
 
-static void exec_queue_placements_event(struct xe_eudebug_client *c,
-					int client_fd, uint32_t vm_id,
-					uint32_t exec_queue_handle,
-					uint16_t width, uint16_t lrc_no,
-					uint16_t num_placements,
-					struct drm_xe_engine_class_instance *eci)
-{
-	struct drm_xe_eudebug_event_exec_queue_placements *ee;
-	struct drm_xe_engine_class_instance *instances;
-	size_t sz = sizeof(*ee) + num_placements * sizeof(uint64_t);
-
-	ee = calloc(1, sz);
-	igt_assert(ee);
-
-	base_event(c, to_base(*ee), DRM_XE_EUDEBUG_EVENT_EXEC_QUEUE_PLACEMENTS,
-		   DRM_XE_EUDEBUG_EVENT_CREATE, sz);
-
-	ee->client_handle = client_fd;
-	ee->vm_handle = vm_id;
-	ee->exec_queue_handle = exec_queue_handle;
-	ee->num_placements = num_placements;
-
-	instances = (struct drm_xe_engine_class_instance *)(ee->instances);
-	for (int j = 0; j < num_placements; j++)
-		instances[j] = eci[j * width + lrc_no];
-
-	xe_eudebug_event_log_write(c->log, (void *)ee);
-	free(ee);
-}
-
 static void metadata_event(struct xe_eudebug_client *c, uint32_t flags,
 			   int client_fd, uint32_t id, uint64_t type, uint64_t len)
 {
@@ -1789,7 +1724,18 @@ static void metadata_event(struct xe_eudebug_client *c, uint32_t flags,
 	xe_eudebug_event_log_write(c->log, (void *)&em);
 }
 
-static int enable_getset(int fd, bool *old, bool *new)
+/**
+ * __xe_eudebug_enable_getset
+ * @fd: xe client
+ * @old: pointer to store current toggle value
+ * @new: pointer to new toggle value
+ *
+ * Stores current eudebug feature state in @old if not NULL. Sets new eudebug
+ * feature state to @new if not NULL. Asserts if both @old and @new are NULL.
+ *
+ * Returns: 0 on success, -1 on failure.
+ */
+int __xe_eudebug_enable_getset(int fd, bool *old, bool *new)
 {
 	static const char * const fname = "enable_eudebug";
 	int ret = 0;
@@ -1846,7 +1792,7 @@ out:
 bool xe_eudebug_enable(int fd, bool enable)
 {
 	bool old = false;
-	int ret = enable_getset(fd, &old, &enable);
+	int ret = __xe_eudebug_enable_getset(fd, &old, &enable);
 
 	if (ret) {
 		igt_skip_on(enable);
@@ -1955,7 +1901,7 @@ uint32_t xe_eudebug_client_exec_queue_create(struct xe_eudebug_client *c, int fd
 	igt_assert(c);
 	igt_assert(create);
 
-	instances = (struct drm_xe_engine_class_instance *)(create->instances);
+	instances = from_user_pointer(create->instances);
 	class = instances[0].engine_class;
 
 	igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_XE_EXEC_QUEUE_CREATE, create), 0);
@@ -1967,18 +1913,10 @@ uint32_t xe_eudebug_client_exec_queue_create(struct xe_eudebug_client *c, int fd
 		    ext->value & DRM_XE_EXEC_QUEUE_EUDEBUG_FLAG_ENABLE)
 			send = true;
 
-	if (send) {
+	if (send)
 		exec_queue_event(c, DRM_XE_EUDEBUG_EVENT_CREATE, fd, create->vm_id,
 				 create->exec_queue_id, class, create->width);
 
-		for (int i = 0; i < create->width; i++) {
-			exec_queue_placements_event(c, fd, create->vm_id, create->exec_queue_id,
-						    create->width, i,
-						    create->num_placements,
-						    instances);
-		}
-	}
-
 	return create->exec_queue_id;
 }
 
@@ -1994,6 +1932,7 @@ uint32_t xe_eudebug_client_exec_queue_create(struct xe_eudebug_client *c, int fd
 void xe_eudebug_client_exec_queue_destroy(struct xe_eudebug_client *c, int fd,
 					  struct drm_xe_exec_queue_create *create)
 {
+	struct drm_xe_engine_class_instance *instances;
 	struct drm_xe_exec_queue_destroy destroy = {};
 	struct drm_xe_ext_set_property *ext;
 	bool send = false;
@@ -2003,7 +1942,8 @@ void xe_eudebug_client_exec_queue_destroy(struct xe_eudebug_client *c, int fd,
 	igt_assert(create);
 
 	destroy.exec_queue_id = create->exec_queue_id;
-	class = ((struct drm_xe_engine_class_instance *)(create->instances))[0].engine_class;
+	instances = from_user_pointer(create->instances);
+	class = instances[0].engine_class;
 
 	for (ext = from_user_pointer(create->extensions); ext;
 	     ext = from_user_pointer(ext->base.next_extension))
