@@ -21,9 +21,9 @@
 #include "intel_hwconfig_types.h"
 #include "ioctl_wrappers.h"
 #include "linux_scaffold.h"
+#include "pciids.h"
 #include "xe_ioctl.h"
 #include "xe_oa.h"
-#include "xe_pciids.h"
 #include "xe_query.h"
 
 #include "xe_oa_metrics_tglgt1.h"
@@ -38,6 +38,7 @@
 #include "xe_oa_metrics_mtlgt3.h"
 #include "xe_oa_metrics_pvc.h"
 #include "xe_oa_metrics_lnl.h"
+#include "xe_oa_metrics_bmg.h"
 
 static struct intel_xe_perf_logical_counter_group *
 intel_xe_perf_logical_counter_group_new(struct intel_xe_perf *perf,
@@ -118,8 +119,8 @@ static bool
 is_acm_gt1(const struct intel_xe_perf_devinfo *devinfo)
 {
 	static const uint32_t devids[] = {
-		XE_DG2_G11_IDS(DEVID),
-		XE_ATS_M75_IDS(DEVID),
+		INTEL_DG2_G11_IDS(DEVID),
+		INTEL_ATS_M75_IDS(DEVID),
 	};
 	for (uint32_t i = 0; i < ARRAY_SIZE(devids); i++) {
 		if (devids[i] == devinfo->devid)
@@ -133,7 +134,7 @@ static bool
 is_acm_gt2(const struct intel_xe_perf_devinfo *devinfo)
 {
 	static const uint32_t devids[] = {
-		XE_DG2_G12_IDS(DEVID),
+		INTEL_DG2_G12_IDS(DEVID),
 	};
 	for (uint32_t i = 0; i < ARRAY_SIZE(devids); i++) {
 		if (devids[i] == devinfo->devid)
@@ -147,8 +148,8 @@ static bool
 is_acm_gt3(const struct intel_xe_perf_devinfo *devinfo)
 {
 	static const uint32_t devids[] = {
-		XE_DG2_G10_IDS(DEVID),
-		XE_ATS_M150_IDS(DEVID),
+		INTEL_DG2_G10_IDS(DEVID),
+		INTEL_ATS_M150_IDS(DEVID),
 	};
 	for (uint32_t i = 0; i < ARRAY_SIZE(devids); i++) {
 		if (devids[i] == devinfo->devid)
@@ -295,6 +296,10 @@ intel_xe_perf_for_devinfo(uint32_t device_id,
 	} else if (devinfo->is_pontevecchio) {
 		perf->devinfo.eu_threads_count = 8;
 		intel_xe_perf_load_metrics_pvc(perf);
+	} else if (devinfo->is_lunarlake) {
+		intel_xe_perf_load_metrics_lnl(perf);
+	} else if (devinfo->is_battlemage) {
+		intel_xe_perf_load_metrics_bmg(perf);
 	} else if (intel_graphics_ver(device_id) >= IP_VER(20, 0)) {
 		intel_xe_perf_load_metrics_lnl(perf);
 	} else {
@@ -566,11 +571,9 @@ xe_perf_for_fd(int drm_fd, int gt)
 	}
 
 	if (!read_sysfs(sysfs_dir_fd, path_min, &gt_min_freq) ||
-	    !read_sysfs(sysfs_dir_fd, path_max, &gt_max_freq)) {
-		igt_warn("Unable to read freqs from sysfs\n");
-		close(sysfs_dir_fd);
-		return NULL;
-	}
+	    !read_sysfs(sysfs_dir_fd, path_max, &gt_max_freq))
+		igt_info("Unable to read freqs from sysfs\n");
+
 	close(sysfs_dir_fd);
 
 	device_id = intel_get_drm_devid(drm_fd);
@@ -1022,8 +1025,17 @@ const char *intel_xe_perf_read_report_reason(const struct intel_xe_perf *perf,
 	return "unknown";
 }
 
-static void xe_oa_prop_to_ext(struct intel_xe_oa_open_prop *properties,
-			      struct drm_xe_ext_set_property *extn)
+/**
+ * intel_xe_oa_prop_to_ext:
+ * @properties: pointer to internal IGT properties
+ * @extn: Pointer to array of set_property uapi structs
+ *
+ * Convert 'struct intel_xe_oa_open_prop' properties used internally in IGT
+ * into chained 'struct drm_xe_ext_set_property' structures used in
+ * OA/observation uapi
+ */
+void intel_xe_oa_prop_to_ext(struct intel_xe_oa_open_prop *properties,
+			     struct drm_xe_ext_set_property *extn)
 {
 	__u64 *prop = from_user_pointer(properties->properties_ptr);
 	struct drm_xe_ext_set_property *ext = extn;
@@ -1060,7 +1072,7 @@ int intel_xe_perf_ioctl(int fd, enum drm_xe_observation_op op, void *arg)
 		struct intel_xe_oa_open_prop *oprop = (struct intel_xe_oa_open_prop *)arg;
 
 		igt_assert_lte(oprop->num_properties, XE_OA_MAX_SET_PROPERTIES);
-		xe_oa_prop_to_ext(oprop, ext);
+		intel_xe_oa_prop_to_ext(oprop, ext);
 	}
 
 	return igt_ioctl(fd, DRM_IOCTL_XE_OBSERVATION, &p);
