@@ -13,43 +13,12 @@
 #include "igt_core.h"
 #include "igt_syncobj.h"
 #include "intel_reg.h"
+
 #include "xe_ioctl.h"
 #include "xe_spin.h"
+#include "xe_util.h"
 
-static uint32_t read_timestamp_frequency(int fd, int gt_id)
-{
-	struct xe_device *dev = xe_device_get(fd);
-
-	igt_assert(dev && dev->gt_list && dev->gt_list->num_gt);
-	igt_assert(gt_id >= 0 && gt_id <= dev->gt_list->num_gt);
-
-	return dev->gt_list->gt_list[gt_id].reference_clock;
-}
-
-static uint64_t div64_u64_round_up(const uint64_t x, const uint64_t y)
-{
-	igt_assert(y > 0);
-	igt_assert_lte_u64(x, UINT64_MAX - (y - 1));
-
-	return (x + y - 1) / y;
-}
-
-/**
- * duration_to_ctx_ticks:
- * @fd: opened device
- * @gt_id: tile id
- * @duration_ns: duration in nanoseconds to be converted to context timestamp ticks
- * @return: duration converted to context timestamp ticks.
- */
-uint32_t duration_to_ctx_ticks(int fd, int gt_id, uint64_t duration_ns)
-{
-	uint32_t f = read_timestamp_frequency(fd, gt_id);
-	uint64_t ctx_ticks = div64_u64_round_up(duration_ns * f, NSEC_PER_SEC);
-
-	igt_assert_lt_u64(ctx_ticks, XE_SPIN_MAX_CTX_TICKS);
-
-	return ctx_ticks;
-}
+#define XE_SPIN_MAX_CTX_TICKS (UINT32_MAX - 1000)
 
 #define MI_SRM_CS_MMIO				(1 << 19)
 #define MI_LRI_CS_MMIO				(1 << 19)
@@ -60,6 +29,16 @@ uint32_t duration_to_ctx_ticks(int fd, int gt_id, uint64_t duration_ns)
 
 enum { START_TS, NOW_TS };
 
+
+uint32_t xe_spin_nsec_to_ticks(int fd, int gt_id, uint64_t nsec)
+{
+	uint32_t ticks = xe_nsec_to_ticks(fd, gt_id, nsec);
+
+	igt_assert_lt_u64(ticks, XE_SPIN_MAX_CTX_TICKS);
+
+	return ticks;
+}
+
 /**
  * xe_spin_init:
  * @spin: pointer to mapped bo in which spinner code will be written
@@ -253,7 +232,7 @@ xe_spin_create(int fd, const struct igt_spin_factory *opt)
 	return spin;
 }
 
-void xe_spin_sync_wait(int fd, struct igt_spin *spin)
+static void xe_spin_sync_wait(int fd, struct igt_spin *spin)
 {
 	igt_assert(syncobj_wait(fd, &spin->syncobj, 1, INT64_MAX, 0, NULL));
 }
@@ -292,77 +271,168 @@ void xe_spin_free(int fd, struct igt_spin *spin)
 	free(spin);
 }
 
-void xe_cork_init(int fd, struct drm_xe_engine_class_instance *hwe,
-		  struct xe_cork *cork)
+/**
+ * xe_cork_create:
+ * @fd: xe device fd
+ * @hwe: Xe engine class instance if device is Xe
+ * @vm: vm handle
+ * @width: number of batch buffers
+ * @num_placements: number of valid placements for this exec queue
+ * @opts: controlling options such as allocator handle, debug.
+ *
+ * xe_cork_create create vm, bo, exec_queue and bind the buffer
+ * using vmbind
+ *
+ * This returns xe_cork after binding buffer object.
+ */
+
+struct xe_cork *
+xe_cork_create(int fd, struct drm_xe_engine_class_instance *hwe,
+		uint32_t vm, uint16_t width, uint16_t num_placements,
+		struct xe_cork_opts *opts)
 {
-	uint64_t addr = xe_get_default_alignment(fd);
-	size_t bo_size = xe_bb_size(fd, SZ_4K);
-	uint32_t vm, bo, exec_queue, syncobj;
-	struct xe_spin *spin;
-	struct drm_xe_sync sync = {
-		.type = DRM_XE_SYNC_TYPE_SYNCOBJ, .flags = DRM_XE_SYNC_FLAG_SIGNAL,
-	};
-	struct drm_xe_exec exec = {
-		.num_batch_buffer = 1,
-		.num_syncs = 1,
-		.syncs = to_user_pointer(&sync),
-	};
+	struct xe_cork *cork = calloc(1, sizeof(*cork));
 
-	vm = xe_vm_create(fd, 0, 0);
+	igt_assert(cork);
+	igt_assert(width && num_placements &&
+		   (width == 1 || num_placements == 1));
+	igt_assert_lt(width, XE_MAX_ENGINE_INSTANCE);
 
-	bo = xe_bo_create(fd, vm, bo_size, vram_if_possible(fd, hwe->gt_id),
-			  DRM_XE_GEM_CREATE_FLAG_NEEDS_VISIBLE_VRAM);
-	spin = xe_bo_map(fd, bo, 0x1000);
-
-	xe_vm_bind_sync(fd, vm, bo, 0, addr, bo_size);
-
-	exec_queue = xe_exec_queue_create(fd, vm, hwe, 0);
-	syncobj = syncobj_create(fd, 0);
-
-	xe_spin_init_opts(spin, .addr = addr, .preempt = true);
-	exec.exec_queue_id = exec_queue;
-	exec.address = addr;
-	sync.handle = syncobj;
-	igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_XE_EXEC, &exec), 0);
-
-	cork->spin = spin;
-	cork->fd = fd;
+	cork->class = hwe->engine_class;
+	cork->width = width;
+	cork->num_placements = num_placements;
 	cork->vm = vm;
-	cork->bo = bo;
-	cork->exec_queue = exec_queue;
-	cork->syncobj = syncobj;
+	cork->cork_opts = *opts;
+
+	cork->exec.num_batch_buffer = width;
+	cork->exec.num_syncs = 2;
+	cork->exec.syncs = to_user_pointer(cork->sync);
+
+	cork->sync[0].type = DRM_XE_SYNC_TYPE_SYNCOBJ;
+	cork->sync[0].flags = DRM_XE_SYNC_FLAG_SIGNAL;
+	cork->sync[0].handle = syncobj_create(fd, 0);
+
+	cork->sync[1].type = DRM_XE_SYNC_TYPE_SYNCOBJ;
+	cork->sync[1].flags = DRM_XE_SYNC_FLAG_SIGNAL;
+	cork->sync[1].handle = syncobj_create(fd, 0);
+
+	cork->bo_size = sizeof(struct xe_spin);
+	cork->bo_size = xe_bb_size(fd, cork->bo_size);
+	cork->bo = xe_bo_create(fd, cork->vm, cork->bo_size,
+				vram_if_possible(fd, hwe->gt_id),
+				DRM_XE_GEM_CREATE_FLAG_NEEDS_VISIBLE_VRAM);
+	if (cork->cork_opts.ahnd) {
+		for (unsigned int i = 0; i < width; i++)
+			cork->addr[i] = intel_allocator_alloc_with_strategy(cork->cork_opts.ahnd,
+					cork->bo, cork->bo_size, 0,
+					ALLOC_STRATEGY_LOW_TO_HIGH);
+	} else {
+		for (unsigned int i = 0; i < width; i++)
+			cork->addr[i] = 0x100000 + 0x100000 * hwe->engine_class;
+	}
+
+	cork->spin = xe_bo_map(fd, cork->bo, cork->bo_size);
+
+	igt_assert_eq(__xe_exec_queue_create(fd, cork->vm, width, num_placements,
+					     hwe, 0, &cork->exec_queue), 0);
+
+	xe_vm_bind_async(fd, cork->vm, 0, cork->bo, 0, cork->addr[0], cork->bo_size,
+			 cork->sync, 1);
+
+	return cork;
 }
 
-bool xe_cork_started(struct xe_cork *cork)
+/**
+ * xe_cork_sync_start:
+ *
+ * @fd: xe device fd
+ * @cork: pointer to xe_cork structure
+ *
+ * run the spinner using xe_spin_init submit batch using xe_exec
+ * and wait for fence using syncobj_wait
+ */
+void xe_cork_sync_start(int fd, struct xe_cork *cork)
 {
-	return xe_spin_started(cork->spin);
-}
+	igt_assert(cork);
+
+	cork->spin_opts.addr = cork->addr[0];
+	cork->spin_opts.write_timestamp = true;
+	cork->spin_opts.preempt = true;
+	xe_spin_init(cork->spin, &cork->spin_opts);
+
+	/* reuse sync[0] as in-fence for exec */
+	cork->sync[0].flags &= ~DRM_XE_SYNC_FLAG_SIGNAL;
+
+	cork->exec.exec_queue_id = cork->exec_queue;
+
+	if (cork->width > 1)
+		cork->exec.address = to_user_pointer(cork->addr);
+	else
+		cork->exec.address = cork->addr[0];
+
+	xe_exec(fd, &cork->exec);
 
-void xe_cork_wait_started(struct xe_cork *cork)
-{
 	xe_spin_wait_started(cork->spin);
+	igt_assert(!syncobj_wait(fd, &cork->sync[1].handle, 1, 1, 0, NULL));
+
+	if (cork->cork_opts.debug)
+		igt_info("%d: spinner started\n", cork->class);
 }
 
-void xe_cork_end(struct xe_cork *cork)
+/*
+ * xe_cork_sync_end
+ *
+ * @fd: xe device fd
+ * @cork: pointer to xe_cork structure
+ *
+ * Wrapper to end spinner created by xe_cork_create. It will
+ * unbind the vm which was binded to the exec_queue and bo.
+ */
+void xe_cork_sync_end(int fd, struct xe_cork *cork)
 {
+	igt_assert(cork);
+
+	if (cork->ended)
+		igt_warn("Don't attempt call end twice %d\n", cork->ended);
+
 	xe_spin_end(cork->spin);
+
+	igt_assert(syncobj_wait(fd, &cork->sync[1].handle, 1, INT64_MAX, 0, NULL));
+
+	cork->sync[0].flags |= DRM_XE_SYNC_FLAG_SIGNAL;
+	syncobj_reset(fd, &cork->sync[0].handle, 1);
+
+	xe_vm_unbind_async(fd, cork->vm, 0, 0, cork->addr[0], cork->bo_size, cork->sync, 1);
+	igt_assert(syncobj_wait(fd, &cork->sync[0].handle, 1, INT64_MAX, 0, NULL));
+
+	cork->ended = true;
+
+	if (cork->cork_opts.debug)
+		igt_info("%d: spinner ended (timestamp=%u)\n", cork->class,
+			cork->spin->timestamp);
 }
 
-void xe_cork_wait_done(struct xe_cork *cork)
+/*
+ * xe_cork_destroy
+ *
+ * @fd: xe device fd
+ * @cork: pointer to xe_cork structure
+ *
+ * It will destroy vm, exec_queue and free the cork.
+ */
+void xe_cork_destroy(int fd, struct xe_cork *cork)
 {
-	igt_assert(syncobj_wait(cork->fd, &cork->syncobj, 1, INT64_MAX, 0,
-				NULL));
-}
+	igt_assert(cork);
 
-void xe_cork_fini(struct xe_cork *cork)
-{
-	syncobj_destroy(cork->fd, cork->syncobj);
-	xe_exec_queue_destroy(cork->fd, cork->exec_queue);
-	xe_vm_destroy(cork->fd, cork->vm);
-	gem_close(cork->fd, cork->bo);
-}
+	syncobj_destroy(fd, cork->sync[0].handle);
+	syncobj_destroy(fd, cork->sync[1].handle);
+	xe_exec_queue_destroy(fd, cork->exec_queue);
 
-uint32_t xe_cork_sync_handle(struct xe_cork *cork)
-{
-	return cork->syncobj;
+	if (cork->cork_opts.ahnd)
+		intel_allocator_free(cork->cork_opts.ahnd, cork->bo);
+
+	munmap(cork->spin, cork->bo_size);
+	gem_close(fd, cork->bo);
+
+	free(cork);
 }
