@@ -30,9 +30,11 @@
 
 #include "igt_aux.h"
 #include "igt_core.h"
+#include "igt_facts.h"
 #include "igt_taints.h"
 #include "igt_vec.h"
 #include "executor.h"
+#include "kmemleak.h"
 #include "output_strings.h"
 #include "runnercomms.h"
 
@@ -217,11 +219,11 @@ static bool load_ping_config_from_env(void)
 
 /*
  * On some hosts, getting network back up after suspend takes
- * upwards of 10 seconds. 20 seconds should be enough to see
+ * upwards of 10 seconds. 40 seconds should be enough to see
  * if network comes back at all, and hopefully not too long to
  * make external monitoring freak out.
  */
-#define PING_ABORT_DEADLINE 20
+#define PING_ABORT_DEADLINE 40
 
 static bool can_ping(void)
 {
@@ -575,15 +577,47 @@ bool open_output_files(int dirfd, int *fds, bool write)
 	return true;
 }
 
+/**
+ * open_output_files_rdonly:
+ * @dirfd: fd of output directory with err.txt, dmesg.txt and other files
+ * @fds: array for fd's of opened output files
+ *
+ * Tries to open output files in read-only mode and saves file descriptors
+ * in fds array.
+ *
+ * Returns: true if all files opened, false otherwise
+ */
+bool open_output_files_rdonly(int dirfd, int *fds)
+{
+	bool ret = true;
+
+	for (int i = 0; i < _F_LAST; i++)
+		if ((fds[i] = open_for_reading(dirfd, filenames[i])) < 0) {
+			fds[i] = -errno;
+			ret = false; /* Remember failure */
+		}
+
+	return ret;
+}
+
 void close_outputs(int *fds)
 {
 	int i;
 
 	for (i = 0; i < _F_LAST; i++) {
-		close(fds[i]);
+		if (fds[i] >= 0)
+			close(fds[i]);
 	}
 }
 
+const char *get_out_filename(int fid)
+{
+	if (fid >= 0 && fid < _F_LAST)
+		return filenames[fid];
+
+	return "output-filename-index-error";
+}
+
 /* Returns the number of bytes written to disk, or a negative number on error */
 static long dump_dmesg(int kmsgfd, int outfd, ssize_t size)
 {
@@ -2360,11 +2394,22 @@ bool execute(struct execute_state *state,
 	sigset_t sigmask;
 	double time_spent = 0.0;
 	bool status = true;
+	char *last_test = NULL;
 
 	if (state->dry) {
 		outf("Dry run, not executing. Invoke igt_resume if you want to execute.\n");
 		return true;
 	}
+	if (settings->facts)
+		igt_facts_lists_init();
+
+	if (settings->kmemleak)
+		if (!runner_kmemleak_init(NULL)) {
+			errf("Failed to initialize kmemleak. Is kernel support enabled?\n"
+			     "Disabling kmemleak on igt_runner and continuing...\n");
+			settings->kmemleak = false;
+			settings->kmemleak_each = false;
+		}
 
 	if (state->next >= job_list->size) {
 		outf("All tests already executed.\n");
@@ -2492,6 +2537,20 @@ bool execute(struct execute_state *state,
 		int result;
 		bool already_written = false;
 
+		/* Collect facts before running each test */
+		if (settings->facts)
+			igt_facts(last_test);
+
+		if (settings->kmemleak_each)
+			if (!runner_kmemleak(last_test, resdirfd,
+					     settings->kmemleak_each,
+					     settings->sync))
+				errf("Failed to collect kmemleak logs after %s\n",
+				     last_test);
+
+		if (settings->facts || settings->kmemleak_each)
+			last_test = entry_display_name(&job_list->entries[state->next]);
+
 		if (should_die_because_signal(sigfd)) {
 			status = false;
 			goto end;
@@ -2581,6 +2640,15 @@ bool execute(struct execute_state *state,
 		}
 	}
 
+	/* Collect facts after the last test runs */
+	if (settings->facts)
+		igt_facts(last_test);
+
+	if (settings->kmemleak)
+		if (!runner_kmemleak(last_test, resdirfd,
+				     settings->kmemleak_each, settings->sync))
+			errf("Failed to collect kmemleak logs after the last test\n");
+
 	if ((timefd = openat(resdirfd, "endtime.txt", O_CREAT | O_WRONLY | O_EXCL, 0666)) >= 0) {
 		dprintf(timefd, "%f\n", timeofday_double());
 		close(timefd);
