@@ -14,7 +14,6 @@ import glob
 import json
 import os
 import re
-import subprocess
 import sys
 
 MIN_PYTHON = (3, 6)
@@ -250,7 +249,8 @@ class TestList:
                  igt_build_path = None,
                  config_dict = None, sources_path = None,
                  test_tag = "TEST", subtest_tag = "SUBTESTS?",
-                 main_name = "igt", subtest_separator = "@"):
+                 main_name = "igt", planned_name = "planned",
+                 subtest_separator = "@"):
         self.doc = {}
         self.test_number = 0
         self.config = None
@@ -260,12 +260,12 @@ class TestList:
         self.igt_build_path = igt_build_path
         self.level_count = 0
         self.field_list = {}
-        self.testlist = {}
-        self.blocklist = {}
         self.title = None
         self.filters = {}
         self.subtest_separator = subtest_separator
-        self.main_name = main_name
+
+        # Set a default value
+        driver_name = main_name
 
         self.internal_fields =  [ '_summary_', '_arg_', '_subtest_line_' ]
 
@@ -273,8 +273,11 @@ class TestList:
         if bool(config_fname) == bool(config_dict):
             sys.exit("Error: either config filename or config dict shall be used")
 
-        if self.main_name:
-            self.main_name += subtest_separator
+        if main_name:
+            main_name += subtest_separator
+
+        if planned_name:
+            planned_name += subtest_separator
 
         implemented_class = None
 
@@ -290,7 +293,11 @@ class TestList:
             self.config = config_dict
             config_origin = "config dict"
             cfg_path = "./"
-            driver_name = main_name
+
+        if "drivers" in self.config:
+            self.drivers = self.config["drivers"]
+        else:
+            self.drivers = [driver_name]
 
         if sources_path:
             cfg_path = os.path.realpath(sources_path) + "/"
@@ -330,21 +337,33 @@ class TestList:
                     del item["_properties_"]["level"]
                     del item["_properties_"]["sublevel"]
 
-            # Read testlist files if any
-            if "testlists" in item["_properties_"]:
-                testlist = {}
-                for name in item["_properties_"]["testlists"].keys():
-                    self.read_testlist(testlist, name, cfg_path + item["_properties_"]["testlists"][name])
+            update = self.props[field]["_properties_"].get("update-from-file")
+            if update:
+                # Read testlist files if any
+                if "include" in update:
+                    testlist = {}
+                    for value in update["include"]:
+                        for name in value.keys():
+                            match_type = update.get("include-type", "subtest-match")
 
-                item["_properties_"]["testlist"] = testlist
+                            self.read_testlist(match_type, main_name, field,
+                                               testlist, name,
+                                               cfg_path + value[name])
 
-            # Read blocklist files if any
-            if "blocklists" in item["_properties_"]:
-                blocklist = {}
-                for name in item["_properties_"]["blocklists"].keys():
-                    self.read_testlist(blocklist, name, cfg_path + item["_properties_"]["blocklists"][name])
+                    update["include"] = testlist
 
-                item["_properties_"]["blocklist"] = blocklist
+                # Read blocklist files if any
+                if "exclude" in update:
+                    testlist = {}
+                    for value in update["exclude"]:
+                        for name in value.keys():
+                            match_type = update.get("exclude-type", "subtest-match")
+
+                            self.read_testlist(match_type, main_name, field,
+                                               testlist, name,
+                                               cfg_path + value[name])
+
+                    update["exclude"] = testlist
 
         if "_properties_" in self.props:
             del self.props["_properties_"]
@@ -404,12 +423,14 @@ class TestList:
             if fname == '':
                 continue
 
-            self.__add_file_documentation(fname, implemented_class, field_re,
+            self.__add_file_documentation(fname, main_name,
+                                          implemented_class, field_re,
                                           test_tag, subtest_tag, config_origin)
 
         if include_plan:
             for fname in self.plan_filenames:
-                self.__add_file_documentation(fname, planned_class, field_re,
+                self.__add_file_documentation(fname, planned_name,
+                                              planned_class, field_re,
                                               test_tag, subtest_tag, config_origin)
 
     #
@@ -446,20 +467,57 @@ class TestList:
 
             self.__add_field(key, sublevel, hierarchy_level, field[key])
 
-    def read_testlist(self, testlist, name, filename):
-        base = r"^\s*({}[^\s\{}]+)(\S*)\s*(\#.*)?$"
-        regex = re.compile(base.format(self.main_name, self.subtest_separator))
+    def read_testlist(self, match_type, base_name, field, testlist,
+                      name, filename):
 
-        testlist[name] = []
+        """
+        Read a list of tests with a common value from a file.
+
+        Tests on this list are matched only for actual tests, not
+        for planned ones.
+        """
+
+        match_type_regex = set(["regex", "regex-ignorecase"])
+        match_type_str = set(["subtest-match"])
+        match_types = match_type_regex | match_type_str
+
+        if match_type not in match_types:
+            sys.exit(f"Error: invalid match type '{match_type}' for {field}")
+
+        if match_type == "regex":
+            flags = 0
+        elif match_type == "regex-ignorecase":
+            flags = re.IGNORECASE
+
+        base = r"^\s*({}[^\s\{}]+)(\S*)\s*(\#.*)?$"
+        regex = re.compile(base.format(base_name, self.subtest_separator))
+
+        if name not in testlist:
+            testlist[name] = []
         with open(filename, 'r', newline = '', encoding = 'utf8') as fp:
             for line in fp:
                 match = regex.match(line)
-                if match:
-                    test = match.group(1)
-                    subtest = match.group(2)
-                    if not subtest.endswith("$"):
-                        subtest += r"(\@.*)?$"
-                    testlist[name].append(re.compile(f"{test}{subtest}"))
+                if not match:
+                    continue
+
+                test = match.group(1)
+                subtest = match.group(2)
+                test_name = f"{test}{subtest}"
+                if not test_name.endswith("$"):
+                    test_name += r"(\@.*)?$"
+
+                if match_type in match_type_regex:
+                    testlist[name].append(re.compile(test_name, flags))
+                    continue
+
+                subtests = test_name.split(self.subtest_separator)[:3]
+
+                prefix=""
+                for subtest in subtests:
+                    subtest = prefix + subtest
+                    prefix = subtest + self.subtest_separator
+
+                testlist[name].append(re.compile(f"{subtest}(@.*)?"))
 
     def __filter_subtest(self, test, subtest, field_not_found_value):
 
@@ -467,10 +525,10 @@ class TestList:
 
         for filter_field, regex in self.filters.items():
             if filter_field in subtest:
-                if not regex.match(subtest[filter_field]):
+                if not regex.search(subtest[filter_field]):
                     return True
             elif filter_field in test:
-                if not regex.match(test[filter_field]):
+                if not regex.search(test[filter_field]):
                     return True
             else:
                 return field_not_found_value
@@ -479,45 +537,55 @@ class TestList:
         return False
 
     def update_testlist_field(self, subtest_dict):
-        for field in self.props.keys():
+
+        """process include and exclude rules used by test lists read from files"""
+
+        expand = re.compile(r",\s*")
+
+        for field in self.props.keys(): # pylint: disable=C0201,C0206
             if "_properties_" not in self.props[field]:
                 continue
 
-            if "testlist" not in self.props[field]["_properties_"]:
+            update = self.props[field]["_properties_"].get("update-from-file")
+            if not update:
                 continue
 
-            default_value = self.props[field]["_properties_"].get("default-testlist")
+            default_value = update.get("default-if-not-excluded")
+            append_value = update.get("append-value-if-not-excluded")
 
             testname = subtest_dict["_summary_"]
 
             value = subtest_dict.get(field)
             if value:
-                values = set(re.split(",\s*", value))
+                values = set(expand.split(value))
             else:
                 values = set()
 
-            for names, regex_array in self.props[field]["_properties_"]["testlist"].items():
-                name = set(re.split(",\s*", names))
+            if append_value:
+                include_names = set(expand.split(append_value))
+                values.update(include_names)
+
+            for names, regex_array in update.get("include", {}).items():
+                name = set(expand.split(names))
                 for regex in regex_array:
-                    if regex.match(testname):
+                    if regex.fullmatch(testname):
                         values.update(name)
                         break
 
             # If test is at a global blocklist, ignore it
-            set_full_if_empty = True
-            if "blocklist" in self.props[field]["_properties_"]:
-                for names, regex_array in self.props[field]["_properties_"]["testlist"].items():
-                    deleted_names = set(re.split(",\s*", names))
-                    for regex in regex_array:
-                        if regex.match(testname):
-                            if sorted(deleted_names) == sorted(values):
-                                set_full_if_empty = False
-                            values.discard(deleted_names)
+            set_default = True
+            for names, regex_array in update.get("exclude", {}).items():
+                deleted_names = set(expand.split(names))
+                for regex in regex_array:
+                    if regex.fullmatch(testname):
+                        if sorted(deleted_names) == sorted(values):
+                            set_default = False
+                        values -= deleted_names
 
-            if default_value and set_full_if_empty and not values:
-                values = set([default_value])
+            if default_value and set_default and not values:
+                values.update([default_value])
 
-            if values:
+            if values or self.props[field]["_properties_"].get("mandatory"):
                 subtest_dict[field] = ", ".join(sorted(values))
 
     def expand_subtest(self, fname, test_name, test, allow_inherit, with_lines = False, with_subtest_nr = False):
@@ -660,10 +728,11 @@ class TestList:
 
         for test in self.doc:                   # pylint: disable=C0206
             fname = self.doc[test]["File"]
+            base_name = self.doc[test]["_base_name_"]
 
             name = re.sub(r'.*/', '', fname)
             name = re.sub(r'\.[\w+]$', '', name)
-            name = self.main_name + name
+            name = base_name + name
 
             if not subtest_only:
                 test_dict[name] = {}
@@ -696,7 +765,7 @@ class TestList:
     # Output methods
     #
 
-    def print_rest_flat(self, filename = None, return_string = False):
+    def print_rest_flat(self, filename = None, return_string = False): # pylint: disable=R1710
 
         """Print tests and subtests ordered by tests"""
 
@@ -706,10 +775,11 @@ class TestList:
 
         for test in sorted(self.doc.keys()):
             fname = self.doc[test]["File"]
+            base_name = self.doc[test]["_base_name_"]
 
             name = re.sub(r'.*/', '', fname)
             name = re.sub(r'\.[ch]', '', name)
-            name = self.main_name + name
+            name = base_name + name
 
             tmp_subtest = self.expand_subtest(fname, name, test, False)
 
@@ -759,7 +829,7 @@ class TestList:
         else:
             return out
 
-    def get_spreadsheet(self, expand_fields = {}):
+    def get_spreadsheet(self, expand_fields = None):
 
         """
         Return a bidimentional array with the test contents.
@@ -768,6 +838,9 @@ class TestList:
         separate python file that would create a workbook's sheet.
         """
 
+        if not expand_fields:
+            expand_fields = {}
+
         expand_field_name = {}
         sheet = []
         row = 0
@@ -837,13 +910,12 @@ class TestList:
 
         return sheet
 
-    def print_nested_rest(self, filename = None, return_string = False):
+    def print_nested_rest(self, filename = None, return_string = False):  # pylint: disable=R1710
 
         """Print tests and subtests ordered by tests"""
 
         handler = None
         if filename:
-            original_stdout = sys.stdout
             handler = open(filename, "w", encoding='utf8') # pylint: disable=R1732
             sys.stdout = handler
 
@@ -967,6 +1039,9 @@ class TestList:
 
         order = None
 
+        if expand:
+            expand = re.compile(expand)
+
         if sort_field:
             if sort_field.lower() not in self.field_list:
                 sys.exit(f"Field '{sort_field}' is not defined")
@@ -980,51 +1055,51 @@ class TestList:
         subtest_array = []
         for test in sorted(self.doc.keys()):
             fname = self.doc[test]["File"]
+            base_name = self.doc[test]["_base_name_"]
 
             test_name = re.sub(r'.*/', '', fname)
             test_name = re.sub(r'\.[ch]', '', test_name)
-            test_name = self.main_name + test_name
+            test_name = base_name + test_name
 
             subtest_array += self.expand_subtest(fname, test_name, test, True)
 
-        subtest_array.sort(key = lambda x : x.get('_summary_'))
+            subtest_array.sort(key = lambda x : x.get('_summary_'))
 
-        for subtest in subtest_array:
-            if self.__filter_subtest(self.doc[test], subtest, True):
-                continue
+            for subtest in subtest_array:
+                if self.__filter_subtest(self.doc[test], subtest, True):
+                    continue
 
-            if sort_field:
-                if sort_field in subtest:
-                    if expand:
-                        test_list = subtest[sort_field].split(expand)
-                        test_list = [s.strip() for s in test_list]
+                if sort_field:
+                    if sort_field in subtest:
+                        if expand:
+                            test_list = expand.split(subtest[sort_field])
 
-                        for test_elem in test_list:
-                            if test_elem not in subtests:
-                                subtests[test_elem] = []
-                            if order:
-                                subtests[test_elem].append((subtest["_summary_"], test_list))
-                            else:
-                                subtests[test_elem].append(subtest["_summary_"])
+                            for test_elem in test_list:
+                                if test_elem not in subtests:
+                                    subtests[test_elem] = []
+                                if order:
+                                    subtests[test_elem].append((subtest["_summary_"], test_list))
+                                else:
+                                    subtests[test_elem].append(subtest["_summary_"])
+                        else:
+                            if subtest[sort_field] not in subtests:
+                                subtests[subtest[sort_field]] = []
+                                if order:
+                                    subtests[test_elem].append((subtest["_summary_"], [subtest[sort_field]]))
+                                else:
+                                    subtests[subtest[sort_field]].append(subtest["_summary_"])
                     else:
-                        if subtest[sort_field] not in subtests:
-                            subtests[subtest[sort_field]] = []
-                            if order:
-                                subtests[test_elem].append((subtest["_summary_"], [subtest[sort_field]]))
-                            else:
-                                subtests[subtest[sort_field]].append(subtest["_summary_"])
+                        if order:
+                            subtests[test_elem].append((subtest["_summary_"], [subtest[sort_field]]))
+                        else:
+                            subtests[""].append(subtest["_summary_"])
+
                 else:
                     if order:
                         subtests[test_elem].append((subtest["_summary_"], [subtest[sort_field]]))
                     else:
                         subtests[""].append(subtest["_summary_"])
 
-            else:
-                if order:
-                    subtests[test_elem].append((subtest["_summary_"], [subtest[sort_field]]))
-                else:
-                    subtests[""].append(subtest["_summary_"])
-
         if order:
             for group, tests in subtests.items():
                 prefix_tests = []
@@ -1052,8 +1127,13 @@ class TestList:
 
         """ Return a list of tests as reported by --list-subtests """
         tests = []
+        no_binaries = self.get_not_compiled()
         for name in self.filenames:
-            testlist = re.sub(r"\.c$", ".testlist", name.split('/')[-1])
+            binary = re.sub(r"\.c$", "", name.split('/')[-1])
+            if binary in no_binaries:
+                continue
+
+            testlist = binary + ".testlist"
             fname = os.path.join(self.igt_build_path, "tests", testlist)
 
             if not os.path.isfile(fname):
@@ -1066,6 +1146,19 @@ class TestList:
 
         return sorted(tests)
 
+    def get_not_compiled(self):
+
+        """ Return a list of tests which were not compiled """
+        no_binaries = []
+        for name in self.filenames:
+            test_basename = re.sub(r"\.c$", "", name.split('/')[-1])
+            fname = os.path.join(self.igt_build_path, "tests", test_basename)
+
+            if not os.path.isfile(fname):
+                no_binaries.append(test_basename)
+
+        return sorted(no_binaries)
+
     #
     # Validation methods
     #
@@ -1082,14 +1175,12 @@ class TestList:
         mandatory_fields = set()
         for field, item in self.props.items():
             if item["_properties_"].get("mandatory"):
-                    mandatory_fields.add(field)
+                mandatory_fields.add(field)
 
         doc_subtests = set()
 
         args_regex = re.compile(r'\<[^\>]+\>')
 
-        missing_mandatory_fields = False
-
         subtests = self.expand_dictionary(True)
         for subtest, data in sorted(subtests.items()):
             subtest = self.subtest_separator.join(subtest.split(self.subtest_separator)[:3])
@@ -1098,62 +1189,36 @@ class TestList:
             for field in mandatory_fields:
                 if field not in data:
                     print(f"Warning: {subtest} {field} documentation is missing")
-                    missing_mandatory_fields = True
 
             doc_subtests.add(subtest)
 
-        doc_subtests = list(sorted(doc_subtests))
-
         # Get a list of tests from
-        run_subtests = self.get_testlist()
+        run_subtests = set(self.get_testlist())
 
-        # Compare arrays
+        not_compiled = set(self.get_not_compiled())
 
-        run_missing = []
-        doc_uneeded = []
+        # Compare sets
 
-        test_regex = r""
-        for doc_test in doc_subtests:
-            if test_regex != r"":
-                test_regex += r"|"
-            test_regex += doc_test
+        run_missing = list(sorted(run_subtests - doc_subtests))
+        doc_uneeded = list(sorted(doc_subtests - run_subtests))
+        doc_uneeded_build = [t for t in doc_uneeded if t.split('@')[1] not in not_compiled]
 
-        test_regex = re.compile(r'^(' + test_regex + r')$')
-
-        for doc_test in doc_subtests:
-            found = False
-            for run_test in run_subtests:
-                if test_regex.match(run_test):
-                    found = True
-                    break
-            if not found:
-                doc_uneeded.append(doc_test)
-
-        for run_test in run_subtests:
-            found = False
-            if test_regex.match(run_test):
-                found = True
-            if not found:
-                run_missing.append(run_test)
-
-        if doc_uneeded:
-            for test_name in doc_uneeded:
-                print(f"Warning: Documented {test_name} doesn't exist on source files")
-
-        if run_missing:
+        if doc_uneeded_build or run_missing:
             for test_name in run_missing:
-                print(f'Warning: Missing documentation for {test_name}')
-            print(f'Please refer: docs/test_documentation.md for more details')
+                print(f'ERROR: Missing documentation for {test_name}')
 
-        if doc_uneeded or run_missing:
+            for test_name in doc_uneeded_build:
+                print(f'ERROR: Unneeded documentation for {test_name}')
+
+            print('Please refer: docs/test_documentation.md for more details')
             sys.exit(1)
 
     #
     # File handling methods
     #
 
-    def __add_file_documentation(self, fname, implemented_class, field_re,
-                                 test_tag, subtest_tag, config_origin):
+    def __add_file_documentation(self, fname, base_name, implemented_class,
+                                 field_re, test_tag, subtest_tag, config_origin):
 
         """Adds the contents of test/subtest documentation form a file"""
 
@@ -1215,6 +1280,7 @@ class TestList:
                         self.doc[current_test] = {}
                         self.doc[current_test]["_arg_"] = {}
                         self.doc[current_test]["_summary_"] = match.group(1)
+                        self.doc[current_test]["_base_name_"] = base_name
                         self.doc[current_test]["File"] = fname
                         self.doc[current_test]["subtest"] = {}
                         self.doc[current_test]["_subtest_line_"] = {}
@@ -1251,7 +1317,6 @@ class TestList:
                         self.doc[current_test]["subtest"][current_subtest][field] = self.doc[current_test][field]
 
                     self.doc[current_test]["subtest"][current_subtest]["_summary_"] = match.group(1)
-                    self.doc[current_test]["subtest"][current_subtest]["Description"] = ''
                     self.doc[current_test]["_subtest_line_"][current_subtest] = file_ln
 
                     if not arg_ref:
@@ -1390,7 +1455,7 @@ class TestList:
 
 
         # NOTE: currently, it uses a comma for multi-value delimitter
-        test_subtests = self.get_subtests(sort_field, ",", with_order = True)
+        test_subtests = self.get_subtests(sort_field, r",\s*", with_order = True)
 
         if not os.path.exists(directory):
             os.makedirs(directory)
