@@ -67,7 +67,7 @@ static void amdgpu_memory_alloc(amdgpu_device_handle device)
  * AMDGPU_HW_IP_GFX
  * @param device
  */
-static void amdgpu_command_submission_gfx(amdgpu_device_handle device)
+static void amdgpu_command_submission_gfx(amdgpu_device_handle device, bool ce_avails)
 {
 	/* write data using the CP */
 	amdgpu_command_submission_write_linear_helper(device, get_ip_block(device, AMDGPU_HW_IP_GFX), false);
@@ -75,10 +75,14 @@ static void amdgpu_command_submission_gfx(amdgpu_device_handle device)
 	amdgpu_command_submission_const_fill_helper(device, get_ip_block(device, AMDGPU_HW_IP_GFX));
 	/* copy data using the CP */
 	amdgpu_command_submission_copy_linear_helper(device, get_ip_block(device, AMDGPU_HW_IP_GFX));
-	/* separate IB buffers for multi-IB submission */
-	amdgpu_command_submission_gfx_separate_ibs(device);
-	/* shared IB buffer for multi-IB submission */
-	amdgpu_command_submission_gfx_shared_ib(device);
+	if (ce_avails) {
+		/* separate IB buffers for multi-IB submission */
+		amdgpu_command_submission_gfx_separate_ibs(device);
+		/* shared IB buffer for multi-IB submission */
+		amdgpu_command_submission_gfx_shared_ib(device);
+	} else {
+		igt_info("separate and shared IB buffers for multi IB submisison testes are skipped due to GFX11\n");
+	}
 }
 
 /**
@@ -303,7 +307,7 @@ static void amdgpu_userptr_test(amdgpu_device_handle device)
 
 	ip_block->funcs->write_linear(ip_block->funcs, ring_context, &ring_context->pm4_dw);
 
-	 amdgpu_test_exec_cs_helper(device, ip_block->type, ring_context);
+	 amdgpu_test_exec_cs_helper(device, ip_block->type, ring_context, 0);
 
 	r = ip_block->funcs->compare(ip_block->funcs, ring_context, 1);
 	igt_assert_eq(r, 0);
@@ -354,6 +358,20 @@ amdgpu_bo_eviction_test(amdgpu_device_handle device_handle)
 				   0, &vram_info);
 	igt_assert_eq(r, 0);
 
+	r = amdgpu_query_heap_info(device_handle, AMDGPU_GEM_DOMAIN_GTT,
+				   0, &gtt_info);
+	igt_assert_eq(r, 0);
+
+	/* For smaller gtt memory sizes, reduce gtt usage on initialization
+	 * to satisfy eviction vram requirements. Example:
+	 * gtt_info.heap_size 3036569600, gtt_info.max_allocation 2114244608  gtt_info.heap_usage 12845056
+	 * gtt_info.heap_size 2895 mb, gtt_info.max_allocation 2016 mb gtt_info.heap_usage 12 mb
+	 * vram_info.heap_size 2114244608, vram_info.max_allocation 2114244608 vram_info.heap_usage 26951680
+	 * vram_info.heap_size 2016 mb, vram_info.max_allocation 2016 mb vram_info.heap_usage 25 mb
+	 */
+	if (gtt_info.heap_size - gtt_info.max_allocation < vram_info.max_allocation)
+		gtt_info.max_allocation /=3;
+
 	r = amdgpu_bo_alloc_wrap(device_handle, vram_info.max_allocation, 4096,
 				 AMDGPU_GEM_DOMAIN_VRAM, 0, &ring_context->boa_vram[0]);
 	igt_assert_eq(r, 0);
@@ -361,10 +379,6 @@ amdgpu_bo_eviction_test(amdgpu_device_handle device_handle)
 				 AMDGPU_GEM_DOMAIN_VRAM, 0, &ring_context->boa_vram[1]);
 	igt_assert_eq(r, 0);
 
-	r = amdgpu_query_heap_info(device_handle, AMDGPU_GEM_DOMAIN_GTT,
-				   0, &gtt_info);
-	igt_assert_eq(r, 0);
-
 	r = amdgpu_bo_alloc_wrap(device_handle, gtt_info.max_allocation, 4096,
 				 AMDGPU_GEM_DOMAIN_GTT, 0, &ring_context->boa_gtt[0]);
 	igt_assert_eq(r, 0);
@@ -408,7 +422,7 @@ amdgpu_bo_eviction_test(amdgpu_device_handle device_handle)
 			ring_context->resources[2] = ring_context->boa_vram[loop2];
 			ring_context->resources[3] = ring_context->boa_gtt[loop2];
 			ip_block->funcs->copy_linear(ip_block->funcs, ring_context, &ring_context->pm4_dw);
-			amdgpu_test_exec_cs_helper(device_handle, ip_block->type, ring_context);
+			amdgpu_test_exec_cs_helper(device_handle, ip_block->type, ring_context, 0);
 			/* fulfill PM4: test DMA copy linear */
 			r = ip_block->funcs->compare_pattern(ip_block->funcs, ring_context, sdma_write_length);
 			igt_assert_eq(r, 0);
@@ -608,22 +622,11 @@ amdgpu_sync_dependency_test(amdgpu_device_handle device_handle)
 	free_cmd_base(base);
 }
 
-static void
-amdgpu_gfx_dispatch_test_gfx(amdgpu_device_handle device_handle)
-{
-	amdgpu_gfx_dispatch_test(device_handle, AMDGPU_HW_IP_GFX);
-}
-
-static void
-amdgpu_gfx_dispatch_test_compute(amdgpu_device_handle device_handle)
-{
-	amdgpu_gfx_dispatch_test(device_handle, AMDGPU_HW_IP_COMPUTE);
-}
-
 igt_main
 {
 	amdgpu_device_handle device;
 	struct amdgpu_gpu_info gpu_info = {0};
+	struct drm_amdgpu_info_hw_ip info = {0};
 	int fd = -1;
 	int r;
 	bool arr_cap[AMD_IP_MAX] = {0};
@@ -642,6 +645,8 @@ igt_main
 
 		r = amdgpu_query_gpu_info(device, &gpu_info);
 		igt_assert_eq(r, 0);
+		r = amdgpu_query_hw_ip_info(device, AMDGPU_HW_IP_GFX, 0, &info);
+		igt_assert_eq(r, 0);
 		r = setup_amdgpu_ip_blocks(major, minor,  &gpu_info, device);
 		igt_assert_eq(r, 0);
 		asic_rings_readness(device, 1, arr_cap);
@@ -662,7 +667,7 @@ igt_main
 	igt_subtest_with_dynamic("cs-gfx-with-IP-GFX") {
 		if (arr_cap[AMD_IP_GFX]) {
 			igt_dynamic_f("cs-gfx")
-			amdgpu_command_submission_gfx(device);
+			amdgpu_command_submission_gfx(device, info.hw_ip_version_major < 11);
 		}
 	}
 
@@ -676,9 +681,11 @@ igt_main
 
 	igt_describe("Check-GFX-CS-for-multi-fence");
 	igt_subtest_with_dynamic("cs-multi-fence-with-IP-GFX") {
-		if (arr_cap[AMD_IP_GFX]) {
+		if (arr_cap[AMD_IP_GFX] && info.hw_ip_version_major < 11) {
 			igt_dynamic_f("cs-multi-fence")
-		amdgpu_command_submission_multi_fence(device);
+			amdgpu_command_submission_multi_fence(device);
+		} else {
+			igt_info("cs-multi-fence-with-IP-GFX testes are skipped due to GFX11 or no GFX_IP\n");
 		}
 	}
 
@@ -714,22 +721,6 @@ igt_main
 		}
 	}
 
-	igt_describe("Check-dispatch-test-compute-for-each-ring-using-memset-memcpy-shaders-and-validate-after");
-	igt_subtest_with_dynamic("amdgpu-dispatch-test-compute-with-IP-COMPUTE") {
-		if (arr_cap[AMD_IP_COMPUTE]) {
-			igt_dynamic_f("amdgpu-dispatch-test-compute")
-			amdgpu_gfx_dispatch_test_compute(device);
-		}
-	}
-
-	igt_describe("Check-dispatch-test-gfx-for-each-ring-using-memset-memcpy-shaders-and-validate-after");
-	igt_subtest_with_dynamic("amdgpu-dispatch-test-gfx-with-IP-GFX") {
-		if (arr_cap[AMD_IP_GFX]) {
-			igt_dynamic_f("amdgpu-dispatch-test-gfx")
-			amdgpu_gfx_dispatch_test_gfx(device);
-		}
-	}
-
 	igt_fixture {
 		amdgpu_device_deinitialize(device);
 		drm_close_driver(fd);
