@@ -9,85 +9,56 @@
 #include "igt.h"
 #include "igt_aux.h"
 #include "intel_allocator.h"
+#include "intel_pat.h"
 #include "xe/xe_ioctl.h"
 #include "xe/xe_query.h"
 
 /**
  * TEST: api intel allocator
- * Category: Infrastructure
- * Functionality: virtual address ranges
- * Run type: FULL
+ * Description: checking the virtual address ranges
+ * Category: Core
+ * Mega feature: General Core features
  * Sub-category: IGT Lib
+ * Functionality: virtual address
  * Test category: GEM_Legacy
+ * Feature: igt_core
  *
  * SUBTEST: alloc-simple
- * Description: checking the virtual address ranges
- * Feature: igt_core
  *
  * SUBTEST: default-alignment
- * Description:
- *   For simple allocator check does default alignment is properly handled in open and alloc
- *   functions
- *   checking the virtual address ranges
- * Feature: igt_core
+ * Description: For simple allocator check does default alignment is properly
+ *		handled in open and alloc functions checking the virtual
+ *		address ranges
  *
  * SUBTEST: execbuf-with-allocator
- * Description: checking the virtual address ranges
- * Feature: igt_core
  *
  * SUBTEST: fork-simple-once
- * Description: checking the virtual address ranges
- * Feature: igt_core
  *
  * SUBTEST: fork-simple-stress
- * Description: checking the virtual address ranges
- * Feature: igt_core
  *
  * SUBTEST: fork-simple-stress-signal
- * Description: checking the virtual address ranges
- * Feature: igt_core
  *
  * SUBTEST: gem-pool
- * Description:
- *   Verifies creating and executing bb from gem pool
- *   checking the virtual address ranges
- * Feature: igt_core
+ * Description: Verifies creating and executing bb from gem pool checking
+ *		the virtual address ranges
  *
  * SUBTEST: open-vm
- * Description: checking the virtual address ranges
- * Feature: igt_core
  *
  * SUBTEST: reloc-allocator
- * Description: checking the virtual address ranges
- * Feature: igt_core
  *
  * SUBTEST: reopen
- * Description: checking the virtual address ranges
- * Feature: igt_core
  *
  * SUBTEST: reopen-fork
- * Description: checking the virtual address ranges
- * Feature: igt_core
  *
  * SUBTEST: reserve-simple
- * Description: checking the virtual address ranges
- * Feature: igt_core
  *
  * SUBTEST: simple-allocator
- * Description: checking the virtual address ranges
- * Feature: igt_core
  *
  * SUBTEST: standalone
- * Description: checking the virtual address ranges
- * Feature: igt_core
  *
  * SUBTEST: two-level-inception
- * Description: checking the virtual address ranges
- * Feature: igt_core
  *
  * SUBTEST: two-level-inception-interruptible
- * Description: checking the virtual address ranges
- * Feature: igt_core
  */
 
 #define OBJ_SIZE 1024
@@ -131,7 +102,8 @@ static void alloc_simple(int fd)
 
 	intel_allocator_get_address_range(ahnd, &start, &end);
 	offset0 = intel_allocator_alloc(ahnd, 1, end - start, 0);
-	offset1 = __intel_allocator_alloc(ahnd, 2, 4096, 0, ALLOC_STRATEGY_NONE);
+	offset1 = __intel_allocator_alloc(ahnd, 2, 4096, 0, DEFAULT_PAT_INDEX,
+					  ALLOC_STRATEGY_NONE);
 	igt_assert(offset1 == ALLOC_INVALID_ADDRESS);
 	intel_allocator_free(ahnd, 1);
 
@@ -204,7 +176,7 @@ static void reserve(int fd, uint8_t type)
 static void default_alignment(int fd)
 {
 	struct test_obj obj[3];
-	uint64_t ahnd, default_alignment = 0x4000;
+	uint64_t ahnd, default_alignment = SZ_4K;
 
 	ahnd = intel_allocator_open_full(fd, 0, 0, 0, INTEL_ALLOCATOR_SIMPLE,
 					 ALLOC_STRATEGY_LOW_TO_HIGH,
@@ -212,13 +184,13 @@ static void default_alignment(int fd)
 
 	for (int i = 0; i < ARRAY_SIZE(obj); i++) {
 		obj[i].handle = gem_handle_gen();
-		obj[i].offset = intel_allocator_alloc(ahnd, obj[i].handle, 4096,
-				i == 2 ? 4096 : 0);
+		obj[i].offset = intel_allocator_alloc(ahnd, obj[i].handle,
+						      SZ_4K, i == 2 ? 0 : SZ_16K);
 		igt_debug("obj[%d].offset: %llx, handle: %u\n", i,
 			 (long long) obj[i].offset, obj[i].handle);
 	}
 
-	igt_assert_eq(obj[1].offset - obj[0].offset, default_alignment);
+	igt_assert_eq(obj[1].offset - obj[0].offset, SZ_16K);
 	/* obj[2] should be between obj[0] and obj[1] */
 	igt_assert(obj[0].offset < obj[2].offset);
 	igt_assert(obj[2].offset < obj[1].offset);
@@ -468,8 +440,8 @@ static void __simple_allocs(int fd)
 
 		size = (rand() % 4 + 1) * 0x1000;
 		if (is_xe)
-			handles[i] = xe_bo_create_flags(fd, 0, size,
-							system_memory(fd));
+			handles[i] = xe_bo_create(fd, 0, size,
+						  system_memory(fd), 0);
 		else
 			handles[i] = gem_create(fd, size);
 
@@ -733,7 +705,7 @@ static void execbuf_with_allocator(int fd)
 		gem_close(fd, object[i].handle);
 	}
 
-	igt_assert(copied == magic);
+	igt_assert_eq_u32(copied, magic);
 	igt_assert(intel_allocator_close(ahnd) == true);
 }
 
@@ -804,13 +776,13 @@ static void gem_pool(int i915)
 	bb[0] = single_exec_from_pool(i915, ahnd, 4096);
 	gem_sync(i915, bb[0]);
 	bb[1] = single_exec_from_pool(i915, ahnd, 4096);
-	igt_assert(bb[0] == bb[1]);
+	igt_assert_eq_u32(bb[0], bb[1]);
 
 	bb[2] = single_exec_from_pool(i915, ahnd, 8192);
 	gem_sync(i915, bb[2]);
 	bb[3] = single_exec_from_pool(i915, ahnd, 8192);
-	igt_assert(bb[2] == bb[3]);
-	igt_assert(bb[0] != bb[2]);
+	igt_assert_eq_u32(bb[2], bb[3]);
+	igt_assert_neq_u32(bb[0], bb[2]);
 
 	spin = igt_spin_new(i915,
 			    .ahnd = ahnd,
@@ -820,8 +792,8 @@ static void gem_pool(int i915)
 	bb[2] = single_exec_from_pool(i915, ahnd, 8192);
 	bb[3] = single_exec_from_pool(i915, ahnd, 8192);
 	igt_spin_free(i915, spin);
-	igt_assert(bb[0] != bb[1]);
-	igt_assert(bb[2] != bb[3]);
+	igt_assert_neq_u32(bb[0], bb[1]);
+	igt_assert_neq_u32(bb[2], bb[3]);
 
 	put_ahnd(ahnd);
 
