@@ -43,210 +43,72 @@
 #include "intel_blt.h"
 /**
  * TEST: api intel bb
- * Description: intel_bb API check.
- * Run type: FULL
+ * Description: checking dummy buffer creation & submission
+ * Category: Core
+ * Mega feature: General Core features
+ * Sub-category: IGT Lib
+ * Functionality: dummy buffer creation & submission
+ * Feature: igt_core
+ * Test category: GEM_Legacy
  *
  * SUBTEST: add-remove-objects
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: bb-with-allocator
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: blit-noreloc-keep-cache
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: blit-noreloc-purge-cache
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: blit-reloc-keep-cache
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: blit-reloc-purge-cache
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: crc32
  * Description: Compare cpu and gpu crc32 sums on input object
- * Feature: igt_core
- * Test category: GEM_Legacy
  *
  * SUBTEST: delta-check
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: destroy-bb
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: full-batch
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: intel-bb-blit-none
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: intel-bb-blit-x
- * Feature: igt_core
  *
  * SUBTEST: intel-bb-blit-y
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: lot-of-buffers
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: misplaced-blitter
- * Category: Infrastructure
- * Description:
- *   Execute intel_bb with set of engines provided by userspace
- *   checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
+ * Description: Execute intel_bb with set of engines provided by userspace
+ *		checking dummy buffer creation & submission
  *
  * SUBTEST: object-noreloc-keep-cache-simple
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: object-noreloc-purge-cache-simple
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: object-reloc-keep-cache
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: object-reloc-purge-cache
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: offset-control
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: purge-bb
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: render
- * Feature: igt_core
  *
  * SUBTEST: render-ccs
- * Feature: igt_core
  *
  * SUBTEST: reset-bb
- * Category: Infrastructure
- * Description:
- *   Ensure reset is possible on fresh bb
- *   checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
+ * Description: Ensure reset is possible on fresh bb checking dummy buffer
+ *		creation & submission
  *
  * SUBTEST: reset-flags
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: simple-bb
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  *
  * SUBTEST: simple-bb-ctx
- * Category: Infrastructure
- * Description: checking dummy buffer creation & submission
- * Feature: igt_core
- * Functionality: dummy buffer creation & submission
- * Sub-category: IGT Lib
- * Test category: GEM_Legacy
  */
 
 #define PAGE_SIZE 4096
@@ -279,15 +141,6 @@ static bool buf_info = false;
 static bool print_base64 = false;
 static int crc_n = 19;
 
-static void *alloc_aligned(uint64_t size)
-{
-	void *p;
-
-	igt_assert_eq(posix_memalign(&p, 16, size), 0);
-
-	return p;
-}
-
 static void fill_buf(struct intel_buf *buf, uint8_t color)
 {
 	uint8_t *ptr;
@@ -996,72 +849,20 @@ static void scratch_buf_init(struct buf_ops *bops,
 			     uint32_t req_tiling,
 			     enum i915_compression compression)
 {
+	int fd = buf_ops_get_fd(bops);
 	int bpp = 32;
+	uint32_t region = REGION_SMEM;
 
-	intel_buf_init(bops, buf, width, height, bpp, 0,
-		       req_tiling, compression);
+	if (compression && gem_has_lmem(fd))
+		region = REGION_LMEM(0);
+
+	intel_buf_init_in_region(bops, buf, width, height, bpp, 0,
+				 req_tiling, compression, region);
 
 	igt_assert(intel_buf_width(buf) == width);
 	igt_assert(intel_buf_height(buf) == height);
 }
 
-static void scratch_buf_draw_pattern(struct buf_ops *bops,
-				     struct intel_buf *buf,
-				     int x, int y, int w, int h,
-				     int cx, int cy, int cw, int ch,
-				     bool use_alternate_colors)
-{
-	cairo_surface_t *surface;
-	cairo_pattern_t *pat;
-	cairo_t *cr;
-	void *linear;
-
-	linear = alloc_aligned(buf->surface[0].size);
-
-	surface = cairo_image_surface_create_for_data(linear,
-						      CAIRO_FORMAT_RGB24,
-						      intel_buf_width(buf),
-						      intel_buf_height(buf),
-						      buf->surface[0].stride);
-
-	cr = cairo_create(surface);
-
-	cairo_rectangle(cr, cx, cy, cw, ch);
-	cairo_clip(cr);
-
-	pat = cairo_pattern_create_mesh();
-	cairo_mesh_pattern_begin_patch(pat);
-	cairo_mesh_pattern_move_to(pat, x,   y);
-	cairo_mesh_pattern_line_to(pat, x+w, y);
-	cairo_mesh_pattern_line_to(pat, x+w, y+h);
-	cairo_mesh_pattern_line_to(pat, x,   y+h);
-	if (use_alternate_colors) {
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 0, 0.0, 1.0, 1.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 1, 1.0, 0.0, 1.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 2, 1.0, 1.0, 0.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 3, 0.0, 0.0, 0.0);
-	} else {
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 0, 1.0, 0.0, 0.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 1, 0.0, 1.0, 0.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 2, 0.0, 0.0, 1.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 3, 1.0, 1.0, 1.0);
-	}
-	cairo_mesh_pattern_end_patch(pat);
-
-	cairo_rectangle(cr, x, y, w, h);
-	cairo_set_source(cr, pat);
-	cairo_fill(cr);
-	cairo_pattern_destroy(pat);
-
-	cairo_destroy(cr);
-
-	cairo_surface_destroy(surface);
-
-	linear_to_intel_buf(bops, buf, linear);
-
-	free(linear);
-}
-
 #define GROUP_SIZE 4096
 static int compare_detail(const uint32_t *ptr1, uint32_t *ptr2,
 			  uint32_t size)
@@ -1192,9 +993,9 @@ static int __do_intel_bb_blit(struct buf_ops *bops, uint32_t tiling)
 		intel_buf_print(&dst);
 	}
 
-	scratch_buf_draw_pattern(bops, &src,
-				 0, 0, width, height,
-				 0, 0, width, height, 0);
+	intel_buf_draw_pattern(bops, &src,
+			       0, 0, width, height,
+			       0, 0, width, height, 0);
 
 	intel_bb_blt_copy(ibb,
 			  &src, 0, 0, src.surface[0].stride,
@@ -1223,11 +1024,11 @@ static int __do_intel_bb_blit(struct buf_ops *bops, uint32_t tiling)
 
 	/* We'll fail on src <-> final compare so just warn */
 	if (tiling == I915_TILING_NONE) {
-		if (compare_bufs(&src, &dst, false) > 0)
-			igt_warn("none->none blit failed!");
+		igt_warn_on_f(compare_bufs(&src, &dst, false) > 0,
+			      "none->none blit failed!");
 	} else {
-		if (compare_bufs(&src, &dst, false) == 0)
-			igt_warn("none->tiled blit failed!");
+		igt_warn_on_f(compare_bufs(&src, &dst, false) == 0,
+			      "none->tiled blit failed!");
 	}
 
 	fails = compare_bufs(&src, &final, true);
@@ -1534,9 +1335,9 @@ static int render(struct buf_ops *bops, uint32_t tiling, bool do_reloc,
 	scratch_buf_init(bops, &final, width, height, I915_TILING_NONE,
 			 I915_COMPRESSION_NONE);
 
-	scratch_buf_draw_pattern(bops, &src,
-				 0, 0, width, height,
-				 0, 0, width, height, 0);
+	intel_buf_draw_pattern(bops, &src,
+			       0, 0, width, height,
+			       0, 0, width, height, 0);
 
 	render_copy = igt_get_render_copyfunc(devid);
 	igt_assert(render_copy);
@@ -1566,11 +1367,11 @@ static int render(struct buf_ops *bops, uint32_t tiling, bool do_reloc,
 
 	/* We'll fail on src <-> final compare so just warn */
 	if (tiling == I915_TILING_NONE) {
-		if (compare_bufs(&src, &dst, false) > 0)
-			igt_warn("%s: none->none failed!\n", __func__);
+		igt_warn_on_f(compare_bufs(&src, &dst, false) > 0,
+			      "%s: none->none failed!\n", __func__);
 	} else {
-		if (compare_bufs(&src, &dst, false) == 0)
-			igt_warn("%s: none->tiled failed!\n", __func__);
+		igt_warn_on_f(compare_bufs(&src, &dst, false) == 0,
+			      "%s: none->tiled failed!\n", __func__);
 	}
 
 	fails = compare_bufs(&src, &final, true);
@@ -1623,6 +1424,7 @@ static void render_ccs(struct buf_ops *bops)
 	uint32_t compressed = 0;
 	uint32_t devid = intel_get_drm_devid(i915);
 	igt_render_copyfunc_t render_copy = NULL;
+	int tiling = IS_DG2(devid) ? I915_TILING_4 : I915_TILING_Y;
 
 	ibb = intel_bb_create(i915, PAGE_SIZE);
 	if (debug_bb)
@@ -1630,9 +1432,9 @@ static void render_ccs(struct buf_ops *bops)
 
 	scratch_buf_init(bops, &src, width, height, I915_TILING_NONE,
 			 I915_COMPRESSION_NONE);
-	scratch_buf_init(bops, &dst, width, height, I915_TILING_Y,
+	scratch_buf_init(bops, &dst, width, height, tiling,
 			 I915_COMPRESSION_RENDER);
-	scratch_buf_init(bops, &dst2, width, height, I915_TILING_Y,
+	scratch_buf_init(bops, &dst2, width, height, tiling,
 			 I915_COMPRESSION_RENDER);
 	scratch_buf_init(bops, &final, width, height, I915_TILING_NONE,
 			 I915_COMPRESSION_NONE);
@@ -1640,9 +1442,9 @@ static void render_ccs(struct buf_ops *bops)
 	render_copy = igt_get_render_copyfunc(devid);
 	igt_assert(render_copy);
 
-	scratch_buf_draw_pattern(bops, &src,
-				 0, 0, width, height,
-				 0, 0, width, height, 0);
+	intel_buf_draw_pattern(bops, &src,
+			       0, 0, width, height,
+			       0, 0, width, height, 0);
 
 	render_copy(ibb,
 		    &src,
@@ -1663,21 +1465,25 @@ static void render_ccs(struct buf_ops *bops)
 		    0, 0);
 
 	intel_bb_sync(ibb);
-
-	fails = compare_bufs(&src, &final, true);
-	compressed = count_compressed(ibb->gen, &dst);
-
 	intel_bb_destroy(ibb);
 
-	igt_debug("fails: %u, compressed: %u\n", fails, compressed);
+	fails = compare_bufs(&src, &final, true);
+	if (!HAS_FLATCCS(devid)) {
+		compressed = count_compressed(ibb->gen, &dst);
+		igt_debug("fails: %u, compressed: %u\n", fails, compressed);
+	} else {
+		igt_debug("fails: %u\n", fails);
+	}
 
 	if (write_png) {
-		intel_buf_write_to_png(&src, "render-ccs-src.png");
-		intel_buf_write_to_png(&dst, "render-ccs-dst.png");
-		intel_buf_write_to_png(&dst2, "render-ccs-dst2.png");
-		intel_buf_write_aux_to_png(&dst, "render-ccs-dst-aux.png");
-		intel_buf_write_aux_to_png(&dst2, "render-ccs-dst2-aux.png");
-		intel_buf_write_to_png(&final, "render-ccs-final.png");
+		intel_buf_raw_write_to_png(&src, "render-ccs-src.png");
+		intel_buf_raw_write_to_png(&dst, "render-ccs-dst.png");
+		intel_buf_raw_write_to_png(&dst2, "render-ccs-dst2.png");
+		intel_buf_raw_write_to_png(&final, "render-ccs-final.png");
+		if (!HAS_FLATCCS(devid)) {
+			intel_buf_write_aux_to_png(&dst, "render-ccs-dst-aux.png");
+			intel_buf_write_aux_to_png(&dst2, "render-ccs-dst2-aux.png");
+		}
 	}
 
 	intel_buf_close(bops, &src);
@@ -1729,7 +1535,7 @@ static void test_crc32(int i915, const intel_ctx_t *ctx,
 
 		put_offset(ahnd, data);
 		gem_close(i915, data);
-		igt_assert(cpu_crc == gpu_crc);
+		igt_assert_eq_u32(cpu_crc, gpu_crc);
 	}
 
 	put_ahnd(ahnd);
