@@ -30,10 +30,10 @@
 #include <fcntl.h>
 #include <inttypes.h>
 #include <errno.h>
+#include <signal.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
-#include <sys/signal.h>
 #include <time.h>
 #include <sched.h>
 
@@ -50,7 +50,10 @@
 #include "i915/gem_create.h"
 /**
  * TEST: gem exec latency
- * Run type: FULL
+ * Category: Core
+ * Mega feature: General Core features
+ * Sub-category: CMD submission
+ * Functionality: execbuf
  *
  * SUBTEST: context-preempt
  *
@@ -814,7 +817,7 @@ static void context_switch(int i915, const intel_ctx_t *ctx,
 		       I915_GEM_DOMAIN_WC, I915_GEM_DOMAIN_WC);
 
 	cs = bbe;
-	*cs++ = 0x5 << 23;
+	*cs++ = MI_ARB_CHECK;
 	*cs++ = 0x24 << 23 | 2; /* SRM */
 	*cs++ = mmio_base + 0x358; /* TIMESTAMP */
 	reloc[0].target_handle = obj[0].handle;
@@ -869,7 +872,7 @@ static void context_switch(int i915, const intel_ctx_t *ctx,
 
 		*bbe = 0xa << 23;
 		gem_sync(i915, obj[1].handle);
-		*bbe = 0x5 << 23;
+		*bbe = MI_ARB_CHECK;
 
 		v = results[0];
 		igt_mean_add(&mean, (results[1] - results[2]) * rcs_clock);
@@ -946,7 +949,7 @@ igt_main
 		if (ring_size > 1024)
 			ring_size = 1024;
 
-		intel_register_access_init(&mmio_data, igt_device_get_pci_device(device), false, device);
+		intel_register_access_init(&mmio_data, igt_device_get_pci_device(device), false);
 		rcs_clock = clockrate(device, 0x2000 + TIMESTAMP);
 		igt_info("RCS timestamp clock: %.0fKHz, %.1fns\n",
 			 rcs_clock / 1e3, 1e9 / rcs_clock);
