@@ -27,169 +27,75 @@
 /**
  * TEST: gem lmem swapping
  * Description: Exercise local memory swapping.
+ * Category: Core
+ * Mega feature: General Core features
+ * Sub-category: Memory management tests
+ * Functionality: local memory
+ * Feature: local_memory
+ * Test category: GEM_Legacy
  *
  * SUBTEST: basic
- * Category: Server
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Functionality: buffer management
- * Run type: BAT
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: heavy-multi
- * Category: Server
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Functionality: buffer management
- * Run type: FULL
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: heavy-random
- * Category: Server
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Functionality: buffer management
- * Run type: FULL
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: heavy-verify-multi
- * Category: Server
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Functionality: buffer management
- * Run type: FULL
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: heavy-verify-multi-ccs
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Run type: FULL
  *
  * SUBTEST: heavy-verify-random
- * Category: Server
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Functionality: buffer management
- * Run type: FULL
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: heavy-verify-random-ccs
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Run type: FULL
  *
  * SUBTEST: massive
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Run type: FULL
- * Test category: GEM_Legacy
  *
  * SUBTEST: massive-random
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Run type: FULL
- * Test category: GEM_Legacy
  *
  * SUBTEST: parallel-multi
- * Category: Server
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Functionality: buffer management
- * Run type: FULL
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: parallel-random
- * Category: Server
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Functionality: buffer management
- * Run type: FULL
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: parallel-random-engines
- * Category: Server
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Functionality: buffer management
- * Run type: BAT
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: parallel-random-verify
- * Category: Server
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Functionality: buffer management
- * Run type: FULL
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: parallel-random-verify-ccs
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Run type: FULL
  *
  * SUBTEST: random
- * Category: Server
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Functionality: buffer management
- * Run type: FULL
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: random-engines
- * Category: Server
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Functionality: buffer management
- * Run type: BAT
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: smem-oom
- * Category: Server
  * Description: Exercise local memory swapping during exhausting system memory
- * Feature: local_memory
- * Functionality: buffer management
- * Run type: FULL
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: verify
- * Category: Server
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Functionality: buffer management
- * Run type: FULL
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: verify-ccs
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Run type: FULL
  *
  * SUBTEST: verify-random
- * Category: Server
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Functionality: buffer management
- * Run type: BAT
- * Sub-category: i915
- * Test category: GEM_Legacy
  *
  * SUBTEST: verify-random-ccs
  * Description: Exercise local memory swapping to system memory
- * Feature: local_memory
- * Run type: FULL
  */
 
 IGT_TEST_DESCRIPTION("Exercise local memory swapping.");
@@ -198,7 +104,6 @@ IGT_TEST_DESCRIPTION("Exercise local memory swapping.");
 #define round_up(x, y) ((((x) - 1) | __round_mask(x, y)) + 1)
 
 #define PAGE_SIZE  (1ULL << 12)
-#define SZ_64K	   (16 * PAGE_SIZE)
 
 static const char *readable_unit(uint64_t size)
 {
@@ -284,6 +189,7 @@ init_object_ccs(int i915, struct object *obj, struct blt_copy_object *tmp,
 {
 	struct blt_block_copy_data_ext ext = {}, *pext = &ext;
 	const struct intel_execution_engine2 *e;
+	struct intel_execution_engine2 ec;
 	struct blt_copy_data blt = {};
 	struct blt_copy_batch *cmd;
 	uint64_t size = 4096;
@@ -291,8 +197,10 @@ init_object_ccs(int i915, struct object *obj, struct blt_copy_object *tmp,
 
 	obj->seed = seed;
 	for_each_ctx_engine(i915, ctx, e) {
-		if (gem_engine_can_block_copy(i915, e))
+		if (gem_engine_can_block_copy(i915, e)) {
+			ec = *e;
 			break;
+		}
 	}
 	igt_assert_f(e, "Ctx don't have blt engine\n");
 
@@ -319,7 +227,7 @@ init_object_ccs(int i915, struct object *obj, struct blt_copy_object *tmp,
 	blt_set_object_ext(&ext.dst, 0, obj->blt_obj->x2, obj->blt_obj->y2,
 			   SURFACE_TYPE_2D);
 
-	blt_block_copy(i915, ctx, e, ahnd, &blt, pext);
+	blt_block_copy(i915, ctx, &ec, ahnd, &blt, pext);
 	free(cmd);
 }
 
@@ -336,7 +244,7 @@ verify_object(int i915, const struct object *obj,  unsigned int flags)
 		uint32_t val = obj->seed + x;
 
 		igt_assert_f(buf[x] == val,
-			     "Object mismatch at offset %zu - found %08x, expected %08x; difference:%08x!\n",
+			     "Object mismatch at offset %lu - found %08x, expected %08x; difference:%08x!\n",
 			     x * sizeof(*buf), buf[x], val, buf[x] ^ val);
 	}
 
@@ -350,14 +258,17 @@ verify_object_ccs(int i915, const struct object *obj,
 {
 	struct blt_block_copy_data_ext ext = {}, *pext = &ext;
 	const struct intel_execution_engine2 *e;
+	struct intel_execution_engine2 ec;
 	struct blt_copy_data blt = {};
 	struct blt_copy_batch *cmd;
 	uint64_t size = 4096;
 	unsigned long j, val, *buf;
 
 	for_each_ctx_engine(i915, ctx, e) {
-		if (gem_engine_can_block_copy(i915, e))
+		if (gem_engine_can_block_copy(i915, e)) {
+			ec = *e;
 			break;
+		}
 	}
 	igt_assert_f(e, "Ctx don't have blt engine\n");
 
@@ -379,7 +290,7 @@ verify_object_ccs(int i915, const struct object *obj,
 	blt_set_object_ext(&ext.src, 0, obj->blt_obj->x2, obj->blt_obj->y2,
 			   SURFACE_TYPE_2D);
 	blt_set_object_ext(&ext.dst, 0, tmp->x2, tmp->y2, SURFACE_TYPE_2D);
-	blt_block_copy(i915, ctx, e, ahnd, &blt, pext);
+	blt_block_copy(i915, ctx, &ec, ahnd, &blt, pext);
 
 	buf = gem_mmap__device_coherent(i915, tmp->handle, 0,
 					obj->size, PROT_READ);
@@ -486,7 +397,7 @@ static void __do_evict(int i915,
 				   INTEL_MEMORY_REGION_ID(I915_SYSTEM_MEMORY, 0));
 		blt_set_object(tmp, tmp->handle, params->size.max,
 			       INTEL_MEMORY_REGION_ID(I915_SYSTEM_MEMORY, 0),
-			       intel_get_uc_mocs(i915), T_LINEAR,
+			       intel_get_uc_mocs_index(i915), 0, T_LINEAR,
 			       COMPRESSION_DISABLED, COMPRESSION_TYPE_3D);
 		blt_set_geom(tmp, stride, 0, 0, width, height, 0, 0);
 	}
@@ -516,7 +427,7 @@ static void __do_evict(int i915,
 			obj->blt_obj = calloc(1, sizeof(*obj->blt_obj));
 			igt_assert(obj->blt_obj);
 			blt_set_object(obj->blt_obj, obj->handle, obj->size, region_id,
-				       intel_get_uc_mocs(i915), T_LINEAR,
+				       intel_get_uc_mocs_index(i915), 0, T_LINEAR,
 				       COMPRESSION_ENABLED, COMPRESSION_TYPE_3D);
 			blt_set_geom(obj->blt_obj, stride, 0, 0, width, height, 0, 0);
 			init_object_ccs(i915, obj, tmp, rand(), blt_ctx,
@@ -531,7 +442,7 @@ static void __do_evict(int i915,
 		}
 	}
 
-	igt_debug("obj size min/max=%lu %s/%lu %s, count=%u, seed: %u\n",
+	igt_debug("obj size min/max=%"PRIu64" %s/%"PRIu64" %s, count=%u, seed: %u\n",
 		  readable_size(params->size.min), readable_unit(params->size.min),
 		  readable_size(params->size.max), readable_unit(params->size.max),
 		  params->count, seed);
@@ -680,7 +591,7 @@ static void fill_params(int i915, struct params *params,
 		(region->probed_size >> 20);
 	igt_info("Memory: system-total %dMiB, lmem-region %lldMiB, usage-limit %dMiB\n",
 		 swap_mb, (region->probed_size >> 20), params->mem_limit);
-	igt_info("Using %u thread(s), %u loop(s), %u objects of %lu %s - %lu %s, seed: %u, oom: %s\n",
+	igt_info("Using %u thread(s), %u loop(s), %u objects of %"PRIu64" %s - %"PRIu64" %s, seed: %u, oom: %s\n",
 		 params->flags & TEST_PARALLEL ? nproc : 1,
 		 params->loops,
 		 params->count,
