@@ -51,8 +51,11 @@
 /**
  * TEST: gem render copy
  * Description: Basic test for the render_copy() function.
+ * Category: Core
+ * Mega feature: Render
+ * Sub-category: Render copy
+ * Functionality: render_copy
  * Feature: mapping
- * Run type: FULL
  *
  * SUBTEST: linear
  * Description: Test render_copy() from a mixed-tiled to a linear buffer.
@@ -237,62 +240,6 @@ static void *linear_copy_ccs(data_t *data, struct intel_buf *buf)
 	return ccs_data;
 }
 
-static void scratch_buf_draw_pattern(data_t *data, struct intel_buf *buf,
-				     int x, int y, int w, int h,
-				     int cx, int cy, int cw, int ch,
-				     bool use_alternate_colors)
-{
-	cairo_surface_t *surface;
-	cairo_pattern_t *pat;
-	cairo_t *cr;
-	void *linear;
-
-	linear = alloc_aligned(buf->surface[0].size);
-
-	surface = cairo_image_surface_create_for_data(linear,
-						      CAIRO_FORMAT_RGB24,
-						      intel_buf_width(buf),
-						      intel_buf_height(buf),
-						      buf->surface[0].stride);
-
-	cr = cairo_create(surface);
-
-	cairo_rectangle(cr, cx, cy, cw, ch);
-	cairo_clip(cr);
-
-	pat = cairo_pattern_create_mesh();
-	cairo_mesh_pattern_begin_patch(pat);
-	cairo_mesh_pattern_move_to(pat, x,   y);
-	cairo_mesh_pattern_line_to(pat, x+w, y);
-	cairo_mesh_pattern_line_to(pat, x+w, y+h);
-	cairo_mesh_pattern_line_to(pat, x,   y+h);
-	if (use_alternate_colors) {
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 0, 0.0, 1.0, 1.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 1, 1.0, 0.0, 1.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 2, 1.0, 1.0, 0.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 3, 0.0, 0.0, 0.0);
-	} else {
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 0, 1.0, 0.0, 0.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 1, 0.0, 1.0, 0.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 2, 0.0, 0.0, 1.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 3, 1.0, 1.0, 1.0);
-	}
-	cairo_mesh_pattern_end_patch(pat);
-
-	cairo_rectangle(cr, x, y, w, h);
-	cairo_set_source(cr, pat);
-	cairo_fill(cr);
-	cairo_pattern_destroy(pat);
-
-	cairo_destroy(cr);
-
-	cairo_surface_destroy(surface);
-
-	linear_to_intel_buf(data->bops, buf, linear);
-
-	free(linear);
-}
-
 static void
 scratch_buf_copy(data_t *data,
 		 struct intel_buf *src, int sx, int sy, int w, int h,
@@ -541,13 +488,13 @@ static void test(data_t *data, uint32_t src_tiling, uint32_t dst_tiling,
 			 I915_COMPRESSION_NONE, region);
 
 	for (int i = 0; i < num_src; i++)
-		scratch_buf_draw_pattern(data, &src[i].buf,
-					 0, 0, WIDTH, HEIGHT,
-					 0, 0, WIDTH, HEIGHT, (i % 2));
+		intel_buf_draw_pattern(data->bops, &src[i].buf,
+				       0, 0, WIDTH, HEIGHT,
+				       0, 0, WIDTH, HEIGHT, (i % 2));
 
-	scratch_buf_draw_pattern(data, &dst,
-				 0, 0, WIDTH, HEIGHT,
-				 0, 0, WIDTH, HEIGHT, false);
+	intel_buf_draw_pattern(data->bops, &dst,
+			       0, 0, WIDTH, HEIGHT,
+			       0, 0, WIDTH, HEIGHT, false);
 
 	scratch_buf_copy(data,
 			 &dst, 0, 0, WIDTH, HEIGHT,
