@@ -28,9 +28,7 @@
  * Description: Tests to verify PSR2 selective fetch by sending multiple damaged
  *              areas with and without fbc
  * Driver requirement: i915, xe
- * Functionality: plane, psr2, selective_fetch
  * Mega feature: PSR
- * Test category: functionality test
  */
 
 #include "igt.h"
@@ -52,18 +50,15 @@
  * Mega feature: Panel Replay
  * Description: Test that selective fetch works on moving %arg[1] plane %arg[2]
  *              visible area (no update)
- * Functionality: plane, pr, selective_fetch
  *
  * SUBTEST: fbc-psr2-%s-plane-move-continuous-%s
  * Description: Test that fbc with selective fetch works on moving %arg[1] plane %arg[2]
  *              visible area (no update)
- * Functionality: plane, psr2, selective_fetch, fbc
  *
  * SUBTEST: fbc-pr-%s-plane-move-continuous-%s
  * Mega feature: Panel Replay
  * Description: Test that fbc with selective fetch works on moving %arg[1] plane %arg[2]
  *              visible area (no update)
- * Functionality: plane, pr, selective_fetch, fbc
  *
  * arg[1]:
  *
@@ -84,16 +79,13 @@
  * SUBTEST: pr-cursor-plane-update-sf
  * Mega feature: Panel Replay
  * Description: Test that selective fetch works on cursor plane
- * Functionality: plane, pr, selective_fetch
  *
  * SUBTEST: fbc-psr2-cursor-plane-update-sf
  * Description: Test that fbc with selective fetch works on cursor plane
- * Functionality: plane, psr2, selective_fetch, fbc
  *
  * SUBTEST: fbc-pr-cursor-plane-update-sf
  * Mega feature: Panel Replay
  * Description: Test that fbc with selective fetch works on cursor plane
- * Functionality: plane, pr, selective_fetch, fbc
  *
  * SUBTEST: psr2-overlay-plane-update-continuous-sf
  * Description: Test that selective fetch works on overlay plane
@@ -101,16 +93,13 @@
  * SUBTEST: pr-overlay-plane-update-continuous-sf
  * Mega feature: Panel Replay
  * Description: Test that selective fetch works on overlay plane
- * Functionality: plane, pr, selective_fetch
  *
  * SUBTEST: fbc-psr2-overlay-plane-update-sf-dmg-area
  * Description: Test that fbc with selective fetch works on overlay plane
- * Functionality: plane, psr2, selective_fetch, fbc
  *
  * SUBTEST: fbc-pr-overlay-plane-update-sf-dmg-area
  * Mega feature: Panel Replay
  * Description: Test that fbc with selective fetch works on overlay plane
- * Functionality: plane, pr, selective_fetch, fbc
  *
  * SUBTEST: psr2-overlay-plane-update-sf-dmg-area
  * Description: Test that selective fetch works on overlay plane
@@ -118,16 +107,13 @@
  * SUBTEST: pr-overlay-plane-update-sf-dmg-area
  * Mega feature: Panel Replay
  * Description: Test that selective fetch works on overlay plane
- * Functionality: plane, pr, selective_fetch
  *
  * SUBTEST: fbc-psr2-overlay-plane-update-continuous-sf
  * Description: Test that fbc with selective fetch works on overlay plane
- * Functionality: plane, psr2, selective_fetch, fbc
  *
  * SUBTEST: fbc-pr-overlay-plane-update-continuous-sf
  * Mega feature: Panel Replay
  * Description: Test that fbc with selective fetch works on overlay plane
- * Functionality: plane, pr, selective_fetch, fbc
  *
  * SUBTEST: psr2-overlay-primary-update-sf-dmg-area
  * Description: Test that selective fetch works on primary plane with blended
@@ -137,18 +123,15 @@
  * Mega feature: Panel Replay
  * Description: Test that selective fetch works on primary plane with blended
  *              overlay plane
- * Functionality: plane, pr, selective_fetch
  *
  * SUBTEST: fbc-psr2-overlay-primary-update-sf-dmg-area
  * Description: Test that fbc with selective fetch works on primary plane with blended
  *              overlay plane
- * Functionality: plane, psr2, selective_fetch, fbc
  *
  * SUBTEST: fbc-pr-overlay-primary-update-sf-dmg-area
  * Mega feature: Panel Replay
  * Description: Test that fbc with selective fetch works on primary plane with blended
  *              overlay plane
- * Functionality: plane, pr, selective_fetch, fbc
  *
  * SUBTEST: psr2-plane-move-sf-dmg-area
  * Description: Test that selective fetch works on moving overlay plane
@@ -156,16 +139,13 @@
  * SUBTEST: pr-plane-move-sf-dmg-area
  * Mega feature: Panel Replay
  * Description: Test that selective fetch works on moving overlay plane
- * Functionality: plane, pr, selective_fetch
  *
  * SUBTEST: fbc-psr2-plane-move-sf-dmg-area
  * Description: Test that fbc with selective fetch works on moving overlay plane
- * Functionality: plane, psr2, selective_fetch, fbc
  *
  * SUBTEST: fbc-pr-plane-move-sf-dmg-area
  * Mega feature: Panel Replay
  * Description: Test that fbc with selective fetch works on moving overlay plane
- * Functionality: plane, pr, selective_fetch, fbc
  *
  * SUBTEST: psr2-primary-plane-update-sf-dmg-area
  * Description: Test that selective fetch works on primary plane
@@ -173,25 +153,20 @@
  * SUBTEST: pr-primary-plane-update-sf-dmg-area
  * Mega feature: Panel Replay
  * Description: Test that selective fetch works on primary plane
- * Functionality: plane, pr, selective_fetch
  *
  * SUBTEST: fbc-psr2-primary-plane-update-sf-dmg-area
  * Description: Test that fbc with selective fetch works on primary plane
- * Functionality: plane, psr2, selective_fetch, fbc
  *
  * SUBTEST: fbc-pr-primary-plane-update-sf-dmg-area
  * Mega feature: Panel Replay
  * Description: Test that fbc with selective fetch works on primary plane
- * Functionality: plane, pr, selective_fetch, fbc
  *
  * SUBTEST: psr2-primary-plane-update-sf-dmg-area-big-fb
  * Description: Test that selective fetch works on primary plane with big fb
- * Functionality: plane, psr2, selective_fetch, big_fbs
  *
  * SUBTEST: pr-primary-plane-update-sf-dmg-area-big-fb
  * Mega feature: Panel Replay
  * Description: Test that selective fetch works on primary plane with big fb
- * Functionality: plane, pr, selective_fetch, big_fbs
  */
 
 IGT_TEST_DESCRIPTION("Tests to verify PSR2 selective fetch by sending multiple"
@@ -1225,10 +1200,9 @@ igt_main
 
 		display_init(&data);
 
-		if ((intel_display_ver(intel_get_drm_devid(data.drm_fd)) >= 20) &&
-		    (intel_fbc_supported_on_chipset(data.drm_fd, data.pipe))) {
+		if (intel_fbc_supported_on_chipset(data.drm_fd, data.pipe) &&
+		    intel_fbc_psr_combo_supported(data.drm_fd))
 			data.fbc_flag = true;
-		}
 
 		data.damage_area_count = MAX_DAMAGE_AREAS;
 		data.primary_format = DRM_FORMAT_XRGB8888;
