@@ -101,9 +101,6 @@
  * SUBTEST: busy-start
  * Description: Test to verify gpu busyness through engine business pmu counters
  *
- * SUBTEST: cpu-hotplug
- * Description: Test the i915 pmu perf interface
- *
  * SUBTEST: enable-race
  * Description: Test the i915 pmu perf interface
  *
@@ -284,39 +281,6 @@ static uint64_t pmu_read_multi(int fd, unsigned int num, uint64_t *val)
 	return buf[1];
 }
 
-#define __assert_within_epsilon(x, ref, tol_up, tol_down, debug_data) \
-	igt_assert_f((double)(x) <= (1.0 + (tol_up)) * (double)(ref) && \
-		     (double)(x) >= (1.0 - (tol_down)) * (double)(ref), \
-		     "'%s' != '%s' (%f not within +%.1f%%/-%.1f%% tolerance of %f)\n%s\n",\
-		     #x, #ref, (double)(x), \
-		     (tol_up) * 100.0, (tol_down) * 100.0, \
-		     (double)(ref), debug_data)
-
-#define assert_within_epsilon(x, ref, tolerance) \
-	__assert_within_epsilon(x, ref, tolerance, tolerance, no_debug_data)
-
-#define assert_within_epsilon_debug(x, ref, tolerance, debug_data) \
-	__assert_within_epsilon(x, ref, tolerance, tolerance, debug_data)
-/*
- * Helper for cases where we assert on time spent sleeping (directly or
- * indirectly), so make it more robust by ensuring the system sleep time
- * is within test tolerance to start with.
- */
-static unsigned int measured_usleep(unsigned int usec)
-{
-	struct timespec ts = { };
-	unsigned int slept;
-
-	slept = igt_nsec_elapsed(&ts);
-	igt_assert(slept == 0);
-	do {
-		usleep(usec - slept);
-		slept = igt_nsec_elapsed(&ts) / 1000;
-	} while (slept < usec);
-
-	return igt_nsec_elapsed(&ts);
-}
-
 #define TEST_BUSY (1)
 #define FLAG_SYNC (2)
 #define TEST_TRAILING_IDLE (4)
@@ -377,7 +341,7 @@ single(int gem_fd, const intel_ctx_t *ctx,
 		spin = NULL;
 
 	val = pmu_read_single(fd);
-	slept = measured_usleep(batch_duration_ns / 1000);
+	slept = igt_measured_usleep(batch_duration_ns / 1000) * NSEC_PER_USEC;
 	if (flags & TEST_TRAILING_IDLE)
 		end_spin(gem_fd, spin, flags);
 	val = pmu_read_single(fd) - val;
@@ -395,7 +359,7 @@ single(int gem_fd, const intel_ctx_t *ctx,
 		igt_assert(!gem_bo_busy(gem_fd, spin->handle));
 
 		val = pmu_read_single(fd);
-		slept = measured_usleep(batch_duration_ns / 1000);
+		slept = igt_measured_usleep(batch_duration_ns / 1000) * NSEC_PER_USEC;
 		val = pmu_read_single(fd) - val;
 
 		assert_within_epsilon(val, 0, tolerance);
@@ -429,7 +393,7 @@ busy_start(int gem_fd, const intel_ctx_t *ctx,
 	fd = open_pmu(gem_fd, I915_PMU_ENGINE_BUSY(e->class, e->instance));
 
 	val = __pmu_read_single(fd, &ts[0]);
-	slept = measured_usleep(batch_duration_ns / 1000);
+	slept = igt_measured_usleep(batch_duration_ns / 1000) * NSEC_PER_USEC;
 	val = __pmu_read_single(fd, &ts[1]) - val;
 	igt_debug("slept=%lu perf=%"PRIu64"\n", slept, ts[1] - ts[0]);
 
@@ -485,7 +449,7 @@ busy_double_start(int gem_fd, const intel_ctx_t *ctx,
 	fd = open_pmu(gem_fd, I915_PMU_ENGINE_BUSY(e->class, e->instance));
 
 	val = __pmu_read_single(fd, &ts[0]);
-	slept = measured_usleep(batch_duration_ns / 1000);
+	slept = igt_measured_usleep(batch_duration_ns / 1000) * NSEC_PER_USEC;
 	val = __pmu_read_single(fd, &ts[1]) - val;
 	igt_debug("slept=%lu perf=%"PRIu64"\n", slept, ts[1] - ts[0]);
 
@@ -565,7 +529,7 @@ busy_check_all(int gem_fd, const intel_ctx_t *ctx,
 
 	spin = igt_sync_spin(gem_fd, ahnd, ctx, e);
 	pmu_read_multi(fd[0], num_engines, tval[0]);
-	slept = measured_usleep(batch_duration_ns / 1000);
+	slept = igt_measured_usleep(batch_duration_ns / 1000) * NSEC_PER_USEC;
 	if (flags & TEST_TRAILING_IDLE)
 		end_spin(gem_fd, spin, flags);
 	pmu_read_multi(fd[0], num_engines, tval[1]);
@@ -640,7 +604,7 @@ most_busy_check_all(int gem_fd, const intel_ctx_t *ctx,
 	usleep(__igt_sync_spin_wait(gem_fd, spin) * num_engines / 1e3);
 
 	pmu_read_multi(fd[0], num_engines, tval[0]);
-	slept = measured_usleep(batch_duration_ns / 1000);
+	slept = igt_measured_usleep(batch_duration_ns / 1000) * NSEC_PER_USEC;
 	if (flags & TEST_TRAILING_IDLE)
 		end_spin(gem_fd, spin, flags);
 	pmu_read_multi(fd[0], num_engines, tval[1]);
@@ -698,7 +662,7 @@ all_busy_check_all(int gem_fd, const intel_ctx_t *ctx,
 	usleep(__igt_sync_spin_wait(gem_fd, spin) * num_engines / 1e3);
 
 	pmu_read_multi(fd[0], num_engines, tval[0]);
-	slept = measured_usleep(batch_duration_ns / 1000);
+	slept = igt_measured_usleep(batch_duration_ns / 1000) * NSEC_PER_USEC;
 	if (flags & TEST_TRAILING_IDLE)
 		end_spin(gem_fd, spin, flags);
 	pmu_read_multi(fd[0], num_engines, tval[1]);
@@ -741,7 +705,7 @@ no_sema(int gem_fd, const intel_ctx_t *ctx,
 		spin = NULL;
 
 	pmu_read_multi(fd[0], 2, val[0]);
-	measured_usleep(batch_duration_ns / 1000);
+	igt_measured_usleep(batch_duration_ns / 1000);
 	if (flags & TEST_TRAILING_IDLE)
 		end_spin(gem_fd, spin, flags);
 	pmu_read_multi(fd[0], 2, val[1]);
@@ -854,7 +818,7 @@ sema_wait(int gem_fd, const intel_ctx_t *ctx,
 		     "sampling failed to start withing 10ms\n");
 
 	val[0] = __pmu_read_single(fd, &ts[0]);
-	slept = measured_usleep(batch_duration_ns / 1000);
+	slept = igt_measured_usleep(batch_duration_ns / 1000) * NSEC_PER_USEC;
 	if (flags & TEST_TRAILING_IDLE)
 		obj_ptr[0] = 1;
 	val[1] = __pmu_read_single(fd, &ts[1]);
@@ -975,11 +939,12 @@ __sema_busy(int gem_fd, uint64_t ahnd, int pmu, const intel_ctx_t *ctx,
 
 	total = pmu_read_multi(pmu, 2, start);
 
-	sema = measured_usleep(batch_duration_ns * sema_pct / 100 / 1000);
+	sema = igt_measured_usleep(batch_duration_ns * sema_pct / 100 / 1000) * NSEC_PER_USEC;
 	*map = 2; __sync_synchronize();
-	busy = measured_usleep(batch_duration_ns * (busy_pct - sema_pct) / 100 / 1000);
+	busy = igt_measured_usleep(batch_duration_ns * (busy_pct - sema_pct) / 100 / 1000)
+	       * NSEC_PER_USEC;
 	igt_spin_end(spin);
-	measured_usleep(batch_duration_ns * (100 - busy_pct) / 100 / 1000);
+	igt_measured_usleep(batch_duration_ns * (100 - busy_pct) / 100 / 1000);
 
 	total = pmu_read_multi(pmu, 2, val) - total;
 	igt_spin_free(gem_fd, spin);
@@ -1044,7 +1009,7 @@ static void test_awake(int i915, const intel_ctx_t *ctx)
 		igt_spin_new(i915, .ahnd = ahnd, .ctx = ctx, .engine = e->flags);
 
 		val = pmu_read_single(fd);
-		slept = measured_usleep(batch_duration_ns / 1000);
+		slept = igt_measured_usleep(batch_duration_ns / 1000) * NSEC_PER_USEC;
 		val = pmu_read_single(fd) - val;
 
 		gem_quiescent_gpu(i915);
@@ -1056,7 +1021,7 @@ static void test_awake(int i915, const intel_ctx_t *ctx)
 		igt_spin_new(i915, .ahnd = ahnd, .ctx = ctx, .engine = e->flags);
 
 	val = pmu_read_single(fd);
-	slept = measured_usleep(batch_duration_ns / 1000);
+	slept = igt_measured_usleep(batch_duration_ns / 1000) * NSEC_PER_USEC;
 	val = pmu_read_single(fd) - val;
 
 	gem_quiescent_gpu(i915);
@@ -1297,13 +1262,13 @@ multi_client(int gem_fd, const intel_ctx_t *ctx,
 	spin = igt_sync_spin(gem_fd, ahnd, ctx, e);
 
 	val[0] = val[1] = __pmu_read_single(fd[0], &ts[0]);
-	slept[1] = measured_usleep(batch_duration_ns / 1000);
+	slept[1] = igt_measured_usleep(batch_duration_ns / 1000) * NSEC_PER_USEC;
 	val[1] = __pmu_read_single(fd[1], &ts[1]) - val[1];
 	perf_slept[1] = ts[1] - ts[0];
 	igt_debug("slept=%lu perf=%"PRIu64"\n", slept[1], perf_slept[1]);
 	close(fd[1]);
 
-	slept[0] = measured_usleep(batch_duration_ns / 1000) + slept[1];
+	slept[0] = (igt_measured_usleep(batch_duration_ns / 1000) * NSEC_PER_USEC) + slept[1];
 	val[0] = __pmu_read_single(fd[0], &ts[1]) - val[0];
 	perf_slept[0] = ts[1] - ts[0];
 	igt_debug("slept=%lu perf=%"PRIu64"\n", slept[0], perf_slept[0]);
@@ -1362,139 +1327,6 @@ static void open_invalid(int i915)
 	igt_assert_lt(fd, 0);
 }
 
-static bool cpu0_hotplug_support(void)
-{
-	return access("/sys/devices/system/cpu/cpu0/online", W_OK) == 0;
-}
-
-static void cpu_hotplug(int gem_fd)
-{
-	igt_spin_t *spin[2];
-	uint64_t ts[2];
-	uint64_t val;
-	int link[2];
-	int fd, ret;
-	int cur = 0;
-	char buf;
-	uint64_t ahnd = get_reloc_ahnd(gem_fd, 0);
-
-	igt_require(cpu0_hotplug_support());
-
-	fd = open_pmu(gem_fd,
-		      I915_PMU_ENGINE_BUSY(I915_ENGINE_CLASS_RENDER, 0));
-
-	/*
-	 * Create two spinners so test can ensure shorter gaps in engine
-	 * busyness as it is terminating one and re-starting the other.
-	 */
-	spin[0] = igt_spin_new(gem_fd, .ahnd = ahnd,
-			       .engine = I915_EXEC_DEFAULT);
-	spin[1] = __igt_spin_new(gem_fd, .ahnd = ahnd,
-				 .engine = I915_EXEC_DEFAULT);
-
-	val = __pmu_read_single(fd, &ts[0]);
-
-	ret = pipe2(link, O_NONBLOCK);
-	igt_assert_eq(ret, 0);
-
-	/*
-	 * Toggle online status of all the CPUs in a child process and ensure
-	 * this has not affected busyness stats in the parent.
-	 */
-	igt_fork(child, 1) {
-		int cpu = 0;
-
-		close(link[0]);
-
-		for (;;) {
-			char name[128];
-			int cpufd;
-
-			igt_assert_lt(snprintf(name, sizeof(name),
-					       "/sys/devices/system/cpu/cpu%d/online",
-					       cpu), sizeof(name));
-			cpufd = open(name, O_WRONLY);
-			if (cpufd == -1) {
-				igt_assert_lt(0, cpu);
-				/*
-				 * Signal parent that we cycled through all
-				 * CPUs and we are done.
-				 */
-				igt_assert_eq(write(link[1], "*", 1), 1);
-				break;
-			}
-
-			/* Offline followed by online a CPU. */
-
-			ret = write(cpufd, "0", 2);
-			if (ret < 0) {
-				/*
-				 * If we failed to offline a CPU we don't want
-				 * to proceed.
-				 */
-				igt_warn("Failed to offline cpu%u! (%d)\n",
-					 cpu, errno);
-				igt_assert_eq(write(link[1], "s", 1), 1);
-				break;
-			}
-
-			usleep(1e6);
-
-			ret = write(cpufd, "1", 2);
-			if (ret < 0) {
-				/*
-				 * Failed to bring a CPU back online is fatal
-				 * for the sanity of a test run so stop further
-				 * testing.
-				 */
-				igt_warn("Failed to online cpu%u! (%d)\n",
-					 cpu, errno);
-				igt_fatal_error();
-			}
-
-			close(cpufd);
-			cpu++;
-		}
-	}
-
-	close(link[1]);
-
-	/*
-	 * Very long batches can be declared as GPU hangs so emit shorter ones
-	 * until the CPU core shuffler finishes one loop.
-	 */
-	for (;;) {
-		usleep(500e3);
-		end_spin(gem_fd, spin[cur], 0);
-
-		/* Check if the child is signaling completion. */
-		ret = read(link[0], &buf, 1);
-		if ( ret == 1 || (ret < 0 && errno != EAGAIN))
-			break;
-
-		igt_spin_free(gem_fd, spin[cur]);
-		spin[cur] = __igt_spin_new(gem_fd, .ahnd = ahnd,
-					   .engine = I915_EXEC_DEFAULT);
-		cur ^= 1;
-	}
-
-	val = __pmu_read_single(fd, &ts[1]) - val;
-
-	end_spin(gem_fd, spin[0], FLAG_SYNC);
-	end_spin(gem_fd, spin[1], FLAG_SYNC);
-	igt_spin_free(gem_fd, spin[0]);
-	igt_spin_free(gem_fd, spin[1]);
-	igt_waitchildren();
-	close(fd);
-	close(link[0]);
-	put_ahnd(ahnd);
-
-	/* Skip if child signals a problem with offlining a CPU. */
-	igt_skip_on(buf == 's');
-
-	assert_within_epsilon(val, ts[1] - ts[0], tolerance);
-}
-
 static int target_num_interrupts(int i915)
 {
 	const intel_ctx_cfg_t cfg = intel_ctx_cfg_all_physical(i915);
@@ -1710,7 +1542,7 @@ test_frequency(int gem_fd, unsigned int gt)
 	spin = spin_sync_gt(gem_fd, ahnd, gt, &ctx);
 
 	slept = pmu_read_multi(fd[0], 2, start);
-	measured_usleep(batch_duration_ns / 1000);
+	igt_measured_usleep(batch_duration_ns / 1000);
 	slept = pmu_read_multi(fd[0], 2, val) - slept;
 
 	min[0] = 1e9*(val[0] - start[0]) / slept;
@@ -1740,7 +1572,7 @@ test_frequency(int gem_fd, unsigned int gt)
 	spin = spin_sync_gt(gem_fd, ahnd, gt, &ctx);
 
 	slept = pmu_read_multi(fd[0], 2, start);
-	measured_usleep(batch_duration_ns / 1000);
+	igt_measured_usleep(batch_duration_ns / 1000);
 	slept = pmu_read_multi(fd[0], 2, val) - slept;
 
 	max[0] = 1e9*(val[0] - start[0]) / slept;
@@ -1772,7 +1604,7 @@ test_frequency(int gem_fd, unsigned int gt)
 	 * On thermally throttled devices we cannot be sure maximum frequency
 	 * can be reached so use larger tolerance downards.
 	 */
-	__assert_within_epsilon(max[0], max_freq, tolerance, 0.15f, no_debug_data);
+	assert_within_epsilon_up_down(max[0], max_freq, tolerance, 0.15f);
 }
 
 static void
@@ -1795,10 +1627,10 @@ test_frequency_idle(int gem_fd, unsigned int gt)
 	fd[1] = open_group(gem_fd, __I915_PMU_ACTUAL_FREQUENCY(gt), fd[0]);
 
 	gem_quiescent_gpu(gem_fd); /* Be idle! */
-	measured_usleep(2000); /* Wait for timers to cease */
+	igt_measured_usleep(2000); /* Wait for timers to cease */
 
 	slept = pmu_read_multi(fd[0], 2, start);
-	measured_usleep(batch_duration_ns / 1000);
+	igt_measured_usleep(batch_duration_ns / 1000);
 	slept = pmu_read_multi(fd[0], 2, val) - slept;
 
 	close(fd[0]);
@@ -1922,7 +1754,7 @@ test_rc6(int gem_fd, unsigned int gt, unsigned int num_gt, unsigned int flags)
 
 	/* While idle check full RC6. */
 	ts[0] = pmu_read_multi(fd[0], pmus, prev);
-	slept = measured_usleep(duration_ns / 1000);
+	slept = igt_measured_usleep(duration_ns / 1000) * NSEC_PER_USEC;
 	ts[1] = pmu_read_multi(fd[0], pmus, idle);
 
 	for (gt_ = 0; gt_ < pmus; gt_++) {
@@ -1962,7 +1794,7 @@ test_rc6(int gem_fd, unsigned int gt, unsigned int num_gt, unsigned int flags)
 		      "failed to enter c6 \n%s\n", drpc = get_drpc(gem_fd, test_idx));
 
 	ts[0] = pmu_read_multi(fd[0], pmus, prev);
-	slept = measured_usleep(duration_ns / 1000);
+	slept = igt_measured_usleep(duration_ns / 1000) * NSEC_PER_USEC;
 	ts[1] = pmu_read_multi(fd[0], pmus, idle);
 
 	for (gt_ = 0; gt_ < pmus; gt_++) {
@@ -1987,7 +1819,7 @@ test_rc6(int gem_fd, unsigned int gt, unsigned int num_gt, unsigned int flags)
 	usleep(1e3); /* wait for the rc6 cycle counter to stop ticking */
 
 	ts[0] = pmu_read_multi(fd[0], pmus, prev);
-	slept = measured_usleep(duration_ns / 1000);
+	slept = igt_measured_usleep(duration_ns / 1000) * NSEC_PER_USEC;
 	ts[1] = pmu_read_multi(fd[0], pmus, busy);
 
 	for (gt_ = 0; gt_ < num_gt; gt_++) {
@@ -2650,12 +2482,6 @@ igt_main
 		all_busy_check_all(fd, ctx, num_engines,
 				   TEST_BUSY | TEST_TRAILING_IDLE);
 
-	/**
-	 * Test counters are not affected by CPU offline/online events.
-	 */
-	igt_subtest("cpu-hotplug")
-		cpu_hotplug(fd);
-
 	/**
 	 * Test GPU frequency.
 	 */
