@@ -3,24 +3,27 @@
  * Copyright © 2023 Intel Corporation
  */
 
+#include <math.h>
+
 #include "igt.h"
 #include "igt_core.h"
 #include "igt_device.h"
 #include "igt_drm_fdinfo.h"
 #include "lib/igt_syncobj.h"
+
 #include "xe_drm.h"
+#include "xe/xe_gt.h"
 #include "xe/xe_ioctl.h"
 #include "xe/xe_query.h"
 #include "xe/xe_spin.h"
-#include "xe/xe_util.h"
 
 /**
  * TEST: xe drm fdinfo
- * Description: Read and verify drm client memory consumption and engine utilization using fdinfo
  * Category: Core
  * Mega feature: General Core features
  * Sub-category: driver
  * Functionality: Per client memory and engine utilization statistics
+ * Description: Read and verify drm client memory consumption and engine utilization using fdinfo
  * Feature: SMI, core
  * Test category: SysMan
  *
@@ -355,6 +358,16 @@ static void basic_memory(int xe)
 	}
 }
 
+static void require_engine_utilization_data(int xe)
+{
+	struct drm_client_fdinfo info = {};
+
+	igt_require(igt_parse_drm_fdinfo(xe, &info, engine_map,
+					 ARRAY_SIZE(engine_map), NULL, 0));
+	igt_require(info.num_engines);
+	igt_require(info.utilization_mask & DRM_FDINFO_UTILIZATION_CYCLES);
+}
+
 static void basic_engine_utilization(int xe)
 {
 	struct drm_client_fdinfo info = { };
@@ -364,168 +377,48 @@ static void basic_engine_utilization(int xe)
 				   ARRAY_SIZE(engine_map), NULL, 0);
 	igt_assert_f(ret != 0, "failed with err:%d\n", errno);
 	igt_assert(!strcmp(info.driver, "xe"));
-	igt_require(info.num_engines);
-}
-
-struct spin_ctx {
-	uint32_t vm;
-	uint64_t addr[XE_MAX_ENGINE_INSTANCE];
-	struct drm_xe_sync sync[2];
-	struct drm_xe_exec exec;
-	uint32_t exec_queue;
-	size_t bo_size;
-	uint32_t bo;
-	struct xe_spin *spin;
-	struct xe_spin_opts spin_opts;
-	bool ended;
-	uint16_t class;
-	uint16_t width;
-	uint16_t num_placements;
-};
-
-static struct spin_ctx *
-spin_ctx_init(int fd, struct drm_xe_engine_class_instance *hwe, uint32_t vm,
-	      uint16_t width, uint16_t num_placements)
-{
-	struct spin_ctx *ctx = calloc(1, sizeof(*ctx));
-
-	igt_assert(width && num_placements &&
-		   (width == 1 || num_placements == 1));
-	igt_assert_lt(width, XE_MAX_ENGINE_INSTANCE);
-
-	ctx->class = hwe->engine_class;
-	ctx->width = width;
-	ctx->num_placements = num_placements;
-	ctx->vm = vm;
-
-	for (unsigned int i = 0; i < width; i++)
-		ctx->addr[i] = 0x100000 + 0x100000 * hwe->engine_class;
-
-	ctx->exec.num_batch_buffer = width;
-	ctx->exec.num_syncs = 2;
-	ctx->exec.syncs = to_user_pointer(ctx->sync);
-
-	ctx->sync[0].type = DRM_XE_SYNC_TYPE_SYNCOBJ;
-	ctx->sync[0].flags = DRM_XE_SYNC_FLAG_SIGNAL;
-	ctx->sync[0].handle = syncobj_create(fd, 0);
-
-	ctx->sync[1].type = DRM_XE_SYNC_TYPE_SYNCOBJ;
-	ctx->sync[1].flags = DRM_XE_SYNC_FLAG_SIGNAL;
-	ctx->sync[1].handle = syncobj_create(fd, 0);
-
-	ctx->bo_size = sizeof(struct xe_spin);
-	ctx->bo_size = xe_bb_size(fd, ctx->bo_size);
-	ctx->bo = xe_bo_create(fd, ctx->vm, ctx->bo_size,
-			       vram_if_possible(fd, hwe->gt_id),
-			       DRM_XE_GEM_CREATE_FLAG_NEEDS_VISIBLE_VRAM);
-	ctx->spin = xe_bo_map(fd, ctx->bo, ctx->bo_size);
-
-	igt_assert_eq(__xe_exec_queue_create(fd, ctx->vm, width, num_placements,
-					     hwe, 0, &ctx->exec_queue), 0);
-
-	xe_vm_bind_async(fd, ctx->vm, 0, ctx->bo, 0, ctx->addr[0], ctx->bo_size,
-			 ctx->sync, 1);
-
-	return ctx;
-}
-
-static void
-spin_sync_start(int fd, struct spin_ctx *ctx)
-{
-	if (!ctx)
-		return;
-
-	ctx->spin_opts.addr = ctx->addr[0];
-	ctx->spin_opts.write_timestamp = true;
-	ctx->spin_opts.preempt = true;
-	xe_spin_init(ctx->spin, &ctx->spin_opts);
-
-	/* re-use sync[0] for exec */
-	ctx->sync[0].flags &= ~DRM_XE_SYNC_FLAG_SIGNAL;
-
-	ctx->exec.exec_queue_id = ctx->exec_queue;
-
-	if (ctx->width > 1)
-		ctx->exec.address = to_user_pointer(ctx->addr);
-	else
-		ctx->exec.address = ctx->addr[0];
-
-	xe_exec(fd, &ctx->exec);
-
-	xe_spin_wait_started(ctx->spin);
-	igt_assert(!syncobj_wait(fd, &ctx->sync[1].handle, 1, 1, 0, NULL));
-
-	igt_debug("%s: spinner started\n", engine_map[ctx->class]);
-}
-
-static void
-spin_sync_end(int fd, struct spin_ctx *ctx)
-{
-	if (!ctx || ctx->ended)
-		return;
-
-	xe_spin_end(ctx->spin);
-
-	igt_assert(syncobj_wait(fd, &ctx->sync[1].handle, 1, INT64_MAX, 0, NULL));
-	igt_assert(syncobj_wait(fd, &ctx->sync[0].handle, 1, INT64_MAX, 0, NULL));
-
-	ctx->sync[0].flags |= DRM_XE_SYNC_FLAG_SIGNAL;
-	xe_vm_unbind_async(fd, ctx->vm, 0, 0, ctx->addr[0], ctx->bo_size, ctx->sync, 1);
-	igt_assert(syncobj_wait(fd, &ctx->sync[0].handle, 1, INT64_MAX, 0, NULL));
-
-	ctx->ended = true;
-	igt_debug("%s: spinner ended (timestamp=%u)\n", engine_map[ctx->class],
-		  ctx->spin->timestamp);
-}
-
-static void
-spin_ctx_destroy(int fd, struct spin_ctx *ctx)
-{
-	if (!ctx)
-		return;
-
-	syncobj_destroy(fd, ctx->sync[0].handle);
-	syncobj_destroy(fd, ctx->sync[1].handle);
-	xe_exec_queue_destroy(fd, ctx->exec_queue);
-
-	munmap(ctx->spin, ctx->bo_size);
-	gem_close(fd, ctx->bo);
-
-	free(ctx);
+	igt_assert(info.num_engines);
 }
 
 static void
 check_results(struct pceu_cycles *s1, struct pceu_cycles *s2,
-	      int class, int width, enum expected_load expected_load)
+	      int class, int width, uint32_t spin_stamp,
+	      enum expected_load expected_load)
 {
 	double percent;
-	u64 den, num;
+	uint64_t den, num;
 
 	igt_debug("%s: sample 1: cycles %"PRIu64", total_cycles %"PRIu64"\n",
 		  engine_map[class], s1[class].cycles, s1[class].total_cycles);
 	igt_debug("%s: sample 2: cycles %"PRIu64", total_cycles %"PRIu64"\n",
 		  engine_map[class], s2[class].cycles, s2[class].total_cycles);
+	igt_debug("spin_stamp: %u width: %d\n", spin_stamp, width);
 
 	num = s2[class].cycles - s1[class].cycles;
 	den = s2[class].total_cycles - s1[class].total_cycles;
+
 	percent = (num * 100.0) / (den + 1);
 
-	/* for parallel submission scale the busyness with width */
+	/* For parallel submission scale the busyness with width */
 	percent /= width;
 
-	igt_debug("%s: percent: %f\n", engine_map[class], percent);
+	igt_debug("%s: percent: %.2f%%\n", engine_map[class], percent);
 
 	switch (expected_load) {
 	case EXPECTED_LOAD_IDLE:
 		igt_assert_eq(num, 0);
 		break;
 	case EXPECTED_LOAD_FULL:
+		/* For parallel submission scale the busyness with width */
+		spin_stamp *= width;
+
 		/*
-		 * We are still relying on CPU sleep time and there could be
-		 * some imprecision when calculating the load. Use a 5% margin.
+		 * Percentage error between value saved by gpu in xe_spin and what
+		 * is reported via fdinfo
 		 */
-		igt_assert_lt_double(95.0, percent);
-		igt_assert_lt_double(percent, 105.0);
+		percent = fabs((int64_t)(num - spin_stamp) * 100.0) / (spin_stamp + 1);
+		igt_debug("%s: error: %.2f%%\n", engine_map[class], percent);
+		igt_assert_lt_double(percent, 5.0);
 		break;
 	}
 }
@@ -535,18 +428,20 @@ utilization_single(int fd, struct drm_xe_engine_class_instance *hwe, unsigned in
 {
 	struct pceu_cycles pceu1[2][DRM_XE_ENGINE_CLASS_COMPUTE + 1];
 	struct pceu_cycles pceu2[2][DRM_XE_ENGINE_CLASS_COMPUTE + 1];
-	struct spin_ctx *ctx = NULL;
+	struct xe_cork *cork = NULL;
 	enum expected_load expected_load;
 	uint32_t vm;
 	int new_fd;
 
+	igt_debug("Target class: %s\n", engine_map[hwe->engine_class]);
+
 	if (flags & TEST_ISOLATION)
 		new_fd = drm_reopen_driver(fd);
 
 	vm = xe_vm_create(fd, 0, 0);
 	if (flags & TEST_BUSY) {
-		ctx = spin_ctx_init(fd, hwe, vm, 1, 1);
-		spin_sync_start(fd, ctx);
+		cork = xe_cork_create_opts(fd, hwe, vm, 1, 1);
+		xe_cork_sync_start(fd, cork);
 	}
 
 	read_engine_cycles(fd, pceu1[0]);
@@ -555,7 +450,7 @@ utilization_single(int fd, struct drm_xe_engine_class_instance *hwe, unsigned in
 
 	usleep(batch_duration_usec);
 	if (flags & TEST_TRAILING_IDLE)
-		spin_sync_end(fd, ctx);
+		xe_cork_sync_end(fd, cork);
 
 	read_engine_cycles(fd, pceu2[0]);
 	if (flags & TEST_ISOLATION)
@@ -563,19 +458,23 @@ utilization_single(int fd, struct drm_xe_engine_class_instance *hwe, unsigned in
 
 	expected_load = flags & TEST_BUSY ?
 	       EXPECTED_LOAD_FULL : EXPECTED_LOAD_IDLE;
-	check_results(pceu1[0], pceu2[0], hwe->engine_class, 1, expected_load);
+
+	check_results(pceu1[0], pceu2[0], hwe->engine_class, 1,
+		      cork ? cork->spin->timestamp : 0, expected_load);
 
 	if (flags & TEST_ISOLATION) {
 		/*
 		 * Load from one client shouldn't spill on another,
 		 * so check for idle
 		 */
-		check_results(pceu1[1], pceu2[1], hwe->engine_class, 1, EXPECTED_LOAD_IDLE);
+		check_results(pceu1[1], pceu2[1], hwe->engine_class, 1, 0,
+			      EXPECTED_LOAD_IDLE);
 		close(new_fd);
 	}
 
-	spin_sync_end(fd, ctx);
-	spin_ctx_destroy(fd, ctx);
+	if (cork)
+		xe_cork_destroy(fd, cork);
+
 	xe_vm_destroy(fd, vm);
 }
 
@@ -584,25 +483,30 @@ utilization_single_destroy_queue(int fd, struct drm_xe_engine_class_instance *hw
 {
 	struct pceu_cycles pceu1[DRM_XE_ENGINE_CLASS_COMPUTE + 1];
 	struct pceu_cycles pceu2[DRM_XE_ENGINE_CLASS_COMPUTE + 1];
-	struct spin_ctx *ctx = NULL;
+	struct xe_cork *cork;
+	uint32_t timestamp;
 	uint32_t vm;
 
+	igt_debug("Target class: %s\n", engine_map[hwe->engine_class]);
+
 	vm = xe_vm_create(fd, 0, 0);
-	ctx = spin_ctx_init(fd, hwe, vm, 1, 1);
-	spin_sync_start(fd, ctx);
+	cork = xe_cork_create_opts(fd, hwe, vm, 1, 1);
+	xe_cork_sync_start(fd, cork);
 
 	read_engine_cycles(fd, pceu1);
 	usleep(batch_duration_usec);
 
 	/* destroy queue before sampling again */
-	spin_sync_end(fd, ctx);
-	spin_ctx_destroy(fd, ctx);
+	xe_cork_sync_end(fd, cork);
+	timestamp = cork->spin->timestamp;
+	xe_cork_destroy(fd, cork);
 
 	read_engine_cycles(fd, pceu2);
 
 	xe_vm_destroy(fd, vm);
 
-	check_results(pceu1, pceu2, hwe->engine_class, 1, EXPECTED_LOAD_FULL);
+	check_results(pceu1, pceu2, hwe->engine_class, 1, timestamp,
+		      EXPECTED_LOAD_FULL);
 }
 
 static void
@@ -610,29 +514,30 @@ utilization_others_idle(int fd, struct drm_xe_engine_class_instance *hwe)
 {
 	struct pceu_cycles pceu1[DRM_XE_ENGINE_CLASS_COMPUTE + 1];
 	struct pceu_cycles pceu2[DRM_XE_ENGINE_CLASS_COMPUTE + 1];
-	struct spin_ctx *ctx = NULL;
+	struct xe_cork *cork;
 	uint32_t vm;
 	int class;
 
-	vm = xe_vm_create(fd, 0, 0);
+	igt_debug("Target class: %s\n", engine_map[hwe->engine_class]);
 
-	ctx = spin_ctx_init(fd, hwe, vm, 1, 1);
-	spin_sync_start(fd, ctx);
+	vm = xe_vm_create(fd, 0, 0);
+	cork = xe_cork_create_opts(fd, hwe, vm, 1, 1);
+	xe_cork_sync_start(fd, cork);
 
 	read_engine_cycles(fd, pceu1);
 	usleep(batch_duration_usec);
-	spin_sync_end(fd, ctx);
+	xe_cork_sync_end(fd, cork);
 	read_engine_cycles(fd, pceu2);
 
 	xe_for_each_engine_class(class) {
 		enum expected_load expected_load = hwe->engine_class != class ?
 			EXPECTED_LOAD_IDLE : EXPECTED_LOAD_FULL;
 
-		check_results(pceu1, pceu2, class, 1, expected_load);
+		check_results(pceu1, pceu2, class, 1, cork->spin->timestamp,
+			      expected_load);
 	}
 
-	spin_sync_end(fd, ctx);
-	spin_ctx_destroy(fd, ctx);
+	xe_cork_destroy(fd, cork);
 	xe_vm_destroy(fd, vm);
 }
 
@@ -641,40 +546,45 @@ utilization_others_full_load(int fd, struct drm_xe_engine_class_instance *hwe)
 {
 	struct pceu_cycles pceu1[DRM_XE_ENGINE_CLASS_COMPUTE + 1];
 	struct pceu_cycles pceu2[DRM_XE_ENGINE_CLASS_COMPUTE + 1];
-	struct spin_ctx *ctx[DRM_XE_ENGINE_CLASS_COMPUTE + 1] = {};
+	struct xe_cork *cork[DRM_XE_ENGINE_CLASS_COMPUTE + 1] = {};
 	struct drm_xe_engine_class_instance *_hwe;
 	uint32_t vm;
 	int class;
 
+	igt_debug("Target class: %s\n", engine_map[hwe->engine_class]);
+
 	vm = xe_vm_create(fd, 0, 0);
 
 	/* spin on one hwe per class except the target class hwes */
 	xe_for_each_engine(fd, _hwe) {
 		int _class = _hwe->engine_class;
 
-		if (_class == hwe->engine_class || ctx[_class])
+		if (_class == hwe->engine_class || cork[_class])
 			continue;
 
-		ctx[_class] = spin_ctx_init(fd, _hwe, vm, 1, 1);
-		spin_sync_start(fd, ctx[_class]);
+		cork[_class] = xe_cork_create_opts(fd, _hwe, vm, 1, 1);
+		xe_cork_sync_start(fd, cork[_class]);
 	}
 
 	read_engine_cycles(fd, pceu1);
 	usleep(batch_duration_usec);
-	xe_for_each_engine_class(class)
-		spin_sync_end(fd, ctx[class]);
+	xe_for_each_engine_class(class) {
+		if (cork[class])
+			xe_cork_sync_end(fd, cork[class]);
+	}
+
 	read_engine_cycles(fd, pceu2);
 
 	xe_for_each_engine_class(class) {
 		enum expected_load expected_load = hwe->engine_class == class ?
 			EXPECTED_LOAD_IDLE : EXPECTED_LOAD_FULL;
 
-		if (!ctx[class])
-			continue;
+		check_results(pceu1, pceu2, class, 1,
+			      cork[class] ? cork[class]->spin->timestamp : 0,
+			      expected_load);
 
-		check_results(pceu1, pceu2, class, 1, expected_load);
-		spin_sync_end(fd, ctx[class]);
-		spin_ctx_destroy(fd, ctx[class]);
+		if (cork[class])
+			xe_cork_destroy(fd, cork[class]);
 	}
 
 	xe_vm_destroy(fd, vm);
@@ -685,7 +595,7 @@ utilization_all_full_load(int fd)
 {
 	struct pceu_cycles pceu1[DRM_XE_ENGINE_CLASS_COMPUTE + 1];
 	struct pceu_cycles pceu2[DRM_XE_ENGINE_CLASS_COMPUTE + 1];
-	struct spin_ctx *ctx[DRM_XE_ENGINE_CLASS_COMPUTE + 1] = {};
+	struct xe_cork *cork[DRM_XE_ENGINE_CLASS_COMPUTE + 1] = {};
 	struct drm_xe_engine_class_instance *hwe;
 	uint32_t vm;
 	int class;
@@ -695,26 +605,29 @@ utilization_all_full_load(int fd)
 	/* spin on one hwe per class */
 	xe_for_each_engine(fd, hwe) {
 		class = hwe->engine_class;
-		if (ctx[class])
+		if (cork[class])
 			continue;
-
-		ctx[class] = spin_ctx_init(fd, hwe, vm, 1, 1);
-		spin_sync_start(fd, ctx[class]);
+		cork[class] = xe_cork_create_opts(fd, hwe, vm, 1, 1);
+		xe_cork_sync_start(fd, cork[class]);
 	}
 
 	read_engine_cycles(fd, pceu1);
 	usleep(batch_duration_usec);
-	xe_for_each_engine_class(class)
-		spin_sync_end(fd, ctx[class]);
+	xe_for_each_engine_class(class) {
+		if (cork[class])
+			xe_cork_sync_end(fd, cork[class]);
+	}
+
 	read_engine_cycles(fd, pceu2);
 
 	xe_for_each_engine_class(class) {
-		if (!ctx[class])
+		if (!cork[class])
 			continue;
 
-		check_results(pceu1, pceu2, class, 1, EXPECTED_LOAD_FULL);
-		spin_sync_end(fd, ctx[class]);
-		spin_ctx_destroy(fd, ctx[class]);
+		check_results(pceu1, pceu2, class, 1,
+			      cork[class]->spin->timestamp,
+			      EXPECTED_LOAD_FULL);
+		xe_cork_destroy(fd, cork[class]);
 	}
 
 	xe_vm_destroy(fd, vm);
@@ -741,7 +654,7 @@ utilization_multi(int fd, int gt, int class, unsigned int flags)
 	struct pceu_cycles pceu[2][DRM_XE_ENGINE_CLASS_COMPUTE + 1];
 	struct pceu_cycles pceu_spill[2][DRM_XE_ENGINE_CLASS_COMPUTE + 1];
 	struct drm_xe_engine_class_instance eci[XE_MAX_ENGINE_INSTANCE];
-	struct spin_ctx *ctx = NULL;
+	struct xe_cork *cork = NULL;
 	enum expected_load expected_load;
 	int fd_spill, num_placements;
 	uint32_t vm;
@@ -755,6 +668,8 @@ utilization_multi(int fd, int gt, int class, unsigned int flags)
 	if (num_placements < 2)
 		return;
 
+	igt_debug("Target class: %s\n", engine_map[class]);
+
 	if (parallel) {
 		width = num_placements;
 		num_placements = 1;
@@ -767,8 +682,8 @@ utilization_multi(int fd, int gt, int class, unsigned int flags)
 
 	vm = xe_vm_create(fd, 0, 0);
 	if (flags & TEST_BUSY) {
-		ctx = spin_ctx_init(fd, eci, vm, width, num_placements);
-		spin_sync_start(fd, ctx);
+		cork = xe_cork_create_opts(fd, eci, vm, width, num_placements);
+		xe_cork_sync_start(fd, cork);
 	}
 
 	read_engine_cycles(fd, pceu[0]);
@@ -777,7 +692,7 @@ utilization_multi(int fd, int gt, int class, unsigned int flags)
 
 	usleep(batch_duration_usec);
 	if (flags & TEST_TRAILING_IDLE)
-		spin_sync_end(fd, ctx);
+		xe_cork_sync_end(fd, cork);
 
 	read_engine_cycles(fd, pceu[1]);
 	if (flags & TEST_ISOLATION)
@@ -785,20 +700,22 @@ utilization_multi(int fd, int gt, int class, unsigned int flags)
 
 	expected_load = flags & TEST_BUSY ?
 	       EXPECTED_LOAD_FULL : EXPECTED_LOAD_IDLE;
-	check_results(pceu[0], pceu[1], class, width, expected_load);
+
+	check_results(pceu[0], pceu[1], class, width,
+		      cork ? cork->spin->timestamp : 0, expected_load);
 
 	if (flags & TEST_ISOLATION) {
 		/*
 		 * Load from one client shouldn't spill on another,
 		 * so check for idle
 		 */
-		check_results(pceu_spill[0], pceu_spill[1], class, width,
+		check_results(pceu_spill[0], pceu_spill[1], class, width, 0,
 			      EXPECTED_LOAD_IDLE);
 		close(fd_spill);
 	}
 
-	spin_sync_end(fd, ctx);
-	spin_ctx_destroy(fd, ctx);
+	if (cork)
+		xe_cork_destroy(fd, cork);
 
 	xe_vm_destroy(fd, vm);
 }
@@ -829,8 +746,10 @@ igt_main
 		basic_memory(xe);
 
 	igt_describe("Check if basic fdinfo content is present for engine utilization");
-	igt_subtest("basic-utilization")
+	igt_subtest("basic-utilization") {
+		require_engine_utilization_data(xe);
 		basic_engine_utilization(xe);
+	}
 
 	igt_describe("Create and compare total and resident memory consumption by client");
 	igt_subtest("mem-total-resident")
@@ -844,50 +763,69 @@ igt_main
 	igt_subtest("mem-active")
 		mem_active(xe, xe_engine(xe, 0));
 
-	igt_subtest("utilization-single-idle")
+	igt_subtest("utilization-single-idle") {
+		require_engine_utilization_data(xe);
 		xe_for_each_engine(xe, hwe)
 			utilization_single(xe, hwe, 0);
+	}
 
-	igt_subtest("utilization-single-full-load")
+	igt_subtest("utilization-single-full-load") {
+		require_engine_utilization_data(xe);
 		xe_for_each_engine(xe, hwe)
 			utilization_single(xe, hwe, TEST_BUSY | TEST_TRAILING_IDLE);
+	}
 
-	igt_subtest("utilization-single-full-load-isolation")
+	igt_subtest("utilization-single-full-load-isolation") {
+		require_engine_utilization_data(xe);
 		xe_for_each_engine(xe, hwe)
 			utilization_single(xe, hwe, TEST_BUSY | TEST_TRAILING_IDLE | TEST_ISOLATION);
+	}
 
-	igt_subtest("utilization-single-full-load-destroy-queue")
+	igt_subtest("utilization-single-full-load-destroy-queue") {
+		require_engine_utilization_data(xe);
 		xe_for_each_engine(xe, hwe)
 			utilization_single_destroy_queue(xe, hwe);
+	}
 
-	igt_subtest("utilization-others-idle")
+	igt_subtest("utilization-others-idle") {
+		require_engine_utilization_data(xe);
 		xe_for_each_engine(xe, hwe)
 			utilization_others_idle(xe, hwe);
+	}
 
-	igt_subtest("utilization-others-full-load")
+	igt_subtest("utilization-others-full-load") {
+		require_engine_utilization_data(xe);
 		xe_for_each_engine(xe, hwe)
 			utilization_others_full_load(xe, hwe);
+	}
 
-	igt_subtest("utilization-all-full-load")
+	igt_subtest("utilization-all-full-load") {
+		require_engine_utilization_data(xe);
 		utilization_all_full_load(xe);
+	}
 
 
 	for (const struct section *s = sections; s->name; s++) {
-		igt_subtest_f("%s-utilization-single-idle", s->name)
+		igt_subtest_f("%s-utilization-single-idle", s->name) {
+			require_engine_utilization_data(xe);
 			xe_for_each_gt(xe, gt)
 				xe_for_each_engine_class(class)
 					utilization_multi(xe, gt, class, s->flags);
+		}
 
-		igt_subtest_f("%s-utilization-single-full-load", s->name)
+		igt_subtest_f("%s-utilization-single-full-load", s->name) {
+			require_engine_utilization_data(xe);
 			xe_for_each_gt(xe, gt)
 				xe_for_each_engine_class(class)
 					utilization_multi(xe, gt, class,
 							  s->flags |
 							  TEST_BUSY |
 							  TEST_TRAILING_IDLE);
+		}
 
 		igt_subtest_f("%s-utilization-single-full-load-isolation",
-			      s->name)
+			      s->name) {
+			require_engine_utilization_data(xe);
 			xe_for_each_gt(xe, gt)
 				xe_for_each_engine_class(class)
 					utilization_multi(xe, gt, class,
@@ -895,6 +833,7 @@ igt_main
 							  TEST_BUSY |
 							  TEST_TRAILING_IDLE |
 							  TEST_ISOLATION);
+		}
 	}
 
 	igt_fixture {
