@@ -20,7 +20,6 @@
 #include <sys/prctl.h>
 
 #include "igt.h"
-#include "igt_sysfs.h"
 #include "intel_pat.h"
 #include "lib/igt_syncobj.h"
 #include "xe/xe_eudebug.h"
@@ -54,16 +53,18 @@ static void test_sysfs_toggle(int fd)
 #define STAGE_PRE_DEBUG_RESOURCES_DONE	1
 #define STAGE_DISCOVERY_DONE		2
 
-#define CREATE_VMS		(1 << 0)
-#define CREATE_EXEC_QUEUES	(1 << 1)
-#define VM_BIND			(1 << 2)
-#define VM_BIND_VM_DESTROY	(1 << 3)
-#define VM_BIND_EXTENDED	(1 << 4)
-#define VM_METADATA		(1 << 5)
-#define VM_BIND_METADATA	(1 << 6)
-#define VM_BIND_OP_MAP_USERPTR	(1 << 7)
-#define EXEC_QUEUES_PLACEMENTS	(1 << 8)
-#define TEST_DISCOVERY		(1 << 31)
+#define CREATE_VMS			(1 << 0)
+#define CREATE_EXEC_QUEUES		(1 << 1)
+#define VM_BIND				(1 << 2)
+#define VM_BIND_VM_DESTROY		(1 << 3)
+#define VM_BIND_EXTENDED		(1 << 4)
+#define VM_METADATA			(1 << 5)
+#define VM_BIND_METADATA		(1 << 6)
+#define VM_BIND_OP_MAP_USERPTR		(1 << 7)
+#define VM_BIND_DELAY_UFENCE_ACK	(1 << 8)
+#define VM_BIND_UFENCE_RECONNECT	(1 << 9)
+#define VM_BIND_UFENCE_SIGINT_CLIENT	(1 << 10)
+#define TEST_DISCOVERY			(1 << 31)
 
 #define PAGE_SIZE SZ_4K
 #define MDATA_SIZE (WORK_IN_PROGRESS_DRM_XE_DEBUG_METADATA_NUM * PAGE_SIZE)
@@ -423,80 +424,6 @@ static void vm_bind_client(int fd, struct xe_eudebug_client *c)
 	xe_eudebug_client_vm_destroy(c, fd, vm);
 }
 
-static void placements_client(int fd, struct xe_eudebug_client *c)
-{
-	struct drm_xe_ext_set_property eq_ext = {
-		.base.name = DRM_XE_EXEC_QUEUE_EXTENSION_SET_PROPERTY,
-		.property = DRM_XE_EXEC_QUEUE_SET_PROPERTY_EUDEBUG,
-		.value = DRM_XE_EXEC_QUEUE_EUDEBUG_FLAG_ENABLE,
-	};
-	struct drm_xe_exec_queue_create create = {
-		.extensions = to_user_pointer(&eq_ext),
-	};
-	struct drm_xe_engine_class_instance *eci;
-	bool at_least_4_ccs = false;
-	uint32_t vm;
-	int i, gt;
-
-	xe_for_each_gt(fd, gt) {
-		int count = 0;
-
-		xe_for_each_engine(fd, eci)
-			if (eci->engine_class == DRM_XE_ENGINE_CLASS_COMPUTE &&
-			    eci->gt_id == gt)
-				count++;
-
-		if (count < 4)
-			continue;
-
-		at_least_4_ccs = true;
-
-		vm = xe_eudebug_client_vm_create(c, fd, DRM_XE_VM_CREATE_FLAG_LR_MODE, 0);
-
-		eci = calloc(count * count - 2, sizeof(*eci));
-
-		for (i = 0; i < count * count - 2; i++) {
-			eci[i].engine_class = DRM_XE_ENGINE_CLASS_COMPUTE;
-			eci[i].gt_id = gt;
-		}
-
-		for (i = 0; i < count; i++)
-			eci[i].engine_instance = i;
-
-		create.instances = to_user_pointer(eci);
-		create.vm_id = vm;
-		create.width = 1;
-		create.num_placements = count;
-		xe_eudebug_client_exec_queue_create(c, fd, &create);
-		xe_eudebug_client_exec_queue_destroy(c, fd, &create);
-
-		create.width = count;
-		create.num_placements = 1;
-		xe_eudebug_client_exec_queue_create(c, fd, &create);
-		xe_eudebug_client_exec_queue_destroy(c, fd, &create);
-
-		// every other instance (logical_mask ~ 1010, 0101)
-		create.width = 2;
-		create.num_placements = count/2;
-		xe_eudebug_client_exec_queue_create(c, fd, &create);
-		xe_eudebug_client_exec_queue_destroy(c, fd, &create);
-
-		// logically contigous placement (logical_mask ~ 1100, 0110, 0011)
-		for (i = 0; i < count * count - 2; i++)
-			eci[i].engine_instance = i / 2 + i % 2;
-
-		create.width = 2;
-		create.num_placements = count - 1;
-		xe_eudebug_client_exec_queue_create(c, fd, &create);
-		xe_eudebug_client_exec_queue_destroy(c, fd, &create);
-
-		xe_eudebug_client_vm_destroy(c, fd, vm);
-		free(eci);
-	}
-
-	igt_require(at_least_4_ccs);
-}
-
 static void run_basic_client(struct xe_eudebug_client *c)
 {
 	int fd, i;
@@ -547,9 +474,6 @@ static void run_basic_client(struct xe_eudebug_client *c)
 		xe_eudebug_client_vm_destroy(c, fd, vm);
 	}
 
-	if (c->flags & EXEC_QUEUES_PLACEMENTS)
-		placements_client(fd, c);
-
 	if (c->flags & VM_BIND || c->flags & VM_BIND_METADATA)
 		basic_vm_bind_client(fd, c);
 
@@ -988,11 +912,6 @@ static void test_read_event(int fd)
  *	Attach the debugger to a process that performs bind, bind array, rebind,
  *	partial unbind, unbind and unbind all operations.
  *
- * SUBTEST: exec-queue-placements
- * Description:
- *	Create compute exec_queues with various placements and compare them against
- *	event sent to the debugger.
- *
  * SUBTEST: multigpu-basic-client
  * Description:
  *	Attach the debugger to process which opens and closes xe drm client on all Xe devices.
@@ -1090,6 +1009,7 @@ static void test_basic_discovery(int fd, unsigned int flags, bool match_opposite
 #define DISCOVERY_CLOSE_CLIENT		(1 << 1)
 #define DISCOVERY_DESTROY_RESOURCES	(1 << 2)
 #define DISCOVERY_VM_BIND		(1 << 3)
+#define DISCOVERY_SIGINT		(1 << 4)
 static void run_discovery_client(struct xe_eudebug_client *c)
 {
 	int fd[RESOURCE_COUNT], i;
@@ -1162,6 +1082,7 @@ static void run_discovery_client(struct xe_eudebug_client *c)
  * arg[1]:
  *
  * @race:		resources creation
+ * @race-sigint:	resources creation with client interruption
  * @race-vmbind:	vm-bind operations
  * @empty:		resources destruction
  * @empty-clients:	client closure
@@ -1197,56 +1118,61 @@ static void *discovery_race_thread(void *data)
 
 			xe_eudebug_debugger_start_worker(s->debugger);
 
-			/*
-			 * Thread can starve for more than one second. Make
-			 * sure we get at least one event before stopping.
-			 */
-			do
-				sleep(1);
-			while (!READ_ONCE(s->debugger->event_count) &&
-			       --max_worker_waits);
+			if (!s->client->done) {
+				/*
+				 * Thread can starve for more than one second. Make
+				 * sure we get at least one event before stopping.
+				 */
+				do
+					if (s->client->flags & DISCOVERY_SIGINT)
+						usleep(100000);
+					else
+						sleep(1);
+				while (!READ_ONCE(s->debugger->event_count) &&
+				       --max_worker_waits);
 
 			igt_assert(READ_ONCE(s->debugger->event_count));
+			}
 
 			xe_eudebug_debugger_stop_worker(s->debugger, 1);
 
 			igt_debug("Resources discovered: %lu\n", s->debugger->event_count);
+			if (!s->client->done) {
+				xe_eudebug_for_each_event(e, s->debugger->log) {
+					if (e->type == DRM_XE_EUDEBUG_EVENT_OPEN) {
+						struct drm_xe_eudebug_event_client *eo = (void *)e;
 
-			xe_eudebug_for_each_event(e, s->debugger->log) {
-				if (e->type == DRM_XE_EUDEBUG_EVENT_OPEN) {
-					struct drm_xe_eudebug_event_client *eo = (void *)e;
-
-					if (i >= 0) {
-						igt_assert_eq(clients[i].vm_count,
-							      RESOURCE_COUNT);
-
-						igt_assert_eq(clients[i].exec_queue_count,
-							      RESOURCE_COUNT);
-
-						if (s->client->flags & DISCOVERY_VM_BIND)
-							igt_assert_eq(clients[i].vm_bind_op_count,
+						if (i >= 0) {
+							igt_assert_eq(clients[i].vm_count,
 								      RESOURCE_COUNT);
+
+							igt_assert_eq(clients[i].exec_queue_count,
+								      RESOURCE_COUNT);
+
+							if (s->client->flags & DISCOVERY_VM_BIND)
+								igt_assert_eq(clients[i].vm_bind_op_count,
+									      RESOURCE_COUNT);
+						}
+
+						igt_assert(++i < RESOURCE_COUNT);
+						clients[i].client_handle = eo->client_handle;
+						clients[i].vm_count = 0;
+						clients[i].exec_queue_count = 0;
+						clients[i].vm_bind_op_count = 0;
 					}
 
-					igt_assert(++i < RESOURCE_COUNT);
-					clients[i].client_handle = eo->client_handle;
-					clients[i].vm_count = 0;
-					clients[i].exec_queue_count = 0;
-					clients[i].vm_bind_op_count = 0;
-				}
+					if (e->type == DRM_XE_EUDEBUG_EVENT_VM)
+						clients[i].vm_count++;
 
-				if (e->type == DRM_XE_EUDEBUG_EVENT_VM)
-					clients[i].vm_count++;
+					if (e->type == DRM_XE_EUDEBUG_EVENT_EXEC_QUEUE)
+						clients[i].exec_queue_count++;
 
-				if (e->type == DRM_XE_EUDEBUG_EVENT_EXEC_QUEUE)
-					clients[i].exec_queue_count++;
-
-				if (e->type == DRM_XE_EUDEBUG_EVENT_VM_BIND_OP)
-					clients[i].vm_bind_op_count++;
-			};
-
-			igt_assert_lte(0, i);
+					if (e->type == DRM_XE_EUDEBUG_EVENT_VM_BIND_OP)
+						clients[i].vm_bind_op_count++;
+				};
 
+				igt_assert_lte(0, i);
+			}
 			for (int j = 0; j < i; j++)
 				for (int k = 0; k < i; k++) {
 					if (k == j)
@@ -1314,8 +1240,15 @@ static void test_race_discovery(int fd, unsigned int flags, int clients)
 		pthread_create(&threads[i], NULL, discovery_race_thread, &sessions[i]);
 	}
 
-	for (i = 0; i < count; i++)
+	if (flags & DISCOVERY_SIGINT) {
+		sleep(2);
+		sessions[0].client->done = 1;
+		kill(sessions[0].client->pid, SIGINT);
+	}
+
+	for (i = 0; i < count; i++) {
 		pthread_join(threads[i], NULL);
+	}
 
 	for (i = count - 1; i > 0; i--) {
 		if (sessions[i].flags & PRIMARY_THREAD) {
@@ -1553,6 +1486,7 @@ static void debugger_test_vma(struct xe_eudebug_debugger *d,
 	struct drm_xe_eudebug_vm_open vo = { 0, };
 	uint64_t *v1, *v2;
 	uint64_t items = va_length / sizeof(uint64_t);
+	uint64_t offset;
 	int fd;
 	int r, i;
 
@@ -1570,14 +1504,24 @@ static void debugger_test_vma(struct xe_eudebug_debugger *d,
 	r = pread(fd, v1, va_length, va_start);
 	igt_assert_eq(r, va_length);
 
+	for (i = 0; i < items; i++)
+		igt_assert_eq(v1[i], va_start + i);
+
+	/* random unaligned offset within the vm */
+	offset = 1 + random() % (va_length / 2);
+	r = pread(fd, (char *)v1 + offset, va_length - offset, va_start + offset);
+	igt_assert_eq(r, va_length - offset);
 	for (i = 0; i < items; i++)
 		igt_assert_eq(v1[i], va_start + i);
 
 	for (i = 0; i < items; i++)
 		v1[i] = va_start + i + 1;
 
-	r = pwrite(fd, v1, va_length, va_start);
-	igt_assert_eq(r, va_length);
+	r = pwrite(fd, v1, offset, va_start);
+	igt_assert_eq(r, offset);
+
+	r = pwrite(fd, (char *)v1 + offset, va_length - offset, va_start + offset);
+	igt_assert_eq(r, va_length - offset);
 
 	lseek(fd, va_start, SEEK_SET);
 	r = read(fd, v2, va_length);
@@ -1770,7 +1714,13 @@ static void vm_trigger_access_parameters(struct xe_eudebug_debugger *d,
 /**
  * SUBTEST: basic-vm-access-parameters
  * Description:
- *      Check negative scenarios of VM_OPEN ioctl and pread/pwrite usage.
+ *      Check negative scenarios of VM_OPEN ioctl and pread/pwrite usage
+ *      with bo backing storage.
+ *
+ * SUBTEST: basic-vm-access-parameters-userptr
+ * Description:
+ *      Check negative scenarios of VM_OPEN ioctl and pread/pwrite usage
+ *      with userptr backing storage.
  */
 static void test_vm_access_parameters(int fd, unsigned int flags, int num_clients)
 {
@@ -2125,6 +2075,19 @@ static int wait_for_ufence_events(struct ufence_priv *priv, int timeout_ms)
  * SUBTEST: basic-vm-bind-ufence
  * Description:
  *      Give user fence in application and check if ufence ack works
+ *
+ * SUBTEST: basic-vm-bind-ufence-delay-ack
+ * Description:
+ *	Give user fence in application and check if delayed ufence ack works
+ *
+ * SUBTEST: basic-vm-bind-ufence-reconnect
+ * Description:
+ *	Give user fence in application, hold it, drop the debugger connection and check if anything
+ *	breaks. Expect that held acks are released when connection is dropped.
+ *
+ * SUBTEST: basic-vm-bind-ufence-sigint-client
+ * Description:
+ *	Give user fence in application, hold it, send SIGINT to client and check if anything breaks.
  */
 static void test_basic_ufence(int fd, unsigned int flags)
 {
@@ -2132,6 +2095,7 @@ static void test_basic_ufence(int fd, unsigned int flags)
 	struct xe_eudebug_session *s;
 	struct xe_eudebug_client *c;
 	struct ufence_priv *priv;
+	uint32_t filter = XE_EUDEBUG_FILTER_EVENT_VM_BIND_UFENCE;
 
 	priv = ufence_priv_create();
 	s = xe_eudebug_session_create(fd, basic_ufence_client, flags, priv);
@@ -2149,7 +2113,24 @@ static void test_basic_ufence(int fd, unsigned int flags)
 	xe_eudebug_debugger_wait_stage(s, STAGE_CLIENT_WAIT_ON_UFENCE_DONE);
 	xe_eudebug_assert_f(d, wait_for_ufence_events(priv, XE_EUDEBUG_DEFAULT_TIMEOUT_SEC * MSEC_PER_SEC) == 0,
 			    "missing ufence events\n");
-	ack_fences(d);
+
+	if (flags & VM_BIND_DELAY_UFENCE_ACK)
+		sleep(XE_EUDEBUG_DEFAULT_TIMEOUT_SEC * 4 / 5);
+
+	if (flags & VM_BIND_UFENCE_SIGINT_CLIENT) {
+		filter = XE_EUDEBUG_FILTER_ALL;
+		kill(c->pid, SIGINT);
+		c->pid = 0;
+		c->done = 1;
+	} else if (flags & VM_BIND_UFENCE_RECONNECT) {
+		filter = XE_EUDEBUG_FILTER_EVENT_VM_BIND | XE_EUDEBUG_FILTER_EVENT_VM |
+				XE_EUDEBUG_FILTER_EVENT_OPEN;
+		xe_eudebug_debugger_detach(d);
+		xe_eudebug_client_wait_done(c);
+		igt_assert_eq(xe_eudebug_debugger_attach(d, c), 0);
+	} else {
+		ack_fences(d);
+	}
 
 	xe_eudebug_client_wait_done(c);
 	xe_eudebug_debugger_stop_worker(d, 1);
@@ -2157,7 +2138,7 @@ static void test_basic_ufence(int fd, unsigned int flags)
 	xe_eudebug_event_log_print(d->log, true);
 	xe_eudebug_event_log_print(c->log, true);
 
-	xe_eudebug_session_check(s, true, XE_EUDEBUG_FILTER_EVENT_VM_BIND_UFENCE);
+	xe_eudebug_session_check(s, true, filter);
 
 	xe_eudebug_session_destroy(s);
 	ufence_priv_destroy(priv);
@@ -2729,40 +2710,14 @@ static void test_basic_exec_queues_enable(int fd)
 	xe_vm_destroy(fd, vm_non_lr);
 }
 
-static void ccs_mode_all_engines(int num_gt)
-{
-	int fd, gt, gt_fd, num_slices, ccs_mode;
-	int num_gts_with_ccs_mode = 0;
-
-	for (gt = 0; gt < num_gt; gt++) {
-		fd = drm_open_driver(DRIVER_XE);
-		gt_fd = xe_sysfs_gt_open(fd, gt);
-		close(fd);
-
-		if (igt_sysfs_scanf(gt_fd, "num_cslices", "%u", &num_slices) <= 0)
-			continue;
-
-		num_gts_with_ccs_mode++;
-
-		igt_assert(igt_sysfs_printf(gt_fd, "ccs_mode", "%u", num_slices) > 0);
-		igt_assert(igt_sysfs_scanf(gt_fd, "ccs_mode", "%u", &ccs_mode) > 0);
-		igt_assert(num_slices == ccs_mode);
-		close(gt_fd);
-	}
-
-	errno = 0;
-	igt_require(num_gts_with_ccs_mode > 0);
-}
-
 igt_main
 {
 	bool was_enabled;
 	bool *multigpu_was_enabled;
-	int fd, gpu_count, gt_count;
+	int fd, gpu_count;
 
 	igt_fixture {
 		fd = drm_open_driver(DRIVER_XE);
-		gt_count = xe_number_gt(fd);
 		was_enabled = xe_eudebug_enable(fd, true);
 	}
 
@@ -2796,6 +2751,9 @@ igt_main
 	igt_subtest("basic-vm-access-parameters")
 		test_vm_access_parameters(fd, 0, 1);
 
+	igt_subtest("basic-vm-access-parameters-userptr")
+		test_vm_access_parameters(fd, VM_BIND_OP_MAP_USERPTR, 1);
+
 	igt_subtest("multiple-sessions")
 		test_basic_sessions(fd, CREATE_VMS | CREATE_EXEC_QUEUES, 4, true);
 
@@ -2814,6 +2772,15 @@ igt_main
 	igt_subtest("basic-vm-bind-ufence")
 		test_basic_ufence(fd, 0);
 
+	igt_subtest("basic-vm-bind-ufence-delay-ack")
+		test_basic_ufence(fd, VM_BIND_DELAY_UFENCE_ACK);
+
+	igt_subtest("basic-vm-bind-ufence-reconnect")
+		test_basic_ufence(fd, VM_BIND_UFENCE_RECONNECT);
+
+	igt_subtest("basic-vm-bind-ufence-sigint-client")
+		test_basic_ufence(fd, VM_BIND_UFENCE_SIGINT_CLIENT);
+
 	igt_subtest("vma-ufence")
 		test_vma_ufence(fd, 0);
 
@@ -2850,23 +2817,15 @@ igt_main
 	igt_subtest("discovery-race-vmbind")
 		test_race_discovery(fd, DISCOVERY_VM_BIND, 4);
 
+	igt_subtest("discovery-race-sigint")
+		test_race_discovery(fd, DISCOVERY_SIGINT, 1);
+
 	igt_subtest("discovery-empty")
 		test_empty_discovery(fd, DISCOVERY_CLOSE_CLIENT, 16);
 
 	igt_subtest("discovery-empty-clients")
 		test_empty_discovery(fd, DISCOVERY_DESTROY_RESOURCES, 16);
 
-	igt_subtest_group {
-		igt_fixture {
-			drm_close_driver(fd);
-			ccs_mode_all_engines(gt_count);
-			fd = drm_open_driver(DRIVER_XE);
-		}
-
-		igt_subtest("exec-queue-placements")
-			test_basic_sessions(fd, EXEC_QUEUES_PLACEMENTS, 1, true);
-	}
-
 	igt_fixture {
 		xe_eudebug_enable(fd, was_enabled);
 		drm_close_driver(fd);
