@@ -158,6 +158,9 @@ static struct gpgpu_shader *get_shader(int fd, const unsigned int flags)
 
 	shader = gpgpu_shader_create(fd);
 
+	if (shader->gen_ver == 3000)
+		gpgpu_shader_set_vrt(shader, VRT_96);
+
 	gpgpu_shader__write_dword(shader, SHADER_CANARY, 0);
 	if (flags & SHADER_BREAKPOINT) {
 		gpgpu_shader__nop(shader);
@@ -569,7 +572,7 @@ static void get_aips_offset_table(struct online_debug_data *data, int threads)
 
 	igt_debug("AIPs offset table:\n");
 	for (int i = 0; i < threads; i++)
-		igt_debug("%lx\n", data->aips_offset_table[i]);
+		igt_debug("%" PRIx64 "\n", data->aips_offset_table[i]);
 }
 
 static int get_stepped_threads_count(struct online_debug_data *data, int threads)
@@ -694,6 +697,9 @@ static void eu_attention_resume_single_step_trigger(struct xe_eudebug_debugger *
 
 	get_aips_offset_table(data, threads);
 
+	if (data->last_eu_control_seqno > att->base.seqno)
+		return;
+
 	if (d->flags & TRIGGER_RESUME_PARALLEL_WALK) {
 		if (data->stepped_threads_count != -1)
 			if (data->steps_done < SINGLE_STEP_COUNT) {
@@ -766,9 +772,9 @@ static void eu_attention_resume_single_step_trigger(struct xe_eudebug_debugger *
 			     data->target_offset + steering_offset(threads)), sz);
 	fsync(data->vm_fd);
 
-	eu_ctl_resume(d->master_fd, d->fd, att->client_handle,
-		      att->exec_queue_handle, att->lrc_handle,
-		      att->bitmask, att->bitmask_size);
+	data->last_eu_control_seqno = eu_ctl_resume(d->master_fd, d->fd, att->client_handle,
+						    att->exec_queue_handle, att->lrc_handle,
+						    att->bitmask, att->bitmask_size);
 
 	if (data->single_step_bitmask)
 		for (int i = 0; i < att->bitmask_size; i++)
@@ -1063,8 +1069,8 @@ static void run_online_client(struct xe_eudebug_client *c)
 	uint32_t *ptr;
 	int fd, vm_flags;
 
-	metadata[0] = calloc(2, sizeof(*metadata));
-	metadata[1] = calloc(2, sizeof(*metadata));
+	metadata[0] = calloc(2, sizeof(**metadata));
+	metadata[1] = calloc(2, sizeof(**metadata));
 	igt_assert(metadata[0]);
 	igt_assert(metadata[1]);
 
@@ -1086,10 +1092,10 @@ static void run_online_client(struct xe_eudebug_client *c)
 	metadata[1][0] = target_offset;
 	metadata[1][1] = buf->size;
 	metadata_id[0] = xe_eudebug_client_metadata_create(c, fd, DRM_XE_DEBUG_METADATA_ELF_BINARY,
-							   2 * sizeof(*metadata), metadata[0]);
+							   2 * sizeof(**metadata), metadata[0]);
 	metadata_id[1] = xe_eudebug_client_metadata_create(c, fd,
 							   DRM_XE_DEBUG_METADATA_PROGRAM_MODULE,
-							   2 * sizeof(*metadata), metadata[1]);
+							   2 * sizeof(**metadata), metadata[1]);
 
 	vm_flags = DRM_XE_VM_CREATE_FLAG_LR_MODE;
 	vm_flags |= c->flags & SHADER_PAGEFAULT ? DRM_XE_VM_CREATE_FLAG_FAULT_MODE : 0;
@@ -1144,10 +1150,10 @@ static void run_online_client(struct xe_eudebug_client *c)
 	xe_eudebug_client_vm_destroy(c, fd, create.vm_id);
 
 	xe_eudebug_client_metadata_destroy(c, fd, metadata_id[0], DRM_XE_DEBUG_METADATA_ELF_BINARY,
-					   2 * sizeof(*metadata));
+					   2 * sizeof(**metadata));
 	xe_eudebug_client_metadata_destroy(c, fd, metadata_id[1],
 					   DRM_XE_DEBUG_METADATA_PROGRAM_MODULE,
-					   2 * sizeof(*metadata));
+					   2 * sizeof(**metadata));
 
 	intel_buf_destroy(buf);
 
@@ -1172,7 +1178,7 @@ static int query_attention_bitmask_size(int fd, int gt)
 {
 	uint32_t thread_count_len;
 	uint32_t *thread_count_ptr;
-	uint32_t thread_count;
+	uint32_t threads_per_eu = 8;
 	struct drm_xe_query_topology_mask *c_dss = NULL, *g_dss = NULL, *eu_per_dss = NULL;
 	struct drm_xe_query_topology_mask *topology;
 	struct drm_xe_device_query query = {
@@ -1181,14 +1187,16 @@ static int query_attention_bitmask_size(int fd, int gt)
 		.size = 0,
 		.data = 0,
 	};
-	int pos = 0, eus;
-	uint8_t *any_dss;
+	uint8_t dss_mask, last_dss;
+	int pos = 0;
+	int i, last_dss_idx;
 
 	thread_count_ptr = xe_hwconfig_lookup_value(fd, INTEL_HWCONFIG_NUM_THREADS_PER_EU,
 						    &thread_count_len);
-	igt_assert(thread_count_ptr);
-	igt_assert(thread_count_len == 1);
-	thread_count = *thread_count_ptr;
+	if (thread_count_ptr) {
+		igt_assert(thread_count_len == 1);
+		threads_per_eu = *thread_count_ptr;
+	}
 
 	igt_assert_eq(igt_ioctl(fd, DRM_IOCTL_XE_DEVICE_QUERY, &query), 0);
 	igt_assert_neq(query.size, 0);
@@ -1224,22 +1232,27 @@ static int query_attention_bitmask_size(int fd, int gt)
 	igt_assert(g_dss && c_dss && eu_per_dss);
 	igt_assert_eq_u32(c_dss->num_bytes, g_dss->num_bytes);
 
-	any_dss = malloc(c_dss->num_bytes);
-	igt_assert(any_dss);
+	for (i = 0; i < c_dss->num_bytes; i++) {
+		dss_mask = c_dss->mask[i] | g_dss->mask[i];
+		if (dss_mask) {
+			last_dss = dss_mask;
+			last_dss_idx = i;
+		}
+	}
 
-	for (int i = 0; i < c_dss->num_bytes; i++)
-		any_dss[i] = c_dss->mask[i] | g_dss->mask[i];
+	last_dss_idx *= BITS_PER_BYTE;
+	do {
+		last_dss_idx++;
+	} while (last_dss >>= 1);
 
-	eus = count_set_bits(any_dss, c_dss->num_bytes);
-	eus *= count_set_bits(eu_per_dss->mask, eu_per_dss->num_bytes);
+	last_dss_idx *= count_set_bits(eu_per_dss->mask, eu_per_dss->num_bytes);
 
 	if (intel_gen_has_lockstep_eus(fd))
-		eus /= 2;
+		last_dss_idx /= 2;
 
-	free(any_dss);
 	free(topology);
 
-	return eus * DIV_ROUND_UP(thread_count, 8);
+	return last_dss_idx * DIV_ROUND_UP(threads_per_eu, 8);
 }
 
 static struct drm_xe_eudebug_event_exec_queue *
@@ -1496,6 +1509,98 @@ static void test_set_breakpoint_online(int fd, struct drm_xe_engine_class_instan
 	online_debug_data_destroy(data);
 }
 
+/**
+ * SUBTEST: set-breakpoint-sigint-debugger
+ * Description:
+ *	A variant of set-breakpoint that sends SIGINT to the debugger thread with random timing
+ *	and checks if nothing breaks, exercising the scenario multiple times.
+ */
+static void test_set_breakpoint_online_sigint_debugger(int fd,
+						       struct drm_xe_engine_class_instance *hwe,
+						       int flags)
+{
+	struct xe_eudebug_session *s;
+	struct online_debug_data *data;
+	struct timespec ts = { };
+	int loop_count = 0;
+	uint64_t sleep_time;
+	uint64_t set_breakpoint_time;
+	uint64_t max_sleep_time;
+	uint64_t events_max = 0;
+	int sigints_during_test = 0;
+
+	/*
+	 * Measure the average time required for basic set-breakpoint variant,
+	 * so sleep_time range is correct.
+	 */
+	igt_nsec_elapsed(&ts);
+	for (int i = 0; i < 10; i++)
+		test_set_breakpoint_online(fd, hwe, SHADER_NOP | TRIGGER_UFENCE_SET_BREAKPOINT);
+	set_breakpoint_time = igt_nsec_elapsed(&ts) / (NSEC_PER_MSEC / USEC_PER_MSEC) / 10;
+	igt_info("Average set-breakpoint execution time: %" PRIu64 " us\n", set_breakpoint_time);
+	max_sleep_time = set_breakpoint_time * 11 / 10;
+	igt_info("Maximum sleep_time: %" PRIu64 " us\n", max_sleep_time);
+
+	ts = (struct timespec) { };
+	igt_nsec_elapsed(&ts);
+
+	while (igt_seconds_elapsed(&ts) < 60) {
+		uint64_t event_count;
+
+		sleep_time = rand() % max_sleep_time;
+		igt_debug("Loop %d: SIGINT after %" PRIu64 " us\n", loop_count, sleep_time);
+
+		data = online_debug_data_create(hwe);
+		s = xe_eudebug_session_create(fd, run_online_client, flags, data);
+		xe_eudebug_debugger_add_trigger(s->debugger, DRM_XE_EUDEBUG_EVENT_OPEN,
+						open_trigger);
+		xe_eudebug_debugger_add_trigger(s->debugger, DRM_XE_EUDEBUG_EVENT_EXEC_QUEUE,
+						exec_queue_trigger);
+		xe_eudebug_debugger_add_trigger(s->debugger, DRM_XE_EUDEBUG_EVENT_VM,
+						vm_open_trigger);
+		xe_eudebug_debugger_add_trigger(s->debugger, DRM_XE_EUDEBUG_EVENT_METADATA,
+						create_metadata_trigger);
+		xe_eudebug_debugger_add_trigger(s->debugger, DRM_XE_EUDEBUG_EVENT_VM_BIND_UFENCE,
+						ufence_ack_set_bp_trigger);
+		xe_eudebug_debugger_add_trigger(s->debugger, DRM_XE_EUDEBUG_EVENT_EU_ATTENTION,
+						eu_attention_resume_trigger);
+
+		igt_assert_eq(xe_eudebug_debugger_attach(s->debugger, s->client), 0);
+		xe_eudebug_debugger_start_worker(s->debugger);
+		igt_assert_eq(READ_ONCE(s->debugger->event_count), 0);
+		xe_eudebug_client_start(s->client);
+
+		/* Sample max events without SIGINT */
+		if (!loop_count++)
+			xe_eudebug_client_wait_done(s->client);
+		else
+			usleep(sleep_time);
+
+		event_count = READ_ONCE(s->debugger->event_count);
+		if (event_count > events_max)
+			events_max = event_count;
+		else if (event_count > 0 && event_count < events_max)
+			sigints_during_test++;
+
+		igt_assert_eq(pthread_kill(s->debugger->worker_thread, SIGINT), 0);
+		close(s->debugger->fd);
+
+		igt_assert_eq(READ_ONCE(s->debugger->worker_state), DEBUGGER_WORKER_ACTIVE);
+		WRITE_ONCE(s->debugger->worker_state, DEBUGGER_WORKER_INACTIVE);
+
+		xe_eudebug_client_wait_done(s->client);
+
+		xe_eudebug_event_log_print(s->debugger->log, true);
+		xe_eudebug_event_log_print(s->client->log, true);
+
+		xe_eudebug_session_destroy(s);
+		online_debug_data_destroy(data);
+	}
+
+	igt_info("%d correctly timed SIGINTs in %d loops\n", sigints_during_test, loop_count);
+	igt_assert_lt(0, sigints_during_test);
+}
+
 /**
  * SUBTEST: pagefault-read
  * Description:
@@ -2234,6 +2339,11 @@ static int find_suitable_engines(struct drm_xe_engine_class_instance **hwes,
 	return engine_count;
 }
 
+static uint64_t timespecs_diff_us(struct timespec *ts1, struct timespec *ts2)
+{
+	return (uint64_t)(fabs(igt_time_elapsed(ts1, ts2)) * USEC_PER_SEC);
+}
+
 /**
  * SUBTEST: breakpoint-many-sessions-single-tile
  * Description:
@@ -2254,7 +2364,7 @@ static void test_many_sessions_on_tiles(int fd, bool multi_tile)
 	struct online_debug_data **data;
 	struct drm_xe_engine_class_instance **hwe;
 	struct drm_xe_eudebug_event_eu_attention *eus;
-	uint64_t current_t, next_t, diff;
+	uint64_t diff;
 	int attempt_mask = 0, final_mask, should_break;
 	int i;
 
@@ -2299,6 +2409,7 @@ static void test_many_sessions_on_tiles(int fd, bool multi_tile)
 				attempt_mask |= BIT(i);
 				should_break = 1;
 
+				usleep(WORKLOAD_DELAY_US);
 				eus = (struct drm_xe_eudebug_event_eu_attention *)data[i]->exception_event;
 				eu_ctl_resume(s[i]->debugger->master_fd, s[i]->debugger->fd,
 					      eus->client_handle, eus->exec_queue_handle,
@@ -2317,18 +2428,16 @@ static void test_many_sessions_on_tiles(int fd, bool multi_tile)
 	igt_assert_eq(attempt_mask, final_mask);
 
 	for (i = 0; i < n - 1; i++) {
-		/* Convert timestamps to microseconds */
-		current_t = data[i]->exception_arrived.tv_nsec * 1000;
-		next_t = data[i + 1]->exception_arrived.tv_nsec * 1000;
-		diff = current_t < next_t ? next_t - current_t : current_t - next_t;
+		diff = timespecs_diff_us(&data[i]->exception_arrived,
+					 &data[i + 1]->exception_arrived);
 
 		if (multi_tile)
 			igt_assert_f(diff < WORKLOAD_DELAY_US,
-				     "Expected to execute workloads concurrently. Actual delay: %lu ms\n",
+				     "Expected to execute workloads concurrently. Actual delay: %" PRIu64 " us\n",
 				     diff);
 		else
 			igt_assert_f(diff >= WORKLOAD_DELAY_US,
-				     "Expected a serialization of workloads. Actual delay: %lu ms\n",
+				     "Expected a serialization of workloads. Actual delay: %" PRIu64 " us\n",
 				     diff);
 	}
 
@@ -2412,6 +2521,10 @@ igt_main
 	test_gt_render_or_compute("set-breakpoint", fd, hwe)
 		test_set_breakpoint_online(fd, hwe, SHADER_NOP | TRIGGER_UFENCE_SET_BREAKPOINT);
 
+	test_gt_render_or_compute("set-breakpoint-sigint-debugger", fd, hwe)
+		test_set_breakpoint_online_sigint_debugger(fd, hwe,
+							   SHADER_NOP | TRIGGER_UFENCE_SET_BREAKPOINT);
+
 	test_gt_render_or_compute("breakpoint-not-in-debug-mode", fd, hwe)
 		test_basic_online(fd, hwe, SHADER_BREAKPOINT | DISABLE_DEBUG_MODE);
 
