@@ -32,9 +32,7 @@
  * Category: Display
  * Description: Test atomic modesetting API
  * Driver requirement: i915, xe
- * Functionality: kms_core, plane
  * Mega feature: General Display Features
- * Test category: functionality test
  */
 
 #include <unistd.h>
@@ -57,7 +55,6 @@
  * Description: Test abuse the atomic ioctl directly in order to test various
  *              invalid conditions which the libdrm wrapper won't allow us to
  *              create.
- * Functionality: kms_core
  *
  * SUBTEST: atomic-plane-damage
  * Description: Simple test cases to use FB_DAMAGE_CLIPS plane property
@@ -65,7 +62,6 @@
  * SUBTEST: test-only
  * Description: Test to ensure that DRM_MODE_ATOMIC_TEST_ONLY really only touches
  *              the free-standing state objects and nothing else.
- * Functionality: kms_core
  *
  * SUBTEST: plane-primary-overlay-mutable-zpos
  * Description: Verify that the overlay plane can cover the primary one (and
@@ -506,6 +502,10 @@ plane_immutable_zpos(data_t *data, igt_output_t *output, enum pipe pipe, int n_p
 					  DRM_FORMAT_XRGB8888,
 					  DRM_FORMAT_MOD_LINEAR,
 					  0.0, 0.0, 1.0, &fb_lower);
+	if (!fb_id_lower) {
+		igt_pipe_crc_stop(pipe_crc);
+		igt_pipe_crc_free(pipe_crc);
+	}
 	igt_assert(fb_id_lower);
 
 	fb_id_upper = igt_create_color_fb(data->drm_fd,
@@ -513,6 +513,10 @@ plane_immutable_zpos(data_t *data, igt_output_t *output, enum pipe pipe, int n_p
 					  DRM_FORMAT_XRGB8888,
 					  DRM_FORMAT_MOD_LINEAR,
 					  1.0, 1.0, 0.0, &fb_upper);
+	if (!fb_id_upper) {
+		igt_pipe_crc_stop(pipe_crc);
+		igt_pipe_crc_free(pipe_crc);
+	}
 	igt_assert(fb_id_upper);
 
 	/*
@@ -560,6 +564,8 @@ plane_immutable_zpos(data_t *data, igt_output_t *output, enum pipe pipe, int n_p
 		igt_plane_set_fb(plane_upper, NULL);
 	}
 
+	igt_pipe_crc_stop(pipe_crc);
+	igt_pipe_crc_free(pipe_crc);
 	igt_remove_fb(data->drm_fd, &fb_ref);
 	igt_remove_fb(data->drm_fd, &fb_lower);
 	igt_remove_fb(data->drm_fd, &fb_upper);
