@@ -21,41 +21,37 @@
  * IN THE SOFTWARE.
  */
 
+/**
+ * TEST: kms prime
+ * Category: Display
+ * Description: Prime tests, focusing on KMS side
+ * Driver requirement: i915, xe
+ * Functionality: hybrid, kms_gem_interop
+ * Mega feature: General Display Features
+ * Test category: functionality test
+ */
+
 #include "igt.h"
 #include "igt_device.h"
 #include "igt_debugfs.h"
 #include "igt_sysfs.h"
 #include <fcntl.h>
 
+#include <poll.h>
 #include <sys/ioctl.h>
-#include <sys/poll.h>
 #include <time.h>
+
 /**
- * TEST: kms prime
- * Category: Display
- * Description: Prime tests, focusing on KMS side
- *
  * SUBTEST: D3hot
  * Description: Validate pci state of dGPU when dGPU is idle and  scanout is on iGPU
- * Driver requirement: i915, xe
- * Functionality: hybrid, kms_gem_interop
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * SUBTEST: basic-modeset-hybrid
  * Description: Basic modeset on the one device when the other device is active
- * Driver requirement: i915, xe
- * Functionality: hybrid, kms_gem_interop
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * SUBTEST: basic-crc-%s
  * Description: Make a dumb color buffer, export to another device and compare
  *              the CRCs with a buffer native to that device
- * Driver requirement: i915, xe
  * Functionality: crc, hybrid, kms_gem_interop
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * arg[1]:
  *
@@ -124,7 +120,7 @@ static igt_output_t *setup_display(int importer_fd, igt_display_t *display,
 		igt_display_reset(display);
 
 		igt_output_set_pipe(output, *pipe);
-		if (i915_pipe_output_combo_valid(display)) {
+		if (intel_pipe_output_combo_valid(display)) {
 			found = true;
 			break;
 		}
@@ -153,8 +149,16 @@ static void prepare_scratch(int exporter_fd, struct dumb_bo *scratch,
 		ptr = kmstest_dumb_map_buffer(exporter_fd, scratch->handle,
 					      scratch->size, PROT_WRITE);
 	} else {
-		igt_calc_fb_size(exporter_fd, mode->hdisplay, mode->vdisplay, DRM_FORMAT_XRGB8888,
-				 DRM_FORMAT_MOD_LINEAR, &scratch->size, &scratch->pitch);
+		struct igt_fb fb;
+
+		igt_init_fb(&fb, exporter_fd, mode->hdisplay, mode->vdisplay,
+			    DRM_FORMAT_XRGB8888, DRM_FORMAT_MOD_LINEAR,
+			    IGT_COLOR_YCBCR_BT709, IGT_COLOR_YCBCR_LIMITED_RANGE);
+		igt_calc_fb_size(&fb);
+
+		scratch->size = fb.size;
+		scratch->pitch = fb.strides[0];
+
 		if (gem_has_lmem(exporter_fd))
 			scratch->handle = gem_create_in_memory_regions(exporter_fd, scratch->size,
 								       REGION_LMEM(0), REGION_SMEM);
@@ -223,7 +227,7 @@ static void import_fb(int importer_fd, struct igt_fb *fb,
 			    DRM_FORMAT_XRGB8888,
 			    handles, pitches, offsets,
 			    &fb->fb_id, 0);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 }
 
 static void set_fb(struct igt_fb *fb,
@@ -239,7 +243,7 @@ static void set_fb(struct igt_fb *fb,
 	igt_plane_set_fb(primary, fb);
 	ret = igt_display_commit(display);
 
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 }
 
 static void collect_crc_for_fb(int importer_fd, struct igt_fb *fb, igt_display_t *display,
@@ -372,7 +376,7 @@ static void kms_poll_state_restore(void)
 {
 	int sysfs_fd;
 
-	igt_assert((sysfs_fd = open(KMS_HELPER, O_RDONLY)) >= 0);
+	igt_assert_lte(0, (sysfs_fd = open(KMS_HELPER, O_RDONLY)));
 	__igt_sysfs_set_boolean(sysfs_fd, "poll", kms_poll_saved_state);
 	close(sysfs_fd);
 
