@@ -27,107 +27,71 @@
 #include "igt_debugfs.h"
 #include "igt_sysfs.h"
 
+#include <poll.h>
 #include <sys/mman.h>
-#include <sys/poll.h>
 #include <sys/stat.h>
 #include <dirent.h>
 /**
  * TEST: vgem basic
- * Category: Infrastructure
  * Description: Basic sanity check of Virtual GEM module (vGEM).
+ * Category: Core
+ * Mega feature: General Core features
+ * Sub-category: DRM
  * Functionality: mock device
  * Test category: GEM_Legacy
+ * Feature: vgem
  *
  * SUBTEST: bad-fence
  * Description: Make sure a non-existent fence cannot be signaled.
- * Feature: vgem
- * Run type: FULL
  *
  * SUBTEST: bad-flag
  * Description: Make sure a fence cannot be attached and signaled with invalid flags.
- * Feature: vgem
- * Run type: FULL
  *
  * SUBTEST: bad-handle
  * Description: Make sure a fence cannot be attached to a invalid handle.
- * Feature: vgem
- * Run type: FULL
  *
  * SUBTEST: bad-pad
  * Description: Make sure a non-zero pad is rejected.
- * Feature: vgem
- * Run type: FULL
  *
  * SUBTEST: busy-fence
  * Description: Make sure a conflicting fence cannot be attached.
- * Feature: vgem
- * Run type: FULL
  *
  * SUBTEST: create
  * Description: Check the basic working of vgem_create ioctl.
- * Feature: vgem
- * Run type: BAT
- * Sub-category: DRM
  *
  * SUBTEST: debugfs
  * Description: Check the basic access to debugfs and also try to read entries in the directory.
- * Feature: vgem
- * Run type: BAT
- * Sub-category: DRM
  *
  * SUBTEST: dmabuf-export
  * Description: Check whether it can export/import the vgem handle using prime.
  * Feature: prime, vgem
- * Run type: BAT
- * Sub-category: DRM
  *
  * SUBTEST: dmabuf-fence
  * Description: Check the working of dma-buf fence interop.
  * Feature: prime, vgem
- * Run type: BAT
- * Sub-category: DRM
  *
  * SUBTEST: dmabuf-fence-before
  * Description: Attach a fence before exporting a vgem handle and check the working of fence.
  * Feature: prime, vgem
- * Run type: BAT
- * Sub-category: DRM
  *
  * SUBTEST: dmabuf-mmap
  * Description: Export the vgem handle along with RDWR capabilities using prime and check if it can be mmaped.
  * Feature: prime, vgem
- * Run type: BAT
- * Sub-category: DRM
  *
  * SUBTEST: mmap
  * Description: Create a vgem handle and check if it can be mmaped.
- * Feature: vgem
- * Run type: BAT
- * Sub-category: DRM
  *
  * SUBTEST: second-client
  * Description: Check whether it can open multiple clients.
- * Feature: vgem
- * Run type: BAT
- * Sub-category: DRM
  *
  * SUBTEST: setversion
  * Description: Check the working of SET_VERSION ioctl.
- * Feature: vgem
- * Run type: BAT
- * Sub-category: DRM
  *
  * SUBTEST: sysfs
  * Description: Check the basic access to sysfs and also try to read entries in the directory.
- * Feature: vgem
- * Run type: BAT
- * Sub-category: DRM
  *
  * SUBTEST: unload
  * Description: Basic test for handling of module unload.
- * Feature: vgem
- * Run type: BAT
- * Sub-category: DRM
  */
 
 IGT_TEST_DESCRIPTION("Basic sanity check of Virtual GEM module (vGEM).");
@@ -438,7 +402,7 @@ static void test_debugfs_read(int fd)
 
 static int module_unload(void)
 {
-	return igt_kmod_unload("vgem", 0);
+	return igt_kmod_unload("vgem");
 }
 
 static void test_unload(void)
